#include "NVMeCommand.hpp"

NVMeCommand::NVMeCommand(COMMAND_TYPE CommandType, string strHelpContent) :
        CommandBaseClass(CommandType, strHelpContent)
{
    //Base on command type to add different operations
    switch(CommandType)
    {
        case CMD_LIST:
            addOperation(new NVMeOperation(NO_OPRATION, "Display NVMe Controller Information."));

            break;
        case CMD_SHOW:
            addOperation(new NVMeOperation(OPERATION_ALL, "Display All Information."));
            addOperation(new NVMeOperation(OPERATION_HW, "Display NVME Hardware Info."));
            addOperation(new NVMeOperation(OPERATION_FW, "Display Firmware Version."));
            addOperation(new NVMeOperation(OPERATION_MODEL, "Display NVME Model"));
            addOperation(new NVMeOperation(OPERATION_SN, "Display NVME Serial Number."));
            addOperation(new NVMeOperation(OPERATION_CAPACITY, "Display NVME Capacity (GB)."));
            addOperation(new NVMeOperation(OPERATION_TEMP, "Display NVME Temperature."));
            addOperation(new NVMeOperation(OPERATION_BDF, "Display Controller Bus:Device.Function."));
            addOperation(new NVMeOperation(OPERATION_NODE, "Display NVME Node Name on OS."));
            addOperation(new NVMeOperation(OPERATION_BLOCK, "Display NVME Block size"));

            break;
        case CMD_ERROR:
            addOperation(new NVMeOperation(OPERATION_ALL_ERROR, "Get All NVME Error statistics."));

            break;
        case CMD_UTILITY:
            addOperation(new NVMeOperation(OPERATION_RESET_CONTROLLER, "Reset NVME Controller."));
            addOperation(new NVMeOperation(OPERATION_IDENTITY_CONTROLLER, "Send NVME Identify Controller, display structure."));
            addOperation(new NVMeOperation(OPERATION_IDENTITY_NAMESPACE, "Send NVME Identify Namespace, display structure."));
            addOperation(new NVMeOperation(OPERATION_WRITE_ZERO, "Write Zero to specified block."));
            addOperation(new NVMeOperation(OPERATION_NVME_SMART, "Display NVMe Smart Log"));

            break;
        case CMD_TEST:
            addOperation(new NVMeOperation(OPERATION_READPFMC, "NVME Read Performance Test"));
            addOperation(new NVMeOperation(OPERATION_WRITEPFMC, "NVME Write Performance Test"));
            addOperation(new NVMeOperation(OPERATION_WRC, "NVME Read/Write Test"));

            break;
        case CMD_FW:
            addOperation(new NVMeOperation(OPERATION_FIRMWARE_ACTIVATE, "Activate new firmware slot."));
            addOperation(new NVMeOperation(OPERATION_FIRMWARE_DOWNLOAD, "Download firmware to NVME Controller."));
            addOperation(new NVMeOperation(OPERATION_FW_LOG, "Display Firmware Log."));

            break;
        default:

            break;
    }
}

NVMeCommand::~NVMeCommand()
{

}

int NVMeCommand::executeCommand(StringParser CommandLine)
{
    INT32 err = 0;
    INT32 OperationIndex;
    COMMAND_TYPE CommandType;
    string strParserOperationName;
    string OperationName;

    CommandLine.getOperation(&strParserOperationName);
    CommandLine.getCommandType(&CommandType);

    switch(CommandType)
    {
        case CMD_LIST:
            if ( strParserOperationName != "")
            {
                cout << "Not Necessary Operation Parameter." << endl;
               return -1;
            }
            break;
        case CMD_SHOW:
        case CMD_TEST:
        case CMD_ERROR:
        case CMD_UTILITY:
        case CMD_FW:
            if(strParserOperationName == "help" || strParserOperationName == "")
            {
                listOperation(CommandLine);
                return -1;
            }

            break;
        default:
            cout << "Not supported command" << endl;
            return -1;
    }

    for(OperationIndex = 0; OperationIndex < m_operation_count; OperationIndex++)
    {
        m_operation[OperationIndex]->getName(&OperationName);

        if(OperationName == strParserOperationName)
        {
            err = m_operation[OperationIndex]->executeOperation(CommandLine);
            break;
        }

    }

    if(OperationIndex == m_operation_count)
    {
        cout << strParserOperationName << " operation not found" << endl;
        return -1;
    }

    return err;
}

