#include "NVMeOperation.hpp"
extern void show_smart_log(struct nvme_smart_log *smart, const char *devname);
extern void show_fw_log(struct nvme_firmware_log_page *fw_log, const char *devname);
extern void show_nvme_id_ctrl(struct nvme_id_ctrl *ctrl, unsigned int mode);
extern void show_nvme_id_ns(struct nvme_id_ns *ns, unsigned int flags);
extern void show_error_log(struct nvme_error_log_page *err_log, int entries, const char *devname);

#define LOG_ENTRIES 64

NVMeOperation::NVMeOperation(OPERATION_TYPE OprationType, string helpContent) :
        OperationBaseClass(OprationType, helpContent)
{
    nvmeDAPI = NULL;
    storageDAPI = NULL;
    progressHandler = NULL;
    setHelp(helpContent);
    operationType = OprationType;

    switch(OprationType)
    {
        case OPERATION_ALL:
            setName("all");

            break;
        case OPERATION_HW:
            setName("hw");

            break;
        case OPERATION_BDF:
            setName("bdf");

            break;
        case OPERATION_TEMP:
            setName("temp");

            break;
        case OPERATION_NODE:
            setName("node");

            break;
        case OPERATION_SN:
            setName("sn");

            break;
        case OPERATION_BLOCK:
            setName("block");

            break;
        case OPERATION_MODEL:
            setName("model");

            break;
        case OPERATION_FW:
            setName("fw");

            break;
        case OPERATION_FW_LOG:
            setName("fwlog");

            break;
        case OPERATION_NVME_SMART:
            setName("smart");

            break;
        case OPERATION_CAPACITY:
            setName("capacity");

            break;
        case OPERATION_IDENTITY_NAMESPACE:
            setName("idns");

            break;
        case OPERATION_IDENTITY_CONTROLLER:
            setName("idctrl");

            break;
        case OPERATION_READPFMC:
            setName("readpfmc");

            break;
        case OPERATION_WRITEPFMC:
            setName("writepfmc");

            break;
        case OPERATION_WRC:
            setName("wrc");

            break;
        case OPERATION_ALL_ERROR:
            setName("all");

            break;
        case OPERATION_RESET_CONTROLLER:
            setName("reset");

            break;
        case OPERATION_FIRMWARE_ACTIVATE:
            setName("activate");

            break;
        case OPERATION_FIRMWARE_DOWNLOAD:
            setName("download");

            break;
        case OPERATION_WRITE_ZERO:
            setName("writezero");

            break;
        default:
            break;
    }
}

NVMeOperation::~NVMeOperation()
{
    if(nvmeDAPI != NULL)
        delete nvmeDAPI;
    if(storageDAPI != NULL)
        delete storageDAPI;
    if(progressHandler != NULL)
        delete progressHandler;
}

int NVMeOperation::executeOperation(StringParser CommandLine)
{
    string parameter1;
    COMMAND_TYPE CommandType;
    int err = 0;

    CommandLine.getParameter(&parameter1, 0);
    CommandLine.getCommandType(&CommandType);

    //If the first parameter is "help" or space, show help message
    if(parameter1 == HELP)
    {
        helpParameter();
        return 0;
    }

    //Execute operation
    switch(operationType)
    {
        case NO_OPRATION:
            err = executeListNVMe(CommandLine);

            break;
        case OPERATION_ALL:
        case OPERATION_HW:
        case OPERATION_BDF:
        case OPERATION_FW:
        case OPERATION_FW_LOG:
        case OPERATION_ERROR_LOG:
        case OPERATION_TEMP:
        case OPERATION_NODE:
        case OPERATION_SN:
        case OPERATION_BLOCK:
        case OPERATION_MODEL:
        case OPERATION_CAPACITY:
        case OPERATION_NVME_SMART:
            err = executeShowController(CommandLine);

            break;
        case OPERATION_ALL_ERROR:
            err = executeError(CommandLine);

            break;
        case OPERATION_RESET_CONTROLLER:
        case OPERATION_IDENTITY_CONTROLLER:
        case OPERATION_IDENTITY_NAMESPACE:
        case OPERATION_WRITE_ZERO:
            err = executeUtility(CommandLine);

            break;
        case OPERATION_FIRMWARE_ACTIVATE:
            err = executeActivateFW(CommandLine);

            break;
        case OPERATION_FIRMWARE_DOWNLOAD:
            err = executeDownloadFW(CommandLine);

            break;
        case OPERATION_READPFMC:
        case OPERATION_WRITEPFMC:
        case OPERATION_WRC:
            err = executeTest(CommandLine);

            break;
        default:
            cout << "Unsupport operation" << endl;
            err = -1;
            break;
    }

    return err;
}

int NVMeOperation::helpParameter()
{
    //Show parameter help message depend on operation type
    switch(operationType)
    {
        case OPERATION_ALL:
        case OPERATION_HW:
        case OPERATION_BDF:
        case OPERATION_FW:
        case OPERATION_FW_LOG:
        case OPERATION_ERROR_LOG:
        case OPERATION_BLOCK:
        case OPERATION_TEMP:
        case OPERATION_NODE:
        case OPERATION_SN:
        case OPERATION_CAPACITY:
        case OPERATION_ALL_ERROR:
            cout << "Show NVMe device characteristic" << endl;
            cout << "Syntax: P1 P2" << endl;
            cout << "P1: Data Elements" << endl;
            cout << "P2: Device Name" << endl;

            break;
        case OPERATION_READPFMC:
            cout << "NVMe Device Read Performance" << endl;
            cout << "Syntax: readpfc P1 P2 P3" << endl;
            cout << "P1: Start Sector(Hex)" << endl;
            cout << "P2: Test Size/MB(Dec)" << endl;
            cout << "P3: Device Name" << endl;

            break;
        case OPERATION_WRITEPFMC:
            cout << "NVMe Device Write Performance" << endl;
            cout << "Syntax: writepfmc P1 P2 P3" << endl;
            cout << "P1: Start Sector(Hex)" << endl;
            cout << "P2: Test Size/MB(Dec)" << endl;
            cout << "P3: Device Name" << endl;

            break;
        case OPERATION_WRC:
            cout << "NVMe Device Read/Write Test" << endl;
            cout << "Syntax: wrc P1 P2 P3" << endl;
            cout << "P1: Start Sector(Hex)" << endl;
            cout << "P2: Test Size/MB(Dec)" << endl;
            cout << "P3: Device Name" << endl;

            break;
        case OPERATION_RESET_CONTROLLER:
            cout << "Reset NVMe device" << endl;
            cout << "Syntax: P1" << endl;
            cout << "P1: Device Name" << endl;

            break;
        case OPERATION_IDENTITY_CONTROLLER:
            cout << "Send an Identify Controller command to NVME and report information" << endl;
            cout << "Syntax: P1" << endl;
            cout << "P1: Device Name" << endl;

            break;
        case OPERATION_IDENTITY_NAMESPACE:
            cout << "Send an Identify Namespace command to NVME and report information" << endl;
            cout << "Syntax: P1" << endl;
            cout << "P1: Device Name" << endl;

            break;
        case OPERATION_FIRMWARE_ACTIVATE:
            cout << "Verify downloaded firmware image and commit to specific firmware slot" << endl;
            cout << "Device is not automatically reset following firmware activation. A reset may be issued" << endl;
            cout << "Syntax: P1 P2 P3" << endl;
            cout << "P1: [0-6]: firmware slot to activate" << endl;
            cout << "P2: [0-2]: replacement action" << endl;
            cout << "P3: Device Name" << endl;

            break;
        case OPERATION_FIRMWARE_DOWNLOAD:
            cout << "Copy a firmware image to controller for future update" << endl;
            cout << "Activate downloaded firmware with activate, and then reset the device to apply the downloaded firmware" << endl;
            cout << "Syntax: download P1 P2" << endl;
            cout << "P1: Firmware file path" << endl;
            cout << "P2: Device Name" << endl;

            break;
        case OPERATION_NVME_SMART:
            cout << "Get NVMe device SMART Log" << endl;
            cout << "Syntax: P1" << endl;
            cout << "P1: Device Name" << endl;

            break;
        case OPERATION_WRITE_ZERO:
            cout << "NVMe Device Write Zero to specified blocks" << endl;
            cout << "Syntax: zero P1 P2 P3" << endl;
            cout << "P1: Start Block(Hex)" << endl;
            cout << "P2: Block(Dec)" << endl;
            cout << "P3: Device Name" << endl;

            break;
        default:
            break;
    }

    return 0;
}

int NVMeOperation::executeListNVMe(StringParser CommandLine)
{
    int err = 0;
    vector<NVMe_DEVICE_INFO> nvmeList;
    vector<NVMe_DEVICE_INFO>::iterator nvmeIter;
    ErrorMsg* errorMsg = NO_ERROR_MSG;
    string showErrMsg;

    nvmeDAPI = new NVMeDAPI("NVMeDAPI");

    err = checkSyntax(CommandLine);
    if(err == 0)
    {
        if((errorMsg = nvmeDAPI->getInformationList(&nvmeList)) != NO_ERROR_MSG)
        {
            printError(errorMsg);
            return -1;
        }

		for(nvmeIter = nvmeList.begin(); nvmeIter != nvmeList.end(); ++nvmeIter)
		{
			cout << nvmeIter->wellKnowName << "(" << nvmeIter->domindBusDeviceFunction << ")" << endl;
		}
	}

	return err;
}

int NVMeOperation::executeShowController(StringParser CommandLine)
{
    int err = 0;
    string targetName, capacityString, tempString;;
    UINT8 logicalID = -1;
    int nsid;
    vector<NVMe_DEVICE_INFO> nvmeList;
    UINT32 temperature;
    long long int lba;
    double nsze;
    CHAR cap_str[64];
    CHAR si_str[64];
    ErrorMsg* errorMsg = NO_ERROR_MSG;

    nvmeDAPI = new NVMeDAPI("NVMeDAPI");

    err = checkSyntax(CommandLine);
    if(err == 0)
    {
        CommandLine.getParameter(&targetName, 0);

        if((errorMsg = nvmeDAPI->getInformationList(&nvmeList)) == NO_ERROR_MSG)
        {
            if((errorMsg = nvmeDAPI->getLogicalIDByName(targetName, &logicalID)) == NO_ERROR_MSG)
            {
                if((errorMsg = nvmeDAPI->getBarResourceByController(logicalID, &nvmeList[logicalID].barInfo)) == NO_ERROR_MSG)
                    if((errorMsg = nvmeDAPI->getIdentityCTRLByController(logicalID, &nvmeList[logicalID].idCtrl)) == NO_ERROR_MSG)
                        if((errorMsg = nvmeDAPI->getIdentityNSByController(logicalID, &nvmeList[logicalID].idNs)) == NO_ERROR_MSG)
                            if((errorMsg = nvmeDAPI->getFirmwareLogByController(logicalID, &nvmeList[logicalID].fwLog)) == NO_ERROR_MSG)
                                if((errorMsg = nvmeDAPI->getSmartLogByController(logicalID, &nvmeList[logicalID].smartLog)) == NO_ERROR_MSG)
                                    errorMsg = nvmeDAPI->getNameSpaceIDByController(logicalID, &nsid);
            }
        }

        if(errorMsg)
        {
            printError(errorMsg);
            return -1;
        }
        // get temperature from smart log
        temperature = ((nvmeList[logicalID].smartLog.temperature[1] << 8) | nvmeList[logicalID].smartLog.temperature[0]) - 273;
        lba = nvmeList[logicalID].idNs.lbaf[(nvmeList[logicalID].idNs.flbas & 0x0f)].ds;
        lba  = (1 << lba);
        nsze = nvmeList[logicalID].idNs.nsze;
        nsze *= lba;
        format_with_thousands_sep(cap_str, sizeof(cap_str), nsze);
        format_capacity(si_str, sizeof(si_str), nsze);
        capacityString = capacityString.assign((const char*)cap_str) + " bytes ";
        capacityString += "[" + tempString.assign((const char*)si_str) + "]";

        switch(operationType)
        {
            case OPERATION_ALL:
                cout << "Node Name      : " << nvmeList[logicalID].chardeviceNodeName << endl;
                cout << "Block Node Name: " << nvmeList[logicalID].blockdeviceNodeName << endl;
                printf( "Model Name     : %-20.20s\n", nvmeList[logicalID].idCtrl.mn);
                printf( "Serial Number  : %-20.20s\n", nvmeList[logicalID].idCtrl.sn);
                cout << "HW Version     : " << (nvmeList[logicalID].barInfo->vs >> 16) << "."
                                            << ((nvmeList[logicalID].barInfo->vs >> 8) & 0xff) << endl;
                printf( "FW Version     : %-.8s\n", nvmeList[logicalID].idCtrl.fr);
                cout << "PCI BDF        : " << nvmeList[logicalID].domindBusDeviceFunction << endl;
                cout << "Namespace ID   : " << nsid << endl;
                cout << "User Capacity  : " << capacityString << endl;
                cout << "Block Size     : " << lba << " bytes" << endl;
                cout << "Temperature    : " << temperature << " C" << endl;

                break;
            case OPERATION_HW:
                cout << "HW Version : " << (nvmeList[logicalID].barInfo->vs >> 16) << "."
                                            << ((nvmeList[logicalID].barInfo->vs >> 8) & 0xff) << endl;
                break;
            case OPERATION_FW:
                printf( "FW Version : %-.8s\n", nvmeList[logicalID].idCtrl.fr);

                break;
            case OPERATION_SN:
                printf( "Serial Number : %-20.20s\n", nvmeList[logicalID].idCtrl.sn);

                break;
            case OPERATION_MODEL:
                printf( "Model Name : %-20.20s\n", nvmeList[logicalID].idCtrl.mn);

                break;
            case OPERATION_BDF:
                cout << "PCI BDF : " <<  nvmeList[logicalID].domindBusDeviceFunction << endl;

                break;
            case OPERATION_TEMP:
                cout << "Temperature : " <<  temperature << " C" << endl;

                break;
            case OPERATION_CAPACITY:
                cout << "User Capacity : " << capacityString << endl;

                break;
            case OPERATION_BLOCK:
                cout << "Block Size : " << lba << " B" << endl;

                break;
            case OPERATION_FW_LOG:
                show_fw_log(&nvmeList[logicalID].fwLog, nvmeList[logicalID].wellKnowName.c_str());

                break;
            case OPERATION_NVME_SMART:
                show_smart_log(&nvmeList[logicalID].smartLog, nvmeList[logicalID].wellKnowName.c_str());

                break;
            default:
                cout << "Unsupport operation parameter" << endl;
                return -1;
        }
        targetName.clear();
    }

    return err;
}

int NVMeOperation::executeError(StringParser CommandLine)
{
    int err = 0;
    string targetName;
    UINT8 logicalID = -1;
    vector<NVMe_DEVICE_INFO> nvmeList;
    ErrorMsg* errorMsg = NO_ERROR_MSG;

    nvmeDAPI = new NVMeDAPI("NVMeDAPI");

    err = checkSyntax(CommandLine);
    if(err == 0)
    {
        CommandLine.getParameter(&targetName, 0);
        errorMsg = nvmeDAPI->getInformationList(&nvmeList);
        if(errorMsg == NO_ERROR_MSG)
        {
            if((errorMsg = nvmeDAPI->getLogicalIDByName(targetName, &logicalID)) == NO_ERROR_MSG)
                if((errorMsg = nvmeDAPI->getErrorLogByController(logicalID, &nvmeList[logicalID].errLog)) == NO_ERROR_MSG)
                    show_error_log(&nvmeList[logicalID].errLog, 1, nvmeList[logicalID].wellKnowName.c_str());
        }
    }

    if(errorMsg)
    {
        printError(errorMsg);
        return -1;
    }

    return err;
}

int NVMeOperation::executeUtility(StringParser CommandLine)
{
    int err = 0;
    string targetName, tempString;
    UINT64 startBlock = 0;
    int blockCount = 0;
    UINT8 logicalID = -1;
    vector<NVMe_DEVICE_INFO> nvmeList;
    ErrorMsg* errorMsg = NO_ERROR_MSG;

    nvmeDAPI = new NVMeDAPI("NVMeDAPI");

    err = checkSyntax(CommandLine);
    if(err == 0)
    {
        if(operationType == OPERATION_WRITE_ZERO)
        {
            CommandLine.getParameter(&tempString, 0);
            CommandLine.convertStringToHex(tempString, &startBlock);

            CommandLine.getParameter(&tempString, 1);
            CommandLine.convertStringToDec(tempString, &blockCount);

            CommandLine.getParameter(&targetName, 2);
        }
        else
            CommandLine.getParameter(&targetName, 0);

        errorMsg = nvmeDAPI->getInformationList(&nvmeList);

        if(errorMsg == NO_ERROR_MSG)
        {
            if((errorMsg = nvmeDAPI->getLogicalIDByName(targetName, &logicalID)) == NO_ERROR_MSG)
            {
                switch(operationType)
                {
                    case OPERATION_RESET_CONTROLLER:
                        errorMsg = nvmeDAPI->resetNVMe(logicalID);

                        break;
                    case OPERATION_IDENTITY_CONTROLLER:
                        if((errorMsg = nvmeDAPI->getIdentityCTRLByController(logicalID, &nvmeList[logicalID].idCtrl)) == NO_ERROR_MSG)
                            show_nvme_id_ctrl(&nvmeList[logicalID].idCtrl, HUMAN);

                        break;
                    case OPERATION_IDENTITY_NAMESPACE:
                        if((errorMsg = nvmeDAPI->getIdentityNSByController(logicalID, &nvmeList[logicalID].idNs)) == NO_ERROR_MSG)
                            show_nvme_id_ns(&nvmeList[logicalID].idNs, HUMAN);

                        break;
                    case OPERATION_WRITE_ZERO:
                        errorMsg = nvmeDAPI->writeZeroByController(logicalID, startBlock, blockCount);

                        break;

                    default:
                        break;
                }
            }
        }
    }

    if(errorMsg)
    {
        printError(errorMsg);
        return -1;
    }

    return err;
}

int NVMeOperation::executeActivateFW(StringParser CommandLine)
{
    int err = 0, intParameterNumber = 0;
    string parameter[3];
    UINT8 logicalID = -1;
    vector<NVMe_DEVICE_INFO> nvmeList;
    ErrorMsg* errorMsg = NO_ERROR_MSG;

    nvmeDAPI = new NVMeDAPI("NVMeDAPI");

    err = checkSyntax(CommandLine);
    if(err == 0)
    {
        //Get parameter
        for(int i = 1; i < intParameterNumber; i++)
            CommandLine.getParameter(&parameter[i], i);

        errorMsg = nvmeDAPI->getInformationList(&nvmeList);

        if(errorMsg == NO_ERROR_MSG)
        {
            if((errorMsg = nvmeDAPI->getLogicalIDByName(parameter[2], &logicalID)) == NO_ERROR_MSG)
            {
                struct nvme_fw_active activate;

                CommandLine.convertStringToDec(parameter[0], (int*)&activate.slot);
                CommandLine.convertStringToDec(parameter[1], (int*)&activate.action);

                if (activate.slot > 7)
                {
                    cout << "invalid slot: " << activate.slot << endl;
                    return -1;
                }
                if (activate.action > 3)
                {
                    cout << "invalid action: " << activate.action << endl;
                    return -1;
                }
                errorMsg = nvmeDAPI->activateFirmwareByController(logicalID, activate);
            }
        }
    }

    if(errorMsg)
    {
        printError(errorMsg);
        return -1;
    }

    return err;
}

int NVMeOperation::executeDownloadFW(StringParser CommandLine)
{
    int err = 0, intParameterNumber = 0;
    string parameter[2];
    UINT8 logicalID = -1;
    vector<NVMe_DEVICE_INFO> nvmeList;
    ErrorMsg* errorMsg = NO_ERROR_MSG;

    nvmeDAPI = new NVMeDAPI("NVMeDAPI");

    err = checkSyntax(CommandLine);
    if(err == 0)
    {
        //Get parameter
        for(int i = 1; i < intParameterNumber; i++)
            CommandLine.getParameter(&parameter[i], i);

        errorMsg = nvmeDAPI->getInformationList(&nvmeList);

        if(errorMsg == NO_ERROR_MSG)
        {
            if((errorMsg = nvmeDAPI->getLogicalIDByName(parameter[1], &logicalID)) == NO_ERROR_MSG)
            {
                errorMsg = nvmeDAPI->downloadFirmwareByController(logicalID, parameter[0].c_str());
            }
        }
    }

    if(errorMsg)
    {
        printError(errorMsg);
        return -1;
    }

    return err;
}

int NVMeOperation::executeTest(StringParser CommandLine)
{
    int err = 0;
    string targetName, tempString;
    UINT8 logicalID = -1;
    INT32 startSector = 0;
    INT32 testSize = 0;
    vector<NVMe_DEVICE_INFO> nvmeList;
    ErrorMsg* errorMsg = NO_ERROR_MSG;

    nvmeDAPI = new NVMeDAPI("NVMeDAPI");
    storageDAPI = new StorageDAPI("StorageDAPI");

    err = checkSyntax(CommandLine);
    if(err == 0)
    {
        CommandLine.getParameter(&tempString, 0);
        CommandLine.convertStringToHex(tempString, &startSector);
        tempString.clear();

        CommandLine.getParameter(&tempString, 1);
        CommandLine.convertStringToDec(tempString, &testSize);
        tempString.clear();

        CommandLine.getParameter(&targetName, 2);

        if((errorMsg = storageDAPI->getLogicalIdByName(targetName.c_str(), (UINT32*)&logicalID)) != NO_ERROR_MSG)
        {
            printError(errorMsg);
            return -1;
        }

        errorMsg = nvmeDAPI->getInformationList(&nvmeList);

        if(errorMsg == NO_ERROR_MSG)
        {
            switch(operationType)
            {
                case OPERATION_READPFMC:
                    progressHandler = new thread(&NVMeOperation::handleTestProgress, this, (UINT32)testSize);
                    errorMsg = storageDAPI->readPerformanceTest(logicalID, (UINT32)startSector, (UINT32)testSize);
                    if(errorMsg)
                    {
                        printError(errorMsg);
                        return -1;
                    }
                    progressHandler->join();
                    cout << storageDAPI->m_strOutputResultString;
                    break;

                case OPERATION_WRITEPFMC:
                    progressHandler = new thread(&NVMeOperation::handleTestProgress, this, (UINT32)testSize);
                    errorMsg = storageDAPI->writePerformanceTest(logicalID, (UINT32)startSector, (UINT32)testSize);
                    if(errorMsg)
                    {
                        printError(errorMsg);
                        return -1;
                    }
                    progressHandler->join();
                    cout << storageDAPI->m_strOutputResultString;
                    break;

                case OPERATION_WRC:
                    progressHandler = new thread(&NVMeOperation::handleTestProgress, this, (UINT32)testSize);
                    errorMsg = storageDAPI->readwriteTest(logicalID, (UINT32)startSector, (UINT32)testSize);
                    if(errorMsg)
                    {
                        printError(errorMsg);
                        return -1;
                    }
                    progressHandler->join();
                    cout << storageDAPI->m_strOutputResultString;
                    break;

                default:
                    cout << "Unsupport operation" << endl;
                    err = -1;

                    break;
            }
        }
    }

    if(errorMsg)
    {
        printError(errorMsg);
        return -1;
    }

    return err;
}

int NVMeOperation::checkSyntax(StringParser CommandLine)
{
    int i, intParameterNumber = 0;
    unsigned long long hex[2];
    string parameter[7];
    int err = 0;

    CommandLine.getParameterNumber(&intParameterNumber);

    switch(operationType)
    {
        case NO_OPRATION:
            if(intParameterNumber > 0)
            {
                cout << "Not Necessary Operation Parameter" << endl;
                helpParameter();
                return -1;
            }
            break;

        case OPERATION_ALL:
        case OPERATION_NODE:
        case OPERATION_SN:
        case OPERATION_MODEL:
        case OPERATION_CAPACITY:
        case OPERATION_TEMP:
        case OPERATION_BDF:
        case OPERATION_HW:
        case OPERATION_BLOCK:
        case OPERATION_FW:
        case OPERATION_FW_LOG:
        case OPERATION_ALL_ERROR:
        case OPERATION_RESET_CONTROLLER:
        case OPERATION_IDENTITY_CONTROLLER:
        case OPERATION_IDENTITY_NAMESPACE:
        case OPERATION_NVME_SMART:
            if(intParameterNumber != 1)
            {
                cout << "Parameter numbers isn't 1" << endl;
                helpParameter();
                return -1;
            }
            break;

        case OPERATION_FIRMWARE_ACTIVATE:
        case OPERATION_READPFMC:
        case OPERATION_WRITEPFMC:
        case OPERATION_WRC:
        case OPERATION_WRITE_ZERO:
            if(intParameterNumber != 3)
            {
                cout << "Parameter numbers isn't 3" << endl;
                helpParameter();
                return -1;
            }
            //Get parameter
            for(i = 0; i < intParameterNumber; i++)
                CommandLine.getParameter(&parameter[i], i);

            //Check parameter 1 is numeric
            err = CommandLine.convertStringToHex(parameter[0], &hex[0]);
            if(err != 0)
            {
                cout << "parameter 1 isn't numeric." << endl;
                helpParameter();
                return -1;
            }
            //Check parameter 2 is numeric
            err = CommandLine.convertStringToHex(parameter[1], &hex[1]);
            if(err != 0)
            {
                cout << "parameter 2 isn't numeric." << endl;
                helpParameter();
                return -1;
            }

            break;

        case OPERATION_FIRMWARE_DOWNLOAD:
            if(intParameterNumber != 2)
            {
                cout << "Parameter numbers isn't 2" << endl;
                helpParameter();
                return -1;
            }
            break;

        default:
            break;
    }

    return 0;
}

void NVMeOperation::handleTestProgress(UINT32 in_u32TestSizeMB)
{
    INT32 progress = 0;

    sleep(1);
    while(progress < 100)// <= 100%
    {
        sleep(1);
        progress = storageDAPI->getProgress();
        printf("Progressing: %3u%% \r", progress);
        fflush(stdout);
    }
    printf("\n");
}

// Format integer with thousands separator
const char *
NVMeOperation::format_with_thousands_sep(char * str, int strsize, uint64_t val,
                                        const char * thousands_sep /* = 0 */)
{
  if (!thousands_sep)
    thousands_sep = ",";

  char num[64];
  snprintf(num, sizeof(num), "%llu", val);
  int numlen = strlen(num);

  int i = 0, j = 0;
  do
    str[j++] = num[i++];
  while (i < numlen && (numlen - i) % 3 != 0 && j < strsize-1);
  str[j] = 0;

  while (i < numlen && j < strsize-1) {
    j += snprintf(str+j, strsize-j, "%s%.3s", thousands_sep, num+i);
    i += 3;
  }

  return str;
}

// Format capacity with SI prefixes
const char *
NVMeOperation::format_capacity(char * str, int strsize, uint64_t val,
                              const char * decimal_point /* = 0 */)
{
  if (!decimal_point)
    decimal_point = ".";

  const unsigned factor = 1000; // 1024 for KiB,MiB,...
  static const char prefixes[] = " KMGTP";

  // Find d with val in [d, d*factor)
  unsigned i = 0;
  uint64_t d = 1;
  for (uint64_t d2 = d * factor; val >= d2; d2 *= factor) {
    d = d2;
    if (++i >= sizeof(prefixes)-2)
      break;
  }

  // Print 3 digits
  uint64_t n = val / d;
  if (i == 0)
    snprintf(str, strsize, "%u B", (unsigned)n);
  else if (n >= 100) // "123 xB"
    snprintf(str, strsize, "%llu %cB", n, prefixes[i]);
  else if (n >= 10)  // "12.3 xB"
    snprintf(str, strsize, "%llu%s%u %cB", n, decimal_point,
        (unsigned)(((val % d) * 10) / d), prefixes[i]);
  else               // "1.23 xB"
    snprintf(str, strsize, "%llu%s%02u %cB", n, decimal_point,
        (unsigned)(((val % d) * 100) / d), prefixes[i]);

  return str;
}
