/*******************************************************************************

NAME			$RCSfile: init.c,v $
SUMMARY			Initial module process
VERSION			$Revision: 1.2 $
UPDATE DATE		$Date: 2010/07/14 08:13:16 $
PROGRAMMER		$Author: alan $

		Copyright 2009 LSI Corporation. All Rights Reserved.

DESCRIPTION:
    The functions used to report diagnostics status

REFERENCE:

*******************************************************************************/

/***  INCLUDES  ***/
#include <linux/init.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <linux/seq_file.h>
#include <linux/slab.h>

//#include <commTypes.h>
//#include <commRpt.h>
//#include <commPciChip.h>

#include "diag_proc.h"
#include "mem_space.h"

/***  EXTERNAL REFERENCES  ***/
extern int 	diagProcInit(void);


/***  proc_dir_entry /proc/daig  ***/
struct proc_dir_entry *entry;
/***  proc_dir_entry /proc/setparam-dbg  ***/
struct proc_dir_entry *diagSetParamEntry;


/* mapiResult start */
#define	mapiResult_BUFSIZE	(4*1024)
#define proc_mapiResult_S struct proc_mapiResult_Structure
proc_mapiResult_S{
	int	flags;
	unsigned char	buf[mapiResult_BUFSIZE];
};

static proc_mapiResult_S mapiResult;
unsigned char *mapiResultBuf=mapiResult.buf;

EXPORT_SYMBOL(mapiResultBuf);

static void *mapiResult_start(struct seq_file *m, loff_t *pos)
{
	if (*pos == 0)
		return &mapiResult;
	*pos=0;
	return NULL;
}

static void *mapiResult_next(struct seq_file *m, void *v, loff_t *pos)
{
	(*pos)++;

	return (mapiResult_start(m,pos));
}

static void mapiResult_stop(struct seq_file *m, void *v)
{
}

static int mapiResult_show(struct seq_file *m, void *v)
{

	seq_printf(m, "%s",	mapiResult.buf);
	mapiResult.buf[0]='\0';
	return 0;
}

int mapi_SeqRelease(struct inode *inode, struct file *file)
{
	struct seq_file *p = file->private_data;

	if(p->buf) {
		kfree(p->buf);
		p->buf=NULL;
	}
	kfree(p);
	file->private_data=NULL;
	return 0;
}

const struct seq_operations mapiResult_op = {
	.start	= mapiResult_start,
	.next	= mapiResult_next,
	.stop	= mapiResult_stop,
	.show	= mapiResult_show
};

static int mapiResult_open(struct inode *inode, struct file *file)
{
	return seq_open(file, &mapiResult_op);
}

static const struct file_operations proc_mapiResult_operations = {
	.open		= mapiResult_open,
	.release	= mapi_SeqRelease,
	.read		= seq_read,
};

static int mapiResult_ProcInit(void)
{
	struct proc_dir_entry *entry;

	mapiResultBuf=mapiResult.buf;
	mapiResult.buf[0]='\0';

	entry = proc_create("mapiResult", 0444, NULL, &proc_mapiResult_operations);
	if (!entry)
		return -ENOMEM;

	return 0;
}
/*
void mapiResult_Init(void)
{
	mapiResult.buf[0]='\0';
}
*/
/*mapiResult end*/

static const struct file_operations diag_fops = {
	.read = diagProcRead,
	.write = diagProcWrite,
};

static const struct file_operations para_fops = {
	.read = diagParamRead,
	.write = diagParamWrite,
};

/*******************************************************************************
* PROCEDURE
*
* NAME:     diagInit
* SUMMARY:   
* SCOPE:    public
*
* DESCRIPTION:  
*
* RETURNS:
*
* NOTES:
*
*/
static int __init diagInit(void)
{
	/* define memory testing area*/
	diagGetKernelMemory();
	/* /proc/diag */
	entry = proc_create_data("diag", 0644, NULL, &diag_fops, NULL);
	if (!entry){
		printk("Failed to init daig proc entry\n");
	}

	/* /proc/setparam-dbg */
	diagSetParamEntry = proc_create_data("setparam-dbg", 0, NULL, &para_fops, NULL);
	if (!diagSetParamEntry){
		printk("[PROC_DBG] /proc/setparam-dbg create ERROR!\n");
	}
	mapiResult_ProcInit();

	return (0);
}

/*******************************************************************************
* PROCEDURE
*
* NAME:     diagExit
* SUMMARY:   
* SCOPE:    public
*
* DESCRIPTION:  
*
* RETURNS:
*
* NOTES:
*
*/
static void __exit diagExit(void)
{
	remove_proc_entry("diag", NULL);
	remove_proc_entry("setparam-dbg", NULL);
	return;
}

module_init(diagInit);
module_exit(diagExit);
MODULE_LICENSE("GPL");
