/****************************************************************************
*                                                                           *
* Copyright 2004-2011 NetApp Inc. All rights reserved. This file is         *
* confidential and a trade secret of NetApp Inc. The receipt or             *
* possession of this file does not convey any rights to reproduce or        *
* disclose its contents or to manufacture, use, or sell anything it may     *
* describe, in whole, or in part, without the specific written consent of   *
* LSI Corporation.                                                          *
*                                                                           *
****************************************************************************/
/****************************************************************************
*                                                                           *
* NAME            fpgaDiagInit.c                                            *
* SUMMARY                                                                   *
* VERSION         %version: %                                               *
* UPDATE DATE     %date_modified: %                                         *
* PROGRAMMER      %created_by:    Merck Hung %                              *
*                                                                           *
*        Copyright 2002-2011 NetApp Inc. All Rights Reserved.               *
*                                                                           *
* DESCRIPTION:                                                              *
*                                                                           *
****************************************************************************/

#include <linux/kobject.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/fcntl.h>
#include <linux/device.h>

#include <linux/pci.h>
#include <linux/wait.h>
#include <linux/interrupt.h>
#include <linux/platform_device.h>
#include <linux/delay.h>

#include <commTypes.h>
#include <fpgaRegLib.h>

#include <pci-zebulon.h>
#include <fpga-subfuncs.h>
#include <midplane.h>

#include "../diags/commPciChip.h"


//HIC IDs
#define INVALID_HOST_CARD       0x00
#define GLEN_COVE_ID_ISCSI      0x01    //iSCSI host card
#define GLACIER_ID_SAS          0x03    //SAS host card
#define ELK_PARK_ID_IB          0x04    //InfiniBand host card
#define MANITOU_ID_FC           0x05    //Fibre Channel host card
#define DEFAULT_NO_CARD         0x0F    //No host card

typedef enum host_card_type {
    HOST_CARD_NONE = 0,
    HOST_CARD_FC,
    HOST_CARD_ISCSI,
    HOST_CARD_IB,
    HOST_CARD_SAS
} HOST_CARD_TYPE;


UINT8 fpagRevisionRead(VOID);
EXTERN INT32 midplaneTypeVerification(VOID);
EXTERN VOID testMidplaneTypeVerify(VOID);
EXTERN VOID fpagHicVerify(VOID);
EXTERN UINT8 fpgaCtlRoleRead(VOID);

HOST_CARD_TYPE sysHostCardType;
EXPORT_SYMBOL(sysHostCardType);

UINT32 sysMidplaneType;


static void pikespeakDiagSetBoardInfo( void ) {

	testMidplaneTypeVerify();
	fpagHicVerify();
}


static void pikespeakDiagShowBoardInfo( void ) {

    UINT8   ctlRole;
    UINT8   ctlSlot;

#ifndef ALANWANG    // 2010/07/14
#define FPGA_BOARD_ID   0x00000001          // ALANWANG, 2010/07/14
extern UINT8 fpgaRegisterRead(UINT32 fpgaOffset);

    //verify the UUT type & position
    ctlSlot = fpgaRegisterRead(FPGA_BOARD_ID);
    if(ctlSlot != 0x94 && ctlSlot != 0x14) {
//  if(ctlSlot != 0x9e && ctlSlot != 0x1e) {
        printk("[BOARD_INFO] WARNNING: unknown controller board [%x] !!!\n", ctlSlot);
    } else {
        ctlRole = fpgaCtlRoleRead();
        //printk("[BOARD_INFO] Platform is Pikes Peak\n");
        printk("[BOARD_INFO] Platform is Soyuz\n");
        printk("[BOARD_INFO] Controller is in %s. This is %s\n",
//                  (ctlSlot==0x9e)? "Slot B":"Slot A", (ctlRole)? "FIXTURE":"UUT");
                    (ctlSlot==0x94)? "Slot B":"Slot A", (ctlRole)? "FIXTURE":"UUT");
    }
#endif

    //show HIC type
    switch (sysHostCardType)
    {
        /* QE8 */
        case HOST_CARD_FC:
            printk("[BOARD_INFO] Host Card FC\n");
            break;
        /* ISCSI */
        case HOST_CARD_ISCSI:
            printk("[BOARD_INFO] Host Card ISCSI\n");
            break;
        /* IB */
        case HOST_CARD_IB:
            printk("[BOARD_INFO] Host Card IB\n");
            break;
		/* SAS */
		case HOST_CARD_SAS:
			printk("[BOARD_INFO] Host Card SAS\n");
			break;
        default:
            printk("[BOARD_INFO] There is no host card on bus\n");
    }

    //verify midplane type
    if(midplaneTypeVerification())
    {   //Quanta test midplane
        printk("[BOARD_INFO] NetApp test midplane\n");
        printk("[BOARD_INFO] Test midplane type: ver.%d \n", sysMidplaneType);
    }
    else
    {   //LSI midplane
        printk("[BOARD_INFO] NetApp midplane\n");
    }

    //verify FPGA revision
    printk("[BOARD_INFO] FPGA Revision: %02X \n", fpagRevisionRead());
}


uint8_t ledMap_12_a[] ={0x0, 0x3, 0x12, 0x21,
            0x6, 0x9, 0x15, 0x1e,
            0xc, 0xf, 0x18, 0x1b},
    ledMap_12_b[] ={0xf, 0x1b, 0x1e, 0x21,
            0x3, 0x12, 0x15, 0x18,
            0x0, 0x6,  0x9,  0xc},
    ledMap_24_a[] ={0x00, 0x03, 0x06, 0x09,
            0x0C, 0x0F, 0x12, 0x15,
            0x27, 0x24, 0x21, 0x1E,
            0x30, 0x18, 0x1B, 0x2D,
            0x33, 0x2A, 0x36, 0x42,
            0x45, 0x3F, 0x3C, 0x39},
    ledMap_24_b[] ={0x0F, 0x0C, 0x09, 0x00,
            0x03, 0x06, 0x12, 0x15,
            0x18, 0x21, 0x1E, 0x1B,
            0x24, 0x27, 0x2A, 0x2D,
            0x30, 0x33, 0x36, 0x39,
            0x3C, 0x3F, 0x45, 0x42};


unsigned short gLEDTestMode = 0;
extern void fpgaLedAssign(uint8_t ledIndex, uint8_t ledPatternIndex);
extern void fpgaOverTempLedEnable(int status);
extern void fpgaLedForProcessorBoot(uint8_t status);
extern void fpgaLedSync(int status);


void fpgaStandByLedEnable(int status){
    uint8_t temp;

    temp = fpgaRegisterRead(0x5);

    if(status){
         fpgaRegisterWrite(0x5, (temp | 0x40));
    }
    else{
      fpgaRegisterWrite(0x5, (temp & (~0x40)));
    }
}


void mfgSetCtlLED(uint16_t ledIndex, uint8_t state ){

    uint8_t ledName = 0, ledNo, temp, bdID; // ssID,
#if defined(CHASSIS_WSAS)
    uint8_t *ledMap = 0;
#else
    uint8_t *ledMap;
#endif

    ledName = (uint8_t)(ledIndex    >> 8);
    ledNo = (uint8_t)(ledIndex & 0xff);

    bdID = fpgaRegisterRead(0x1);
    temp = fpgaRegisterRead(FPGA_SUBSYSTEM_ID);

#if defined(CHASSIS_WSAS)
    //printk("bdID = 0x%02x, temp = 0x%02x\n", bdID, temp);  //Modify by Diran
#endif

    if((temp & 0x0F) == SBB2_MIDPLANE_12){
//      ssID = 0x12;
        ledMap=(bdID&0x80)?ledMap_12_b:ledMap_12_a;
    }
    else if((temp & 0x0F) == SBB2_MIDPLANE_24){
//      ssID = 0x24;
        ledMap=(bdID&0x80)?ledMap_24_b:ledMap_24_a;
    }
    else{
#if defined(CHASSIS_WSAS)
        //return;
#else
        return;
#endif
//      ssID = 0x0;
    }


#if defined(CHASSIS_WSAS)
    //printk("ledName = 0x%02x, ledNo = 0x%02x\n", ledName, ledNo);  //Modify by Diran
#endif


    switch(ledName){
    case 1:
/*      if(ssID == 0x12){
            if(bdID&0x80)
                fpgaLedAssign(ledMap_12_b[ledNo], state);
            else
                fpgaLedAssign(ledMap_12_a[ledNo], state);
            gLEDTestMode = 1;
        }
        else if(ssID == 0x24){
            if(bdID&0x80)
                fpgaLedAssign(ledMap_24_b[ledNo], state);
            else
                fpgaLedAssign(ledMap_24_a[ledNo], state);
            gLEDTestMode = 1;
        }
*/
        fpgaLedAssign(ledMap[ledNo], state);
        gLEDTestMode = 1;
        break;
    case 5:
/*      if(ssID == 0x12){
            if(bdID&0x80)
                fpgaLedAssign(ledMap_12_b[ledNo]+1, state);
            else
                fpgaLedAssign(ledMap_12_a[ledNo]+1, state);
                gLEDTestMode = 1;
        }
        else if(ssID == 0x24){
            if(bdID&0x80)
                fpgaLedAssign(ledMap_24_b[ledNo]+1, state);
            else
                fpgaLedAssign(ledMap_24_a[ledNo]+1, state);
            gLEDTestMode = 1;
         }
*/
        fpgaLedAssign(ledMap[ledNo]+1, state);
        gLEDTestMode = 1;
        break;
    case 6:
          fpgaLedAssign(ledNo, state);
          gLEDTestMode = 1;
          break;
    case 2:
        if(ledNo == 1){
            fpgaOverTempLedEnable(state);
        }
        else if(ledNo == 2){
            fpgaStandByLedEnable(state);
        }
        else {

#if 0
            qe8MapiLED(((ledNo>>4)&0x0f) -3, state, (ledNo&0x0f));
#endif
        }
/*          else if(ledNo == 0x30 || ledNo == 0x31){
                qe8MapiLED(0, state, (ledNo&0x0f));
            }
            else if(ledNo == 0x40 || ledNo == 0x41){
                qe8MapiLED(1, state, (ledNo&0x0f));
            }
            else if(ledNo == 0x50 || ledNo == 0x51){
                qe8MapiLED(2, state, (ledNo&0x0f));
            }
            else if(ledNo == 0x60 || ledNo == 0x61){
                qe8MapiLED(3, state, (ledNo&0x0f));
            }
            else if(ledNo == 0x70 || ledNo == 0x71){
                qe8MapiLED(4, state, (ledNo&0x0f));
            }
            else if(ledNo == 0x80 || ledNo == 0x81){
                qe8MapiLED(5, state, (ledNo&0x0f));
            }
            else if(ledNo == 0x90 || ledNo == 0x91){
                qe8MapiLED(6, state, (ledNo&0x0f));
            }
            else if(ledNo == 0xa0 || ledNo == 0xa1){
                qe8MapiLED(7, state, (ledNo&0x0f));
            }
*/
        gLEDTestMode = 1;
        break;
    case 3:
        fpgaLedForProcessorBoot(FALSE);
        fpgaRegisterWrite(FPGA_SEVEN_SEGMENT_BLINK_CONTROL, 0x01);
        temp = fpgaRegisterRead(FPGA_SEVEN_SEGMENT_DISPLAY_1);
        if(!state)
            ledNo = (temp&~ledNo);
        else
            ledNo |= temp;
        fpgaRegisterWrite(FPGA_SEVEN_SEGMENT_DISPLAY_1, ledNo);
        fpgaLedSync(TRUE);
        mdelay(1); //delay 1ms
        fpgaLedSync(FALSE);
        gLEDTestMode = 1;
        break;
    case 4:
        fpgaLedForProcessorBoot(FALSE);
        fpgaRegisterWrite(FPGA_SEVEN_SEGMENT_BLINK_CONTROL, 0x01);
        temp = fpgaRegisterRead(FPGA_SEVEN_SEGMENT_DISPLAY_2);
        if(!state)
            ledNo = (temp&~ledNo);
        else
            ledNo |= temp;
            fpgaRegisterWrite(FPGA_SEVEN_SEGMENT_DISPLAY_2, ledNo);
        fpgaLedSync(TRUE);
        mdelay(1); //delay 1ms
        fpgaLedSync(FALSE);
        gLEDTestMode = 1;
        break;
    default:
        break;
    }
}


int fpgaPciRegReadTest( void ) {

    return pciRegTest( NETAPP_ROCKET_FPGA, pciReadTest );
}

int fpgaPciAddressLineTest( void ) {

    return pciRegTest( NETAPP_ROCKET_FPGA, pciAddressLineTest );
}

int fpgaPciDataLineTest( void ) {

    return pciRegTest( NETAPP_ROCKET_FPGA, pciDataLineTest );
}


static int __init fpgaDiagInit( void ) {

	pikespeakDiagSetBoardInfo();
	pikespeakDiagShowBoardInfo();
	return 0;
}


static void __exit fpgaDiagExit( void ) {
}


module_init( fpgaDiagInit );
module_exit( fpgaDiagExit );

MODULE_AUTHOR( "merck.hung@netapp.com" );
MODULE_DESCRIPTION( "Zebulon FPGA Diag Code" );
MODULE_LICENSE( "GPL" );


