/*******************************************************************************

NAME            fpgaEppDiag.c
SUMMARY         Zebulon FPGA EPP function diagnostics
VERSION         %version: 1 %
UPDATE DATE     %date_modified: Apr 28 14:45 2009 %
PROGRAMMER      %created_by:    Jim Tu %

        Copyright 2009 Quanta Corporation. All Rights Reserved.

DESCRIPTION:
        This file has code to test EPP function of Zebulon FPGA

NOTES:

REFERENCE:

*******************************************************************************/

#include <linux/module.h>

#include <linux/kernel.h>
#include <linux/delay.h>

#include <commTypes.h>
#include <commRpt.h>
#include <commMEC.h>

#include "fpgaRegLib.h"
#include "fpgaEppDiag.h"

extern VOID    fpgaEppInit(VOID);
extern VOID    fpgaEppWriteByte(UINT32 addr, UINT8 data);
extern VOID    fpgaEppWriteDword(UINT32 addr, UINT32 data);
extern UINT8   fpgaEppReadByte(UINT32 addr);
extern UINT32  fpgaEppReadDword(UINT32 addr);
extern UINT32  fpgaEppWriteAddr(UINT8 val);
extern UINT8   fpgaEppReadAddr(VOID);
extern VOID    fpgaEppWriteData(UINT8 val);





/****************************/
/***  EPP diag procedure  ***/
/****************************/

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaEppDiag
*
* SUMMARY:      diagnostics FPGA-SAS2x36 EPP interface
*
* SCOPE:        public
*
* DESCRIPTION:  diagnostics FPGA-SAS2x36 EPP interface
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE: 	Pass
*				FALSE:	Fail
*/
INT32 fpgaEppDiag(VOID)
{
    INT32 			result = TRUE;
    UINT32 			val, temp;
    INT32 			i;
    STATIC UINT32 	testPattern[4] = {FPGA_EPP_TEST_PATTERN_0, FPGA_EPP_TEST_PATTERN_1,
										FPGA_EPP_TEST_PATTERN_2, FPGA_EPP_TEST_PATTERN_3};
    
	fpgaEppInit();
	
	//backup original value
	temp = fpgaEppReadDword(SAS2X36_RAM_TEST_OFFSET);
		
    for(i = 0; i < 4; i++)
	{	
        //write register
		fpgaEppWriteDword(SAS2X36_RAM_TEST_OFFSET, testPattern[i]);   
        mdelay(20);
		
        //verify register
        val = fpgaEppReadDword(SAS2X36_RAM_TEST_OFFSET);
	
        if(val != testPattern[i])
		{	//if the reading is not the same as test pattern
            commReportError(MEC_FPGA, FPGA_EPP_MEC, FPGA_EPP_ERROR, MSG_FPGA_EPP_ERROR, 0, 0, 0, 0);
            result = FALSE;
        }
    }
	
	//restore original value
	fpgaEppWriteDword(SAS2X36_RAM_TEST_OFFSET, temp);		            
	
    return result;
}

/************************/
/***  EPP debug tool  ***/
/************************/

VOID fpgaEppRegShow(UINT8 offset)
{
	fpgaEppWriteAddr(offset);
    printk("[FPGA EPP] register: 0x%02X \n", fpgaEppReadAddr());
}

VOID fpgaEppRegWrite(UINT8 offset, UINT8 data)
{
    UINT8 temp;

    fpgaEppWriteAddr(offset);
    temp = fpgaEppReadAddr();
    fpgaEppWriteData(data);
	printk("[FPGA EPP] register: 0x%02X \n", temp);
}

VOID fpgaEppByteShow(UINT32 base)
{
	printk("[FPGA EPP] offset:0x%08X -> data:0x%02X\n", base, fpgaEppReadByte(base));            
}
