/*******************************************************************************

NAME            fpgaI2cDiag.c
SUMMARY         Zebulon FPGA I2C function diagnostics
VERSION         %version: 1 %
UPDATE DATE     %date_modified: Apr 27 20:30 2009 %
PROGRAMMER      %created_by:    Jim Tu %

        Copyright 2009 Quanta Corporation. All Rights Reserved.

DESCRIPTION:
        This file has code to test I2C function of Zebulon FPGA
		Not include the I2C master controller in Zebulon
		
NOTES:

REFERENCE:

*******************************************************************************/
#include <linux/kernel.h>
#include <linux/i2c.h>
#include <linux/delay.h>
#include <asm/io.h>

#include <commTypes.h>
#include <commRpt.h>
#include <commI2cApi.h>
#include <commMEC.h>

#include "fpgaRegLib.h"
#include "fpgaI2cDiag.h"


/***************************************/
/**  I2C Bus Interface Diag Function  **/
/***************************************/

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaI2cBusDiag
*
* SUMMARY:      verify processor I2C funtion on FPGA
*
* SCOPE:        public
*
* DESCRIPTION:  verify processor I2C function on FPGA
*
* RESTRICTIONS:
* 
* RETURNS:     	
*/
INT32 fpgaI2cBusDiag(VOID)
{
    INT32 result = TRUE;
#if 0	//TBD
    UINT32 	Ibmr0Value;
    VOIDP 	Ibmr0Base;
    
    fpgaProcI2cReset();
    
    //disable hot swap chips
    fpgaRegisterWrite(FPGA_I2C_CONTROL_2, 0x00);
    
    Ibmr0Base = (VOIDP)ioremap(REGISTER_ADDRESS_BASE + IBMR0_OFFSET, 4);
    
    //set I2C bus DL and CLK
    result = fpgaRegisterWrite(FPGA_I2C_CONTROL_1, 0x00);
    mdelay(10);
    
	//verify processor register
    Ibmr0Value = __raw_readl(Ibmr0Base) & 0x03;
    if(Ibmr0Value != 0x00)
	{
        commReportError(MEC_FPGA, FPGA_I2C_MEC, FPGA_I2C_0_ERROR, MSG_FPGA_I2C_ERROR, 0, 0, 0, 0);
        result = FALSE;
    }
    
    fpgaProcI2cReset();
    
    result = fpgaRegisterWrite(FPGA_I2C_CONTROL_1, 0x03);  //release bus
    mdelay(15);
    
	//verify processor register
    Ibmr0Value = __raw_readl(Ibmr0Base) & 0x03;
    if(Ibmr0Value != 0x03)
	{
        commReportError(MEC_FPGA, FPGA_I2C_MEC, FPGA_I2C_0_ERROR, MSG_FPGA_I2C_ERROR, 0, 0, 0, 0);
        result = FALSE;
    }
    
	//free memory mapping
    iounmap(Ibmr0Base);
#endif
    return result;
}


/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaI2cHotSwapChipEnableDiag
*
* SUMMARY:      verify hot swap chip enable function
*
* SCOPE:        public
*
* DESCRIPTION:  verify hot swap chip enable function
*
* RESTRICTIONS:
* 
* RETURNS:     	
*/
INT32 fpgaI2cHotSwapChipEnableDiag(VOID)
{
    INT32 result = TRUE;

    //Enable hot swap chip 1
    fpgaI2cMdpl1Select();
    mdelay(15);
    
	//Verify hot swap chip 1 is ready
    if((fpgaRegisterRead(FPGA_I2C_STATUS) & 0x01) != I2C1_READY_BIT)
    {
        //commReportError(MEC_FPGA, FPGA_I2C_MEC, FPGA_I2C_HOT_SWAP_1_ERROR, 
		//				MSG_FPGA_I2C_HOT_SWAP_1_ERROR, 0, 0, 0, 0);
        printk("[FPGA I2C] I2C hot swap buffer 1 ready pin is not functional.\n");
		result = FALSE;
    }

	//Enable hot swap chip 2
	fpgaI2cMdpl2Select();
    mdelay(15);
	
	//Verify hot swap chip 2 is ready
	if((fpgaRegisterRead(FPGA_I2C_STATUS) & 0x02) != I2C2_READY_BIT)
	{
		//commReportError(MEC_FPGA, FPGA_I2C_MEC, FPGA_I2C_HOT_SWAP_2_ERROR, 
        //                MSG_FPGA_I2C_HOT_SWAP_2_ERROR, 0, 0, 0, 0);
		printk("[FPGA I2C] I2C hot swap buffer 2 ready pin is not functional.\n");
		result = FALSE;
	}

    return result;
}

/*********************************/
/**  Main Diagnostics Function  **/
/*********************************/
/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaI2cDiag
*
* SUMMARY:      verify FPGA I2C function
*
* SCOPE:        public
*
* DESCRIPTION:  verify FPGA I2C function
*
* RESTRICTIONS:
* 
* RETURNS:     	
*/
INT32 fpgaI2cDiag(VOID)
{
    //g_platform = MATTERHORN;	//Jim

    if(fpgaI2cBusDiag() & fpgaI2cHotSwapChipEnableDiag())
	{
        return TRUE;
    }
	else
	{
        return FALSE;
    }
}


/******************************/
/**  I2C Bus Tools Function  **/
/******************************/
/*******************************************************************************
* PROCEDURE
*
* NAME:         i2cDeviceRead
*
* SUMMARY:      read content from I2C device
*
* SCOPE:        public
*
* DESCRIPTION:  read content from I2C device
*
* RESTRICTIONS:
* 
* RETURNS:     	
*/
INT32 i2cDeviceRead
(
 UINT8 	addr, 
 UINT8 	offset, 
 UINT32 size
)
{
    struct i2c_msg 		msgIn[2];
    struct i2c_adapter 	*adap;
    UINT8  				bufIn[size], bufOut, i;
    INT32  				ret;
	
    /* set I2C adapter */
    adap = zebulonAdapter[ZEBULON_I2C_BUS_0];
	
	/* command write */
	bufOut = offset;
	
    msgIn[0].addr = addr;
    msgIn[0].flags = 0;
    msgIn[0].len = 1;
    msgIn[0].buf = &bufOut;

    /* read content */
    msgIn[1].addr = addr;
    msgIn[1].flags = I2C_M_RD;
    msgIn[1].len = size;
    msgIn[1].buf = bufIn;

    if((ret = i2c_transfer(adap, msgIn, 2)) != 2)
	{
		printk("[I2C] Device Read error! ret = %d \n", ret);
   	}
	else
	{
		for(i=0;i<size;i++)
		{
			if((i%8) == 0)
			{
				printk("\n");	
			}
			printk("0x%02X ", bufIn[i]);
		}
		printk("\n \n");   
	}
    return TRUE;
}

