/*******************************************************************************

NAME            fpgaProtectRegDiag.c
SUMMARY         Zebulon FPGA register protect function diagnostics
VERSION         %version: 1 %
UPDATE DATE     %date_modified: Apr 27 19:20 2009 %
PROGRAMMER      %created_by:    Jim Tu %

        Copyright 2009 Quanta Corporation. All Rights Reserved.

DESCRIPTION:
        This file has code to test register protect function of Zebulon FPGA

NOTES:

REFERENCE:

*******************************************************************************/
#include <linux/kernel.h>
#include <linux/delay.h>

#include <commTypes.h>
#include <commRpt.h>
#include <commMEC.h>

#include "fpgaRegLib.h"

/*******************************************************************************
* PROCEDURE
*
* NAME:         fpgaProtectedWriteRegisterDiag
*
* SUMMARY:      Test register protect function of Zebulom FPGA
*
* SCOPE:        public
*
* DESCRIPTION:  1.Lock register protect and write test pattern INT32o the 
				  protected register.
*				2.Verify if the register content has been modified.
*
* RESTRICTIONS:
* 
* RETURNS:      TRUE:	Pass
*				FALSE:	Fail
*/
INT32 fpgaProtectedWriteRegisterDiag(VOID)
{
	UINT8 	temp1, temp2, temp3, temp4;
	INT32 	result = TRUE;
	INT32 	resultTemp;
    
	//get original register content
	temp1 = fpgaRegisterRead(FPGA_PROTECTED_WRITE_1);
	temp2 = fpgaRegisterRead(FPGA_PROTECTED_WRITE_2);
	temp3 = fpgaRegisterRead(FPGA_PROTECTED_WRITE_3);
	temp4 = fpgaRegisterRead(FPGA_PROTECTED_WRITE_4);
	
	//modify protected reigster content 
	resultTemp = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_1, 0xff);
	resultTemp = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_2, 0xff);
	resultTemp = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_3, 0xff);
	resultTemp = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_4, 0xff);

	//verify if the protected register has been modified
    if(fpgaRegisterRead(FPGA_PROTECTED_WRITE_1) != temp1)
	{
        commReportError(MEC_FPGA, FPGA_PROTECTED_WRITE_REGISTER_MEC, FPGA_PROTECTED_WRITE_1_ERROR,
						MSG_FPGA_PROTECTED_WRITE_1_ERROR, 0, 0 ,0, 0);
		resultTemp = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_1, temp1);
        result = FALSE;
    }
    if(fpgaRegisterRead(FPGA_PROTECTED_WRITE_2) != temp2)
	{
        commReportError(MEC_FPGA, FPGA_PROTECTED_WRITE_REGISTER_MEC, FPGA_PROTECTED_WRITE_2_ERROR,
						MSG_FPGA_PROTECTED_WRITE_2_ERROR, 0, 0 ,0, 0);
		resultTemp = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_2, temp2);
        result = FALSE;
    }
    if(fpgaRegisterRead(FPGA_PROTECTED_WRITE_3) != temp3)
	{
        commReportError(MEC_FPGA, FPGA_PROTECTED_WRITE_REGISTER_MEC, FPGA_PROTECTED_WRITE_3_ERROR,
						MSG_FPGA_PROTECTED_WRITE_3_ERROR, 0, 0 ,0, 0);
		resultTemp = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_3, temp3);
        result = FALSE;
    }
	if(fpgaRegisterRead(FPGA_PROTECTED_WRITE_4) != temp4)
	{
        commReportError(MEC_FPGA, FPGA_PROTECTED_WRITE_REGISTER_MEC, FPGA_PROTECTED_WRITE_4_ERROR,
						MSG_FPGA_PROTECTED_WRITE_4_ERROR, 0, 0 ,0, 0);
		resultTemp = fpgaRegisterWrite(FPGA_PROTECTED_WRITE_4, temp4);
        result = FALSE;
    }
	
    return result;
}
