#include "BBUDAPI.hpp"



BBUDAPI :: BBUDAPI(STRING className) : BaseClassDAPI(className)
{
   int i;

   //initi all value
   m_count = 0;
   for (i=0; i<BBU_NUMBER; i++)
   {
      m_bbu[i] = NULL;
   }

   scan_i2c_BBU();
}



BBUDAPI :: ~BBUDAPI()
{
   int i;

   for (i=0; i<BBU_NUMBER; i++)
   {
      if (m_bbu[i] != NULL)
      {
         delete m_bbu[i];
      }
   }
}



ErrorMsg*
BBUDAPI :: scan_i2c_BBU()
{
   I2CDAPI   my_i2c_dapi("Temp");
   I2CDevice *temp_i2c_device[DEVICE_NUMBER];
   int i;


   my_i2c_dapi.DisplayI2cDevicesListByType(temp_i2c_device, BATTERY);

   for (i=0; temp_i2c_device[i]; i++)
   {
      /*vender = temp_i2c_device[i]->get_vender();
      model = temp_i2c_device[i]->get_model();
      printf("name:%s  vender:%s  model:%s\n",temp_i2c_device[i]->get_Name(), vender.c_str(), model.c_str());*/
      Add_I2C_BBU_item(temp_i2c_device[i]);
   }

   //Delete the non-NULL point
   for (i=0; i<DEVICE_NUMBER; i++)
   {
      if (temp_i2c_device[i] != NULL)
      {
         delete temp_i2c_device[i];
      }
   }

   return NO_ERROR_MSG;
}



ErrorMsg*
BBUDAPI :: Add_I2C_BBU_item(I2CDevice *i2c_device)
{
   string name, vender, model;

   name = i2c_device->get_Name();
   vender = i2c_device->get_vender();
   model = i2c_device->get_model();
   //printf("Name:%s  vender:%s   model:%s\n",name.c_str(), vender.c_str(), model.c_str());

   if (vender.compare("TI") == 0 && model.compare("bq40z50") == 0)
   {
      m_bbu[m_count] = new BBU_I2C_TI_bq40z50;
      m_bbu[m_count]->Set_name(name);
      //m_bbu[m_count]->Set_type(TEMPERATURE_I2C);
      m_count++;
   }

   return NO_ERROR_MSG;
}



ErrorMsg*
BBUDAPI :: Get_BBU_List(BBUBaseClassDDAPI BBU_list[], int *count)
{
   int i;

   for (i=0; i<m_count; i++)
   {
      BBU_list[i].Set_name(m_bbu[i]->Get_name());
   }

   *count = m_count;

   return NO_ERROR_MSG;
}



ErrorMsg*
BBUDAPI :: Get_BBU_List(list<string> *BBU_list)
{
   int      i;
   string   device_result;

   for (i=0; i<m_count; i++)
   {
      device_result.clear();
      //BBU_list[i].Set_name(m_bbu[i]->Get_name());
      device_result = m_bbu[i]->Get_name();

      BBU_list->push_back(device_result);
   }

   return NO_ERROR_MSG;
}



ErrorMsg*
BBUDAPI :: Get_BBU_Device(string bbu_name, BBUBaseClassDDAPI **BBU)
{
   int    i;
   string get_name;

   for (i=0; i<m_count; i++)
   {
      get_name = m_bbu[i]->Get_name();
      if (bbu_name.compare(get_name) == 0)
      {
         *BBU = m_bbu[i];
         return NO_ERROR_MSG;
      }
   }

   if (i == m_count)
   {
      return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), "There is no match name.", NULL);
   }

   return NO_ERROR_MSG;
}



ErrorMsg*
BBUDAPI :: Charge(string bbu_name)
{
   int i;

   for (i=0; i<m_count; i++)
   {
      if ( bbu_name.compare(m_bbu[i]->Get_name()) == 0 )
      {
         m_errorCode = m_bbu[i]->Charge();

         if (m_errorCode != NO_ERROR_MSG)
         {
            return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
         }
         else
         {
            return NO_ERROR_MSG;
         }
      }
   }

   if (i == m_count)
   {
      return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), "There is no match name.", NULL);
   }

   return NO_ERROR_MSG;
}



ErrorMsg*
BBUDAPI :: Discharge(string bbu_name)
{
   int i;

   for (i=0; i<m_count; i++)
   {
      if ( bbu_name.compare(m_bbu[i]->Get_name()) == 0 )
      {
         m_errorCode = m_bbu[i]->Discharge();

         if (m_errorCode != NO_ERROR_MSG)
         {
            return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
         }
         else
         {
            return NO_ERROR_MSG;
         }
      }
   }

   if (i == m_count)
   {
      return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), "There is no match name.", NULL);
   }

   return NO_ERROR_MSG;
}



ErrorMsg*
BBUDAPI :: GetChargeStatus(string bbu_name, int *status)
{
   int i;

   for (i=0; i<m_count; i++)
   {
      if ( bbu_name.compare(m_bbu[i]->Get_name()) == 0 )
      {
         m_errorCode = m_bbu[i]->ShowChargeStatus(status);

         if (m_errorCode != NO_ERROR_MSG)
         {
            return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
         }
         else
         {
            return NO_ERROR_MSG;
         }
      }
   }

   if (i == m_count)
   {
      return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), "There is no match name.", NULL);
   }

   return NO_ERROR_MSG;
}



ErrorMsg*
BBUDAPI :: Read(string bbu_name, int start_address, int end_address, int *data, int *count)
{
   int i;

   for (i=0; i<m_count; i++)
   {
      if ( bbu_name.compare(m_bbu[i]->Get_name()) == 0 )
      {
         m_errorCode = m_bbu[i]->Read(bbu_name, start_address, end_address, data, count);

         if (m_errorCode != NO_ERROR_MSG)
         {
            return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
         }
         else
         {
            return NO_ERROR_MSG;
         }
      }
   }

   if (i == m_count)
   {
      return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), "There is no match name.", NULL);
   }

   return NO_ERROR_MSG;

}








ErrorMsg*
BBUDAPI ::  Write(string bbu_name, int address, int *data, int count, int *numberRealWrite)
{
   int i;

   for (i=0; i<m_count; i++)
   {
      if ( bbu_name.compare(m_bbu[i]->Get_name()) == 0 )
      {
         m_errorCode = m_bbu[i]->Write(bbu_name, address, data, count, numberRealWrite);

         if (m_errorCode != NO_ERROR_MSG)
         {
            return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
         }
         else
         {
            return NO_ERROR_MSG;
         }
      }
   }

   if (i == m_count)
   {
      return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), "There is no match name.", NULL);
   }

   return NO_ERROR_MSG;

}



ErrorMsg*
BBUDAPI :: getBBUInformation(string bbuName)
{
    int    i;
    float  float_data;
    int    int_data;
    string s, temperature, voltage, current, charging_current, max_error, battery_status, specification_info;
    string manufacture_date, manufacturer_name, device_name, device_chemistry, manufacturer_data;
    int    relative_state_charge, absolute_state_charge, remaining_capacity, full_charge_capacity;
    int    run_time_empty, average_time_empty, average_time_full, cycle_count, design_capacity, design_voltage;
    int    serial_number;
    stringstream ss(s);

    for (i=0; i<m_count; i++)
    {
       if ( bbuName == m_bbu[i]->Get_name())
       {
          //Get Temperature
          m_errorCode = m_bbu[i]->Get_Temperature(&float_data);;
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }
          ss << float_data;
          temperature = ss.str();
          ss.str("");

          //Get Voltage
          m_errorCode = m_bbu[i]->Get_Voltage(&int_data);;
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }
          ss << int_data;
          voltage = ss.str();
          ss.str("");

          //Get Current
          m_errorCode = m_bbu[i]->Get_Current(&int_data);
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }
          ss << int_data;
          current = ss.str();
          ss.str("");

          //Get Charging Current
          m_errorCode = m_bbu[i]->Get_Charging_Current(&int_data);
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }
          ss << int_data;
          charging_current = ss.str();
          ss.str("");

          //Get Max Error
          m_errorCode = m_bbu[i]->Get_Max_Error(&int_data);
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }
          ss << int_data;
          max_error = ss.str();
          ss.str("");

          //Get battery status
          m_errorCode = m_bbu[i]->Get_FW_BBU_State(&int_data);
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }
          ss << uppercase << hex << int_data;
          battery_status = "0x" + ss.str();
          ss.str("");

          //Get Relative State of Charge
          m_errorCode = m_bbu[i]->Get_Relative_State_Charge(&relative_state_charge);
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }

          //Get Absolute State Of Charge
          m_errorCode = m_bbu[i]->Get_Absolute_State_Charge(&absolute_state_charge);
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }

          //Get remaining capacity
          m_errorCode = m_bbu[i]->Get_Remaining_Capacity(&remaining_capacity);
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }

          //Get Full Charge Capacity
          m_errorCode = m_bbu[i]->Get_Full_Charge_Capacity(&full_charge_capacity);
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }

          //Get Run Time To Empty
          m_errorCode = m_bbu[i]->Get_Time_Empty(&run_time_empty);
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }

          //Get Average Time To Empty
          m_errorCode = m_bbu[i]->Get_Average_Time_Empty(&average_time_empty);
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }

          //Get Average Time To Full
          m_errorCode = m_bbu[i]->Get_Average_Time_Full(&average_time_full);
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }

          //Get Cycle Count
          m_errorCode = m_bbu[i]->Get_Cycle_Count(&cycle_count);
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }

          //Get Design Capacity
          m_errorCode = m_bbu[i]->Get_Design_Capacity(&design_capacity);
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }

          //Get Design Voltage
          m_errorCode = m_bbu[i]->Get_Design_Voltage(&design_voltage);
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }

          //Get Specification Info
          m_errorCode = m_bbu[i]->Get_Specification_Info(&int_data);
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }
          ss << uppercase << hex << int_data;
          specification_info = "0x" + ss.str();
          ss.str("");

          //Get Specification Info
          m_errorCode = m_bbu[i]->Get_Date_Manufacture(&manufacture_date);
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }

          //Get Serial Number
          m_errorCode = m_bbu[i]->Get_Serial_Number(&serial_number);
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }

          //Get Manufacturer Name
          m_errorCode = m_bbu[i]->Get_Manufacturer_Name(&manufacturer_name);
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }

          //Get Device Name
          m_errorCode = m_bbu[i]->Get_Device_name(&device_name);
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }

          //Get Device Chemistry
          m_errorCode = m_bbu[i]->Get_Device_Chemistry(&device_chemistry);
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }

          //Get Manufacturer Data
          m_errorCode = m_bbu[i]->Get_Design_Data(&manufacturer_data);
          if (m_errorCode != NO_ERROR_MSG)
          {
             return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), NULL, m_errorCode);
          }


          cout << "*** INTELLIGENT BATTERY MONITORED INFORMATION ***" << endl;
          cout << "_________________________________________________________________" << endl;
          cout << setw(21) << setiosflags(ios::left) << "Temperature" << ":" << setw(14) << temperature + " C"  << setw(20) << "Voltage" << ":" << voltage << " mV" << endl;
          cout << setw(21) << setiosflags(ios::left) << "Current" << ":" << setw(14) << current + " mA" << setw(20) << "Charging Current" << ":" << charging_current <<" mA" << endl;
          cout << setw(21) << setiosflags(ios::left) << "Max Error" << ":" << setw(14) << max_error + " %" << setw(20) << "GasGaugeStat" << ":" << "N/A" << endl;
          cout << setw(21) << setiosflags(ios::left) << "Charger System State" << ":" << setw(14) << battery_status << setw(20) << "Charger Sys Control" << ":" << "N/A" << endl;
          cout << setw(21) << setiosflags(ios::left) << "FWbbuState" << ":" << setw(14) << "N/A" << endl;
          cout << "_________________________________________________________________" << endl << endl << endl;

          cout << "*** BATTERY CAPACITY INFORMATION ***" << endl;
          cout << "_________________________________________________________________" << endl;
          cout << setw(25) << setiosflags(ios::left) << "Relative State of Charge" << ":" << relative_state_charge << " %" << endl;
          cout << setw(25) << setiosflags(ios::left) << "Absolute State of Charge" << ":" << absolute_state_charge << " %" << endl;
          cout << setw(25) << setiosflags(ios::left) << "Remaining Capacity" << ":" << remaining_capacity << " mW" << endl;
          cout << setw(25) << setiosflags(ios::left) << "Full Charge Capacity" << ":" << full_charge_capacity << " mW" << endl;
          cout << setw(25) << setiosflags(ios::left) << "Time to Empty" << ":" << run_time_empty << " Min" << endl;
          cout << setw(25) << setiosflags(ios::left) << "Average Time to Empty" << ":" << average_time_empty << " Min" << endl;
          cout << setw(25) << setiosflags(ios::left) << "Average Time to Full" << ":" << average_time_full << " Min" << endl;
          cout << setw(25) << setiosflags(ios::left) << "Cycle Count" << ":" << cycle_count << endl;
          cout << setw(25) << setiosflags(ios::left) << "Max Error" << ":" << max_error << " %" << endl;
          cout << "_________________________________________________________________" << endl << endl << endl;

          cout << "*** BATTERY DESIGN INFORMATION ***" << endl;
          cout << "_________________________________________________________________" << endl;
          cout << setw(25) << setiosflags(ios::left) << "Design Capacity" << ":" << design_capacity << " mWh" << endl;
          cout << setw(25) << setiosflags(ios::left) << "Design Voltage" << ":" << design_voltage << " mV" << endl;
          cout << setw(25) << setiosflags(ios::left) << "Specification Info" << ":" << specification_info << " " << endl;
          cout << setw(25) << setiosflags(ios::left) << "Date of Manufacture" << ":" << manufacture_date << endl;
          cout << setw(25) << setiosflags(ios::left) << "Serial Number" << ":" << serial_number << endl;
          cout << setw(25) << setiosflags(ios::left) << "Pack Status-Config" << ":" << "N/A" << endl;
          cout << setw(25) << setiosflags(ios::left) << "Manufacturer Name" << ":" << manufacturer_name << endl;
          cout << setw(25) << setiosflags(ios::left) << "Device Name" << ":" << device_name << endl;
          cout << setw(25) << setiosflags(ios::left) << "Device Chemistry" << ":" << device_chemistry << endl;
          cout << setw(25) << setiosflags(ios::left) << "Design Data" << ":" << manufacturer_data << endl;

          break;
       }
    }

    if (i == m_count)
    {
       return returnErrorMsg((ERROR_TEMPERATURE_DAPI_GET_CURRENT_TEMPERATURE), "There is no match name.", NULL);
    }

    return NO_ERROR_MSG;
}