#include "PCIDAPI.hpp"

PCIDAPI::PCIDAPI(STRING className) : BaseClassDAPI(className)
{
    m_pciDriverAPI = new PCIDDAPI("PCIDDAPI");
}

PCIDAPI::~PCIDAPI()
{
    if(m_pciDriverAPI != NULL)
    {
        delete m_pciDriverAPI;
        m_pciDriverAPI = NULL;
    }
}

ErrorMsg*  
PCIDAPI::showSpecificErrorStatistics(PFA               in_bdf,
                                     PCI_ERROR_TYPE    in_pciError,
                                     bool              in_showSummary)
{
    UINT32  u32ErrorStatistics = 0x0;
    string  strErrorType;

    list<PCI_ERROR>             pciErrorList;
    list<PCI_ERROR>::iterator   it;
  
    m_pErrorMsg = getErrorStatistics(in_bdf, in_pciError, &pciErrorList);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PCI_DAPI_SHOW_ERROR , NULL, m_pErrorMsg);
    }

    switch(in_pciError)
    {
        case PCI_COMPATIBLE_ERROR:
            in_showSummary ? strErrorType = "PCI-Compatible Error Statistics" : strErrorType = "==== PCI-compatible ====";
            break;
        case PCIE_ERROR:
            in_showSummary ? strErrorType = "PCIE Error Statistics" : strErrorType = "==== PCIE ====";
            break;
        case AER_ERROR:
            in_showSummary ? strErrorType = "PCIE AER Statistics" : strErrorType = "==== AER ====";
            break;
        default:
            break;
    }

    if(in_showSummary)
    {
        for(it = pciErrorList.begin(); it != pciErrorList.end(); ++it)
        {
            u32ErrorStatistics = u32ErrorStatistics + it->u32Data;
        }
        cout << strErrorType << ": "<< "0x" << u32ErrorStatistics << endl;
    }
    else
    {
        cout << strErrorType << endl;
        if(in_pciError == AER_ERROR)
        {
            for(it = pciErrorList.begin(); it != pciErrorList.end(); ++it)
            {
                printf("%s: 0x%08x\n",it->strErrorName.c_str(), it->u32Data);
            }
        }
        else 
        {
            for(it = pciErrorList.begin(); it != pciErrorList.end(); ++it)
            {
                cout << it->strErrorName << ": " << it->u32Data << endl;
            }
        }
    }

    return NO_ERROR_MSG;
}

    
bool 
PCIDAPI::isPCIe(PFA  in_bdf)
{
    return m_pciDriverAPI->isPCIe(in_bdf);
}


void
PCIDAPI::progressHandler(float in_u32DWordCount)
{
    int     s32Progress = 0;
    UINT32  u32Index;

    //printf("PCIDAPI::progressHandler, %.f\n", in_u32DWordCount);//check 
    for(u32Index = 0; u32Index < (UINT32)in_u32DWordCount; u32Index++)
    {
        usleep(10000);
        s32Progress = (u32Index/in_u32DWordCount) * 100;
        //s32Progress += getProgress()/100;
        //printf("PCIDAPI::progressHandler, s32Progress: %d\n", s32Progress);//check 
        printf("progress:%3d%%\r", s32Progress);
        fflush(stdout);
    }

    printf("Progressing: 100\%\n");
    printf("Completed!\n");
}



ErrorMsg* 
PCIDAPI::readConfig(PFA     in_bdf,
                    UINT32  in_u32StartOffset,
                    UINT32  in_u32EndOffset,
                    UINT8*  in_u8pBuffer)
{
    m_pciDriverAPI->ConfigReadByte(in_bdf, in_u32StartOffset, in_u32EndOffset, in_u8pBuffer);
    
    return NO_ERROR_MSG;
}
                    

ErrorMsg*
PCIDAPI::clearErrors(PFA  in_bdf)
{
    m_eErrorCode = m_pciDriverAPI->clearErrors(in_bdf);
    if(m_eErrorCode != NO_ERROR)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PCI_DAPI_RESET_STATISTICS | m_eErrorCode), NULL, NULL);
    }
    return NO_ERROR_MSG;
}



ErrorMsg*
PCIDAPI::getErrorStatistics(PFA                 in_bdf,
                            PCI_ERROR_TYPE      in_pciError,
                            list<PCI_ERROR>*    out_pErrorList)
{
    ERRORCODE  eErrorCode;

    //eErrorCode = m_pciDriverAPI->checkPresenceOfDevice(in_bdf);
    //if(eErrorCode != NO_ERROR)
    //{
    //    //return error 
    //}

    eErrorCode = m_pciDriverAPI->getErrorStatistics(in_bdf, in_pciError, out_pErrorList);
    if(eErrorCode != NO_ERROR)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PCI_DAPI_ERROR_REPORT | eErrorCode), NULL, NULL);
    }
    return NO_ERROR_MSG;
}

bool
PCIDAPI::isNtbDevice(RetrainInfo in_retrainInfo)
{
    return m_pciDriverAPI->isNtbDevice(in_retrainInfo);
}


void
PCIDAPI::getRetrainSupportedDeviceList(vector<RetrainInfo>*  out_retrainDeviceList)
{
    m_pciDriverAPI->getRetrainSupportDevices(out_retrainDeviceList);
}


ErrorMsg*
PCIDAPI::convertStringToClassCode(string    in_ClassString,
                                  UINT32*   out_u32ClassCode)
{
    ERRORCODE eErrorCode;

    eErrorCode = m_pciDriverAPI->convertStringToClassCode(in_ClassString, out_u32ClassCode);
    if(eErrorCode != NO_ERROR)
    {
        return returnErrorMsg(ERROR_PCI_DAPI_GET_CLASS, "invalid class string", NULL);
    }
    return NO_ERROR_MSG;
}


ErrorMsg*
PCIDAPI::getDevices(vector<PCIInformantion>*    out_pPciList)
{
    m_pciDriverAPI->getDeviceList(out_pPciList, SORT_PCI_DEVICES);
    m_pciDriverAPI->getDevicesName(out_pPciList);

    return NO_ERROR_MSG;
}


void 
PCIDAPI::DisplayPCIeSwitchEepromList()
{
    vector<EEPROM_INFO>               eepromList;
    vector<EEPROM_INFO>::iterator     it;
    
    m_pciDriverAPI->scanEeproms(&eepromList);
    for(it = eepromList.begin(); it != eepromList.end(); ++it)
    {
        transform(it->strEepromName.begin(), it->strEepromName.end(), it->strEepromName.begin(), ::toupper);
        printf("%s (attached to PCIE switch %04x:%02x:%02x.%x)\n", 
                it->strEepromName.c_str(), it->u16Domain, it->u16Bus, it->u16Device, it->u8Function);
    }
}


ERRORCODE
PCIDAPI::checkPresenceOfDevice(PFA  in_bdf)
{
    return m_pciDriverAPI->checkPresenceOfDevice(in_bdf); 
}


//This function assumes that the selected device is valid,
//so please check device using checkPresenceOfDevice function before using this function.
float
PCIDAPI::GetReadableSpeed(PFA           in_bdf,
                          DATA_TYPE     in_eDataType)
{
    return m_pciDriverAPI->GetFloatDataByID(in_bdf, in_eDataType);
}


//This function assumes that the selected device is valid,
//so please check device using checkPresenceOfDevice function before using this function.
UINT32
PCIDAPI::GetDataByBDF(PFA        in_bdf,
                      DATA_TYPE  in_eDataType)
{
    return m_pciDriverAPI->GetDataByBDF(in_bdf, in_eDataType);
}



//This function assumes that the logicalID is valid,
//so please check logicalID before using it.
string 
PCIDAPI::getReadableStringByBDF(PFA         in_bdf,
                                DATA_TYPE   in_eDataType)
{
    switch(in_eDataType) 
    {
        case PCI_DATA_TYPE_CLASS_TYPE:
            return m_pciDriverAPI->getClassTypeString(in_bdf);
        case PCI_DATA_TYPE_PORT_TYPE:
            return m_pciDriverAPI->getPortTypeString(in_bdf);
        case PCI_DATA_TYPE_LINK_STATUS:
            return m_pciDriverAPI->getLinkStatus(in_bdf);
        default:
            return "null";
    }
}

void 
PCIDAPI::writeConfig(PFA        in_bdf,
                     UINT32     in_u32StartOffset,
                     UINT8*     in_u8pBuffer,
                     UINT32     in_u32BufferSize)
{
    m_pciDriverAPI->ConfigWriteByte(in_bdf, in_u32StartOffset, in_u8pBuffer, in_u32BufferSize);
}

void 
PCIDAPI::ConfigWriteByte(PFA        in_bdf,
                         UINT32     in_u32Offset,
                         UINT8      in_u8WriteData)
{
    m_pciDriverAPI->ConfigWriteByte(in_bdf, in_u32Offset, in_u8WriteData);
}


void 
PCIDAPI::ConfigWriteWord(PFA        in_bdf,
                         UINT32     in_u32Offset,
                         UINT16     in_u16WriteData)
{
    m_pciDriverAPI->ConfigWriteWord(in_bdf, in_u32Offset, in_u16WriteData);
}


void 
PCIDAPI::ConfigWriteDword(PFA       in_bdf,
                          UINT32    in_u32Offset,
                          UINT32    in_u32WriteData)
{
    m_pciDriverAPI->ConfigWriteDword(in_bdf, in_u32Offset, in_u32WriteData);
}


ErrorMsg*
PCIDAPI::testEeprom(string  in_strEepromName)
{
    EEPROM_INFO     eepromInfo;
    PFA             bdf;

    m_pErrorMsg = m_pciDriverAPI->getEeprom(in_strEepromName, &eepromInfo);
    if(m_pErrorMsg == NO_ERROR_MSG)
    {
        bdf.u16Domain   = eepromInfo.u16Domain;
        bdf.u16Bus      = eepromInfo.u16Bus;
        bdf.u16Device   = eepromInfo.u16Device;
        bdf.u8Function  = eepromInfo.u8Function;

        m_pErrorMsg = m_pciDriverAPI->testEeprom(bdf);
    }
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PCI_DAPI_EEPROM_TEST, NULL, m_pErrorMsg);
    }
    return m_pErrorMsg;
}


ErrorMsg*
PCIDAPI::saveEeprom(string  in_strEepromName,
                    STRING  in_filePath)
{
    FILE*           fp;
    UINT32          u32Address;
    UINT32          u32ByteCount;
    UINT32          u32DWordCount;
    UINT32*         u32pBuffer = NULL;
    bool            bRunPass = true;
    thread*         pProgressHandler = NULL;
    int             s32Progress = 0;

    //get eeprom image size, eeprom memory base address(defined in DDAPI) is also set 
    m_pErrorMsg = getEepromSize(in_strEepromName, &u32ByteCount);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PCI_DAPI_SAVE_EEPROM, NULL, m_pErrorMsg);
    }
    u32DWordCount = u32ByteCount/4;
    //printf("PCIDAPI::saveEeprom, u32DWordCount: %d\n", u32DWordCount);//check 

    fp = fopen(in_filePath, "wb");
    if(fp == NULL)
    {
        return returnErrorMsg(ERROR_PCI_DAPI_SAVE_EEPROM, "open file fail", NULL);
    }

    u32pBuffer = (UINT32*)malloc(sizeof(UINT32) * u32DWordCount);

    //u32DWordCount = 5; //check 
    pProgressHandler = new thread(&PCIDAPI::progressHandler, this, (float)u32DWordCount);

    //initProgress(u32DWordCount);
    for(u32Address = 0; u32Address < u32DWordCount; u32Address++)
    {
        m_pErrorMsg = m_pciDriverAPI->readEepromDword(u32Address, &u32pBuffer[u32Address]);
        if(m_pErrorMsg != NO_ERROR_MSG)
        {
            bRunPass = false;
            break;
        }
        //s32Progress = (u32Address/10.0) * 100;
        //printf("progress:%3d%%\r", s32Progress);
        //fflush(stdout);
        //updateProgress(u32Address);
    }
    
    if(bRunPass)
    {
        //save data to file
        fwrite(u32pBuffer, sizeof(UINT32), u32DWordCount, fp);
    }

    fclose(fp);
    free(u32pBuffer);

    pProgressHandler->join();
    delete pProgressHandler;

    if(!bRunPass)
    {
        return returnErrorMsg(ERROR_PCI_DAPI_SAVE_EEPROM, NULL, m_pErrorMsg);
    }      
    return NO_ERROR_MSG;
}

ErrorMsg*
PCIDAPI::programEeprom(string  in_strEepromName,
                       string  in_filePath)
{
    FILE*           fp;
    UINT32          u32FileSize;
    UINT32          u32Address;
    UINT32          u32ByteCount;
    UINT32          u32DWordCount;
    UINT32*         u32pBuffer = NULL;
    thread*         pProgressHandler = NULL;
    int             s32Progress = 0;
    PFA             bdf;
    EEPROM_INFO     eepromInfo;
    UINT32          u32Remainder;

    m_pErrorMsg = m_pciDriverAPI->getEeprom(in_strEepromName, &eepromInfo);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PCI_DAPI_PROGRAM_EEPROM, NULL, m_pErrorMsg);
    }
    bdf.u16Domain   = eepromInfo.u16Domain;
    bdf.u16Bus      = eepromInfo.u16Bus;
    bdf.u16Device   = eepromInfo.u16Device;
    bdf.u8Function  = eepromInfo.u8Function;
   
    //get Image from in_filePath 
    fp = fopen(in_filePath.c_str(), "r");
    if(fp == NULL)
    {
        return returnErrorMsg(ERROR_PCI_DAPI_PROGRAM_EEPROM, "open file fail", NULL);
    }
    fseek(fp, 0, SEEK_END); //move to end-of-file
    u32FileSize = ftell(fp);
    //printf("u32FileSize: %d\n", u32FileSize);//check 
    fseek(fp, 0, SEEK_SET); //move back to start of file
    u32DWordCount = u32FileSize/4;
    if(u32FileSize % 4 != 0)
    {
        u32DWordCount = u32DWordCount + 1;
    }
    //printf("u32DWordCount: %d\n", u32DWordCount);//check

    u32pBuffer = (UINT32*)malloc(sizeof(UINT32) * u32DWordCount);

    //read data from file 
    fread(u32pBuffer, sizeof(UINT32), u32DWordCount, fp);
    fclose(fp);
   
#if 0
    //check 
    for(UINT32 u32Index = 0; u32Index < u32DWordCount; u32Index++)
    {
        if(u32Index % 4 == 0)
        {
            printf("\n");
        }
        printf("%08x ", u32pBuffer[u32Index]);
    }
    printf("\n");
#endif 

    m_pciDriverAPI->setEepromBase(bdf);

    //u32DWordCount = 5; //check 
    pProgressHandler = new thread(&PCIDAPI::progressHandler, this, (float)u32DWordCount);

    //initProgress(u32DWordCount);
    for(u32Address = 0; u32Address < u32DWordCount; u32Address++)
    {
        m_pErrorMsg = m_pciDriverAPI->writeEepromDword(u32Address, u32pBuffer[u32Address]);
        if(m_pErrorMsg != NO_ERROR_MSG)
        {
            break;
        }
        //s32Progress = (u32Address/10.0) * 100;
        //printf("progress:%3d%%\r", s32Progress);
        //fflush(stdout);
        //updateProgress(u32Address);
    }
    
    free(u32pBuffer);

    pProgressHandler->join();
    delete pProgressHandler;

    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PCI_DAPI_PROGRAM_EEPROM, NULL, m_pErrorMsg);
    }      
    return NO_ERROR_MSG;
}


ErrorMsg* 
PCIDAPI::showEepromImageVersion(string   in_strEepromName)
{
    ErrorMsg*   pErrorMsg;
    
    pErrorMsg = m_pciDriverAPI->showEepromImageVersion(in_strEepromName);
    return pErrorMsg;
}

ErrorMsg*
PCIDAPI::getEepromSize(string   in_strEepromName,
                       UINT32*  out_u32EepromSize)
{
    EEPROM_INFO     eepromInfo;
    ErrorMsg*       pErrorMsg;
    PFA             bdf;

    pErrorMsg = m_pciDriverAPI->getEeprom(in_strEepromName, &eepromInfo);
    if(pErrorMsg != NO_ERROR_MSG)
    {
        return pErrorMsg;
    }
  
    bdf.u16Domain   = eepromInfo.u16Domain;
    bdf.u16Bus      = eepromInfo.u16Bus;
    bdf.u16Device   = eepromInfo.u16Device;
    bdf.u8Function  = eepromInfo.u8Function;

    pErrorMsg = m_pciDriverAPI->getEepromSize(bdf, out_u32EepromSize);
    if(pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PCI_DAPI_GET_EEPROM_SIZE, NULL, m_pErrorMsg);
    }
    return pErrorMsg;
}


ErrorMsg*
PCIDAPI::readEeprom(string      in_strEepromName,
                    UINT32      in_u32Address,
                    UINT32      in_u32ReadSize,
                    UINT32*     out_u32pData)
{
    PFA             bdf;
    EEPROM_INFO     eepromInfo;

    m_pErrorMsg = m_pciDriverAPI->getEeprom(in_strEepromName, &eepromInfo);
    if(m_pErrorMsg == NO_ERROR_MSG)
    {
        bdf.u16Domain   = eepromInfo.u16Domain;
        bdf.u16Bus      = eepromInfo.u16Bus;
        bdf.u16Device   = eepromInfo.u16Device;
        bdf.u8Function  = eepromInfo.u8Function;
        
        m_pErrorMsg = m_pciDriverAPI->readEeprom(bdf, in_u32Address, in_u32ReadSize, out_u32pData);
    }

    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PCI_DAPI_READ_EEPROM, NULL, m_pErrorMsg);
    }
    return NO_ERROR_MSG;
}



ErrorMsg*
PCIDAPI::writeEeprom(string     in_strEepromName,
                     UINT32     in_u32Address,
                     UINT32     in_u32BufferSize, //DWord aligned
                     UINT32*    in_u32pWriteBuffer)
{
    PFA             bdf;
    EEPROM_INFO     eepromInfo;

    m_pErrorMsg = m_pciDriverAPI->getEeprom(in_strEepromName, &eepromInfo);
    if(m_pErrorMsg == NO_ERROR_MSG)
    {
        bdf.u16Domain   = eepromInfo.u16Domain;
        bdf.u16Bus      = eepromInfo.u16Bus;
        bdf.u16Device   = eepromInfo.u16Device;
        bdf.u8Function  = eepromInfo.u8Function;
        
        m_pErrorMsg = m_pciDriverAPI->writeEeprom(bdf, in_u32Address, in_u32BufferSize, in_u32pWriteBuffer);
    }

    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PCI_DAPI_WRITE_EEPROM, NULL, m_pErrorMsg);
    }
    return NO_ERROR_MSG;
}



ErrorMsg*
PCIDAPI::testRetrain(PFA  in_bdf)
{
    m_pErrorMsg = m_pciDriverAPI->testRetrain(in_bdf);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PCI_DAPI_RETRAIN, NULL, m_pErrorMsg);
    }
    return m_pErrorMsg;
}


void 
PCIDAPI::getHiddenDevices(vector<PCIInformantion>   in_pciList,
                          vector<PCIInformantion>   in_pciListIO,
                          vector<PCIInformantion>*  out_pPciList)
{
    bool    bDeviceFoundInSys;

    vector<PCIInformantion>::iterator  it;
    vector<PCIInformantion>::iterator  itIO;

    for(itIO = in_pciListIO.begin(); itIO != in_pciListIO.end(); ++itIO)
    {
        bDeviceFoundInSys = false;
        for(it = in_pciList.begin(); it != in_pciList.end(); ++it)
        {
            if(itIO->u16Domain == it->u16Domain && itIO->u16Bus == it->u16Bus &&
               itIO->u16Device == it->u16Device && itIO->u8Function == it->u8Function)
            {
                bDeviceFoundInSys = true;
            }
        }
        if(!bDeviceFoundInSys)
        {
            out_pPciList->push_back(*itIO);
        }
    }
}


ErrorMsg* 
PCIDAPI::scanHiddenDevices(UINT16                       in_u16Domain,
                           vector<PCIInformantion>*     out_pPciList)
{
    vector<PCIInformantion>  pciList;
    vector<PCIInformantion>  pciListIO;

    m_pErrorMsg = returnErrorMsg((ERRORCODE)(ERROR_PCI_DAPI_SCAN_DEVICE | ERROR_FUNCTION_NOT_SUPPORTED), NULL, NULL);
    
    if(in_u16Domain == 0)
    {
        //cout << "scan devices through sysfs ..." << endl;
        getDevices(&pciList);

        //cout << "scan devices through IO ..." << endl;
        m_pErrorMsg = m_pciDriverAPI->scanDevicesFromIO(&pciListIO);
        if(m_pErrorMsg == NO_ERROR_MSG)
        {
            getHiddenDevices(pciList, pciListIO, out_pPciList);
            m_pciDriverAPI->getDevicesName(out_pPciList);
        }
        else 
        {
            return returnErrorMsg(ERROR_PCI_DAPI_SCAN_DEVICE, NULL, m_pErrorMsg);
        }
    }
    
    return m_pErrorMsg;
}


ErrorMsg*
PCIDAPI::displayHBAs()
{
    UINT32  u32SlotIndex = 1;
    PFA     bdf;
    float   fSpeed;
    UINT32  u32Width; 
    char    charSpeed[16];
    char    charWidth[8];

    vector<SLOT_INFO>            slotList;
    vector<SLOT_INFO>::iterator  it;
    
    m_pciDriverAPI->scanSlot(&slotList);
    //check
    /*
    for(it = slotList.begin(); it != slotList.end(); it++)
    {
        printf("%x:%x.%x, LinkStatus: %s\n",
                it->u16Bus, it->u16Device, it->u8Function,
                it->bLinkStatus ? "Up" : "Down");
        printf("%x:%x.%x ------ %x:%x.%x [%s: %s]\n", 
                it->u16Bus, it->u16Device, it->u8Function,
                it->downStreamDevice.u16Bus, it->downStreamDevice.u16Device,
                it->downStreamDevice.u8Function,
                it->hbaName.c_str(), it->description.c_str());

    }
    */

    cout << "slot " << setw(10) << "speed" << setw(15) << "width" << setw(9) << "name" << setw(22) << "description" << endl;
    cout << "------------------------------------------------------------------------------" << endl;
    for(it = slotList.begin(); it != slotList.end(); ++it)
    {
        if(it->bLinkStatus)
        {
            bdf.u16Domain   = it->u16Domain;
            bdf.u16Bus      = it->u16Bus;
            bdf.u16Device   = it->u16Device;
            bdf.u8Function  = it->u8Function;
            fSpeed = GetReadableSpeed(bdf, PCI_DATA_TYPE_CURRENT_SPEED);
            sprintf(charSpeed, "%.1f(GT/s)", fSpeed);
            u32Width = GetDataByBDF(bdf, PCI_DATA_TYPE_CURRENT_WIDTH);
            sprintf(charWidth, "x%d", u32Width);

            cout << setw(10) << setiosflags(ios::left) << u32SlotIndex; 
            cout << setw(15) << setiosflags(ios::left) << charSpeed;
            cout << setw(10) << setiosflags(ios::left) << charWidth;
            cout << setw(15) << setiosflags(ios::left) << it->hbaName.c_str();
            cout << setw(15) << setiosflags(ios::left) << it->description.c_str();
            cout << "(" << it->slotType.c_str() << ")" << endl;
        }
        else
        {
            cout << setw(10) << setiosflags(ios::left) << u32SlotIndex;
            cout << setw(15) << setiosflags(ios::left) << "-";
            cout << setw(10) << setiosflags(ios::left) << "-";
            cout << setw(15) << setiosflags(ios::left) << "-";
            cout << setw(15) << setiosflags(ios::left) << "-" << endl;
        }
        
        u32SlotIndex++;
    }

    return NO_ERROR_MSG;    
}


void
PCIDAPI::showPciDefinedTypes()
{
    m_pciDriverAPI->showPciDefinedTypes();
}



void 
PCIDAPI::JustTest()
{
    UINT32                              u32Bar;
    vector<PCIInformantion>             pciInfoList;
    vector<PCIInformantion>::iterator   it;


    //m_pciDriverAPI->scanDevicesFromIO(0x0, &pciInfoList);

    ///*
    //m_pciDriverAPI->getPciInfoByClass(PCI_SAS, &pciInfoList);
    m_pciDriverAPI->getPciInfoByClass(PCI_ETH, &pciInfoList);

    for(it = pciInfoList.begin(); it != pciInfoList.end(); ++it)
    {
        printf("%04x:%02x:%02x.%x %s\n", 
                it->u16Domain, it->u16Bus, it->u16Device, it->u8Function, it->szDeviceName.c_str());

        ////get bar 
        //m_pciDriverAPI->getBarByBDF(it->u16Domain, it->u16Bus, it->u16Device, it->u8Function, 0, &u32Bar);
        //printf("Bar[0] -- 0x%x\n", u32Bar);
        //m_pciDriverAPI->getBarByBDF(it->u16Domain, it->u16Bus, it->u16Device, it->u8Function, 1, &u32Bar);
        //printf("Bar[1] -- 0x%x\n", u32Bar);
        //printf("\n-------------------------------------------------\n");
    }
    //*/
}





