#include "RTCDAPI.hpp"

RTCDAPI :: RTCDAPI(STRING className) : BaseClassDAPI(className)
{

}

RTCDAPI :: ~RTCDAPI()
{

}

ErrorMsg * RTCDAPI :: showDateTime(void)
{
    ErrorMsg *pErrorMsg;
    
    m_rtcDriver = new RTCDDAPI("RTCDDAPI");
    if ((pErrorMsg =  m_rtcDriver->getErrorMsg()) != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)ERROR_RTC_DAPI_SHOW_DATE_TIME, NULL, pErrorMsg);
    }

    if((pErrorMsg = m_rtcDriver->showDateTime()) != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)ERROR_RTC_DAPI_SHOW_DATE_TIME, NULL, pErrorMsg);
    }

    delete m_rtcDriver;
    m_rtcDriver = NULL;
    
    return NO_ERROR_MSG;
}

ErrorMsg * RTCDAPI :: setDate(INT32 year, INT32 month, INT32 date, STRING day)
{
    ErrorMsg    *pErrorMsg;
    INT32       maxDate;
    
    if ((strcmp(day, "MON") != 0) &&
        (strcmp(day, "TUE") != 0) &&
        (strcmp(day, "WED") != 0) &&
        (strcmp(day, "THU") != 0) &&
        (strcmp(day, "FRI") != 0) &&
        (strcmp(day, "SAT") != 0) &&
        (strcmp(day, "SUN") != 0))
    {
        sprintf(m_szTempString, "Day: %s\tDay is out of range!\n", day);
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DAPI_SET_DATE | ERROR_RTC_DAPI_DATA_OUT_OF_RANGE), m_szTempString, NULL);
    }

    // Month validation
    if ((month < 1) || (month > 12))
    {
        sprintf(m_szTempString, "Month: %d\tMonth is out of range!\n", month);
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DAPI_SET_DATE | ERROR_RTC_DAPI_DATA_OUT_OF_RANGE), m_szTempString, NULL);
    }

    // Year validation
    if ((year < 2000) || (year > 2099))
    {
        sprintf(m_szTempString, "Year: %d\tYear is out of range!\n", year);
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DAPI_SET_DATE | ERROR_RTC_DAPI_DATA_OUT_OF_RANGE), "Year is out of range!", NULL);
    }

    // Calculate the maximum date based upon the month parameter
    maxDate = 31;
    if ((month == 4) || (month == 6) || (month == 9) || (month == 11))
        maxDate = 30;
    if (month == 2)
    {
        maxDate = 28;
        if (((year % 4) == 0) && ((year % 100) != 0))
            maxDate = 29;
    }

    // Date validation
    if ((date < 1) || (date > maxDate))
    {       
        sprintf(m_szTempString, "Date: %d\tDate is out of range!\n", date);
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DAPI_SET_DATE | ERROR_RTC_DAPI_DATA_OUT_OF_RANGE), m_szTempString, NULL);
    }
    
    m_rtcDriver = new RTCDDAPI("RTCDDAPI");
    if ((pErrorMsg =  m_rtcDriver->getErrorMsg()) != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)ERROR_RTC_DAPI_SET_DATE, "Construct RTC DDAPI failed", pErrorMsg);
    }

    if ((pErrorMsg = m_rtcDriver->setDate(year, month, date)) != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)ERROR_RTC_DAPI_SET_DATE, NULL, pErrorMsg);
    }

    delete m_rtcDriver;
    m_rtcDriver = NULL;
    
    return NO_ERROR_MSG;
}

ErrorMsg * RTCDAPI :: setTime(INT32 hour, INT32 minute, INT32 second)
{
    ErrorMsg    *pErrorMsg;

    if(hour < 0 || hour > 23)
    {
        sprintf(m_szTempString, "Hour: %d\tHour is out of range!\n", hour);
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DAPI_SET_TIME | ERROR_RTC_DAPI_DATA_OUT_OF_RANGE), m_szTempString, NULL);
    }
        
    if(minute < 0 || minute > 59)
    {
        sprintf(m_szTempString, "Minute: %d\tMinute is out of range!\n", minute);
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DAPI_SET_TIME | ERROR_RTC_DAPI_DATA_OUT_OF_RANGE), m_szTempString, NULL);
    }
        
    if(second < 0 || second > 59)
    {
        sprintf(m_szTempString, "Second: %d\tSecond is out of range!\n", second);
        return returnErrorMsg((ERRORCODE)(ERROR_RTC_DAPI_SET_TIME | ERROR_RTC_DAPI_DATA_OUT_OF_RANGE), m_szTempString, NULL);
    }
    
    m_rtcDriver = new RTCDDAPI("RTCDDAPI");
    if ((pErrorMsg =  m_rtcDriver->getErrorMsg()) != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)ERROR_RTC_DAPI_SET_TIME, "Construct RTC DDAPI failed", pErrorMsg);
    }

    if((pErrorMsg = m_rtcDriver->setTime(hour, minute, second)) != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)ERROR_RTC_DAPI_SET_TIME, NULL, pErrorMsg);
    }

    delete m_rtcDriver;
    m_rtcDriver = NULL;
    
    return NO_ERROR_MSG;
}

ErrorMsg * RTCDAPI :: setDateTime(INT32 year, INT32 month, INT32 date, STRING day, INT32 hour, INT32 minute, INT32 second)
{
    ErrorMsg *pErrorMsg;
    
    if ((pErrorMsg = setDate(year, month, date, day)) == NO_ERROR_MSG)
    {
        pErrorMsg = setTime(hour, minute, second);
    }
    
    return pErrorMsg;
}

ErrorMsg * RTCDAPI :: rtcAccuracyTest(void)
{
    ErrorMsg    *pErrorMsg;
    BOOL        status;      
    INT32       lowerBound;
    INT32       upperBound;
    UINT32      timerCount;
    UINT32      currentCount;
    UINT32      testTime;
    INT32       diff1, diff2;

    //Setup
    testTime = TIEMER_ACCURACY_TEST_TIME;
    
    m_rtcDriver = new RTCDDAPI("RTCDDAPI");
    if ((pErrorMsg =  m_rtcDriver->getErrorMsg()) != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)ERROR_RTC_DAPI_RTC_ACCURACY_TEST, "Construct RTC DDAPI failed", pErrorMsg);
    }

    m_rtcDriver->convertGPTTimeToCount((FLOAT)testTime, &timerCount);
    
    // Initialize timer's configurations     
    pErrorMsg = m_rtcDriver->setRTCTimerRate((FLOAT)testTime);
    
    // Start RTC timer
    if (pErrorMsg == NO_ERROR_MSG)
        pErrorMsg = m_rtcDriver->startRTCTimer();                

    //Excecute
    // Wait for RTC's interrupt
    if (pErrorMsg == NO_ERROR_MSG)
        pErrorMsg = m_rtcDriver->readRTCTimerStatus(&status);

    if (pErrorMsg == NO_ERROR_MSG)  
    {
        // Start tested timer
        m_rtcDriver->startGPTTimer(timerCount, GPT_MODE3); 

        // Wait for RTC's interrupt
        pErrorMsg = m_rtcDriver->readRTCTimerStatus(&status);
        
        // Get current count of tested timer
        m_rtcDriver->readGPTTimer(&currentCount);
    }
    
    if (pErrorMsg == NO_ERROR_MSG)
    {
        upperBound = TIEMER_ACCURACY_TEST_MIDDLE_COUNT + TIEMER_ACCURACY_TEST_LIMIT;
        lowerBound = TIEMER_ACCURACY_TEST_MIDDLE_COUNT - TIEMER_ACCURACY_TEST_LIMIT;
        lowerBound += (INT32)timerCount;
        
        if (((INT32)currentCount < lowerBound) && (upperBound < (INT32)currentCount))
        {
            diff1 = lowerBound - currentCount;
            diff2 = currentCount - upperBound;
            
            if (diff1 > diff2)
            {
                sprintf(m_szTempString, "Count %d should be less than %d or greater than %d. Diff: %d\n", 
                        currentCount, upperBound, lowerBound, diff2); 
            }
            else
            {
                sprintf(m_szTempString, "Count %d should be less than %d or greater than %d. Diff: %d\n", 
                        currentCount, upperBound, lowerBound, diff1); 
            }
            
            //return returnErrorMsg((ERRORCODE)(ERROR_RTC_DAPI_RTC_ACCURACY_TEST | ERROR_RTC_DAPI_RTC_UNEXPECTED_TEST_RESULT), m_szTempString, NULL);
            addErrorMsg((ERRORCODE)(ERROR_RTC_DAPI_RTC_ACCURACY_TEST | ERROR_RTC_DAPI_RTC_UNEXPECTED_TEST_RESULT), m_szTempString, pErrorMsg);
        }
    }

    //Cleanup
    m_rtcDriver->stopRTCTimer();
    m_rtcDriver->stopGPTTimer();

    delete m_rtcDriver;
    m_rtcDriver = NULL;
    
    // update the error code if necessary
    if (pErrorMsg != NO_ERROR_MSG)
    {
        addErrorMsg((ERRORCODE)ERROR_RTC_DAPI_RTC_ACCURACY_TEST, NULL, pErrorMsg);
    }
    
    return getErrorMsg();
}

ErrorMsg * RTCDAPI :: rtcWDTTest(void)
{
    ErrorMsg *pErrorMsg;
    
    m_rtcDriver = new RTCDDAPI("RTCDDAPI");
    if ((pErrorMsg =  m_rtcDriver->getErrorMsg()) != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)ERROR_RTC_DAPI_RTC_WDT_TEST, "Construct RTC DDAPI failed", pErrorMsg);
    }

    if((pErrorMsg = m_rtcDriver->rtcWDTTest()) != NO_ERROR_MSG)
    {
        return returnErrorMsg((ERRORCODE)ERROR_RTC_DAPI_RTC_WDT_TEST, NULL, pErrorMsg);
    }

    delete m_rtcDriver;
    m_rtcDriver = NULL;
    
    return pErrorMsg;
}

