#include <stdlib.h>
#include <stdarg.h>

#include "../memory/MemoryDDAPI.hpp"
#include "PCIDDAPI.hpp"

extern "C"{
#include <pci/pci.h>
}

MemoryDDAPI   *m_memoryDriver;


PCI_NAME_MAPPING pciNameMapping[] =
{
    {0x8086, 0x6f0d, "Intel", "NTB"},
    {0x1077, 0x8832, "QLogic", "EP83xx - iSCSI"},
    {0x1077, 0x8830, "QLogic", "EP83xx - NIC"},
    {0x1077, 0x8831, "QLogic", "EP83xx - FCoE"},
    {0x1077, 0x2831, "QLogic", "EP83xx - FC"},
    {0x1000, 0x0097, "LSI", "SAS3008"},
    {0xffff, 0xffff, "NULL", "NULL"},
};

ClassCodeDetailedDescription PCIClassInformation[] = {

    //Base class = 00h
    {NVGA,  "NVGA",         "All currently implemented devices except VGA-compatible devices"},
    {VGA,   "VGA_DEVICE",   "VGA-compatible device"},
    
    //Base class = 01h
    {PCI_SCSI,      "SCSI",         "NULL"},
    {IDE,           "IDE",          "NULL"},
    {FD,            "FD",           "Floppy disk controller"},
    {IPI,           "IPI",          "IPI bus controller"},
    {RAID,          "RAID",         "RAID controller"},
    {ATA,           "ATA",          "ATA controller, 2 types"},
    {PCI_SATA,      "SATA",         "NULL"},
    {PCI_SAS,       "SAS",          "NULL"},
    //{SSS,         "SSS",          "Solid State Storage Controller, 3 types"},
    {PCI_NVME,      "NVME",          "Solid State Storage Controller, 3 types"},
    {UFS,           "UFS",          "Universal Flash Storage controller, 2 types"},
    {OTHER_01,      "MASS_STORAGE", "Other mass storage controller"},

    //Base class = 02h
    {PCI_ETH,       "ETH",      "Ethernet controller"},
    {TR,        "TR",       "Token Ring controller"},
    {FDDI,      "FDDI",     "FDDI controller"},
    {ATM,       "ATM",      "ATM controller"},
    {ISDN,      "ISDN",     "ISDN controller"},
    {WF,        "WF",       "WorldFip controller"},
    {PICMG,     "PICMG",    "PICMG 2.14 Multi Computing"},
    {PCI_IB,        "IB",       "InfiniBand Controller"},
    {OTHER_02,  "OTHER_02", "Other network controller"},

    //Base class = 03h
    {VGAC,          "VGA_CONTROLLER",       "VGA-compatible / 8514-compatible controller"},
    {XGA,           "XGA_CONTROLLER",       "XGA controller"},
    {C3D,           "3D_CONTROLLER",        "3D controller"},
    {OTHER_03,      "DISPALY_CONTOLLER",    "Other display controller"},
        
    //Base class = 04h
    {VIDEO,     "VIDEO",    "Video device"},
    {AUDIO,     "AUDIO",    "Audio device"},
    {CT,        "CT",       "Computer telephony device"},
    {MMD,       "MMD",      "Mixed mode device"},
    {OTHER_04,  "OTHER_04", "Other multimedia device"},

    //Base class = 05h
    {RAM,       "RAM",      "RAM"},
    {FLASH_CLASS,     "FLASH",    "Flash"},
    {OTHER_05,  "OTHER_05", "Other memory controller"},

    //Base class = 06h
    {HOST_BRIDGE,       "HOST_BRIDGE",      "Host bridge"},
    {ISA_BRIDGE,        "ISA_BRIDGE",       "ISA bridge"},
    {EISA_BRIDGE,       "EISA_BRIDGE",      "EISA bridge"},
    {MCA_BRIDGE,        "MCA_BRIDGE",       "MCA bridge"},
    {P2P_BRIDGE,        "P2P_BRIDGE",       "PCI-to-PCI bridge, 2 types"},
    {PCMCIA_BRIDGE,     "PCMCIA_BRIDGE",    "PCMCIA bridge"},
    {NUBUS_BRIDGE,      "NUBUS_BRIDGE",     "NuBus bridge"},
    {CARD_BUS_BRIDGE,   "CARD_BUS_BRIDGE",  "CardBus bridge"},
    {RACEWAY_BRIDGE,    "RACEWAY_BRIDGE",   "RACEway bridge"},
    {ST_P2P_BRIDGE,     "ST_P2P_BRIDGE",    "Semi-transparent PCI-to-PCI bridge, 2 types"},
    {I2P_HOST_BRIDGE,   "I2P_HOST_BRIDGE",  "InfiniBand-to-PCI host bridge"},
    {A2P_HOST_BRIDGE,   "A2P_HOST_BRIDGE",  "Advanced Switching to PCI host bridge, 2 types"},
    {OTHER_06,          "OTHER_06",         "Other bridge device"},

    //Base class = 07h
    //Base class = 08h
    {PICC,              "PIC",                  "PIC"},
    {DMA,               "DMA",                  "DMA controller"},
    {Timer,             "Timer",                "Timer"},
    {RTC,               "RTC",                  "RTC"},
    {PCI_HB,            "PCI_HB",               "Generic PCI Hot-Plug controller"},
    {SD_HOST,           "SD_HOST",              "SD host controller"},
    {IOMMU,             "IOMMU",                "IOMMU"},
    {RCE,               "RCE" ,                 "Root Complex Event controller"},
    {OTHER_08,          "SYSTEM_PERIPHERAL",    "Other system peripheral"},

    //Base class = 09h
    //Base class = 0Ah

    //Base class = 0Bh
    {PROCESSOR_386,     "PROCESSOR_386",        "386"},
    {PROCESSOR_486,     "PROCESSOR_486",        "486"},
    {PROCESSOR_PENTIUM, "PROCESSOR_PENTIUM",    "Pentium"},
    {PROCESSOR_ALPHA,   "PROCESSOR_ALPHA",      "Alpha"},
    {PROCESSOR_PPC,     "PROCESSOR_PPC",        "PowerPC"},
    {PROCESSOR_MIPS,    "PROCESSOR_MIPS",       "MIPS"},
    {PROCESSOR_CP,      "PROCESSOR_CP",         "Co-processor"},
    {OTHER_0B,          "OTHER_0B",             "Other processors"},

    //Base class = 0Ch
    {IEEE_1394,     "IEEE_1394",    "IEEE 1394, 2 types"},
    {ACCESS_BUS,    "ACCESS_BUS",   "ACCESS.bus"},
    {SSA,           "SSA",          "SSA"},
    {PCI_USB,           "USB",          "Universal Serial Bus, 6 types"},
    {FC,            "FC",           "Fibre Channel"},
    {SMBUS,         "SMBUS",        "System Management Bus"},
    {IBD,           "IBD",          "This sub-class is deprecated"},
    {IPMI,          "IPMI",         "3 types"},
    {SERCOS,        "SERCOS",       "SERCOS Interface Standard (IEC 61491)"},
    {CANBUS,        "CANBUS",       "CANbus"},
    {OTHER_0C,      "OTHER_0C",     "Other Serial Bus Controllers"},

    //Base class = 0Dh
    {IRDA,              "IRDA",             "iRDA compatible controller"},
    {Code_0D01,         "NULL",             "NULL"}, //TBD
    {RF,                "RF",               "RF controller"},
    {BLUETOOTH,         "BLUETOOTH",        "Bluetooth"},
    {BROADBAND,         "BROADBAND",        "Broadband"},
    {ETH_802_11A_5G,    "ETH_802_11A_5G",   "Ethernet (802.11a - 5 GHz)"},
    {ETH_802_11B_2P4G,  "ETH_802_11B_2P4G", "Ethernet (802.11b - 2.4 GHz)"},
    {OTHER_0D,          "OTHER_0D",         "Other type of wireless controller"},
    
    {0xffff,    "NULL",   "NULL"},

    //Base class = 0xf , satellite communication controllers
    //Base class = 0x10, encryption and decryption controllers
    //Base class = 0x11, data acquisition and signal processing controllers
    //Base class = 0x12, processing accelerators
    //Base class = 0x13, Non-Essential Instrumentation Function
};


PCIDDAPI::PCIDDAPI(STRING className) : BaseClassDDAPI(className)
{
    m_device_amount = 0;
    //m_u8NtbPortNumber = 0xff;

    m_memoryDriver = new MemoryDDAPI("MemoryDAPI");
}



PCIDDAPI::~PCIDDAPI()
{
    releasePciObjects();
    releaseMemoryDriver();
    //pci_cleanup(m_libPacc);
}

/*
void 
PCIDDAPI::createMemoryDriver()
{
    //if(m_memoryDriver == NULL)
    //{
        m_memoryDriver = new MemoryDDAPI("MemoryDAPI");
   // }
}
*/

void 
PCIDDAPI::releaseMemoryDriver()
{
    if(m_memoryDriver != NULL)
    {
        delete m_memoryDriver;
        m_memoryDriver = NULL;
    }
}

void 
PCIDDAPI::releasePciObjects()
{
    UINT32 u32Index;

    for(u32Index = 0; u32Index < m_device_amount; u32Index++)
    {
        if(m_pciDeviceList[u32Index] != NULL)
        {
            delete m_pciDeviceList[u32Index];
            m_pciDeviceList[u32Index] = NULL;
        }
    }
    
}

void
PCIDDAPI::releasePciObject()
{
    if(m_pciDevice != NULL)
    {
        delete m_pciDevice;
        m_pciDevice = NULL;
    }
}

void 
PCIDDAPI::releasePciStructs()
{
    //printf("------------------------ release pci tree -----------------------\n");//check 
    if(m_host_bridge.chain != NULL) //release structures of pci tree
        m_pciLibrary.release_pci_tree(&m_host_bridge);

    //printf("--------------------- release device struct ------------------------\n");//check 
    if(m_first_dev != NULL)
        m_pciLibrary.release_device_struct(m_first_dev);
    
    //printf("$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$\n");//check 
}

bool
PCIDDAPI::isPCIe(PFA    in_bdf)
{
    bool    bIsPCIe;

    createPciObject(in_bdf);
    
    bIsPCIe = m_pciDevice->IsPCIe();

    releasePciObject();

    return bIsPCIe;
}


ERRORCODE
PCIDDAPI::convertStringToClassCode(string    in_ClassString,
                                   UINT32*   out_u32ClassCode)
{
    UINT32  u32Index;

    //convert class string to be uppercase
    transform(in_ClassString.begin(), in_ClassString.end(), in_ClassString.begin(), ::toupper);

    u32Index = 0;
    while(PCIClassInformation[u32Index].code != 0xffff)
    {
        //printf("PCIDDAPI::convertStringToClassCode, class code: 0x%x\n", PCIClassInformation[u32Index].code);//check 
        if(in_ClassString.compare(PCIClassInformation[u32Index].szShortName) == 0)
        {
            *out_u32ClassCode = PCIClassInformation[u32Index].code;
            return NO_ERROR;
        }
        u32Index++;
    }

    *out_u32ClassCode = 0xffff;
    return ERROR_PCI_DDAPI_INVALID_CLASS_CODE;
}


ERRORCODE
PCIDDAPI::checkPresenceOfDevice(PFA in_bdf)
{
    int     fd;
    char    namebuf[MAX_STRING_LENGTH];

    snprintf(namebuf, MAX_STRING_LENGTH, "/sys/bus/pci/devices/%04x:%02x:%02x.%d/config",
             in_bdf.u16Domain, in_bdf.u16Bus, in_bdf.u16Device, in_bdf.u8Function);

    fd = open(namebuf, O_RDONLY);
    if(fd == -1)
    {
        return ERROR_NO_DEVICE_FOUND;
    }
    else
    {
        return NO_ERROR;
    }
}


void 
PCIDDAPI::getDevicesName(vector<PCIInformantion>*   inout_pPciList)
{
    vector<PCIInformantion>::iterator   it;

    getPciDeviceNameMappingTable();

    for(it = inout_pPciList->begin(); it != inout_pPciList->end(); ++it)
    {
        it->szDeviceName.clear();
        if(!getNameFromIds(&(*it)))
        {
            defineName(&(*it));
        }
    }
}

bool
PCIDDAPI::getNameFromIds(PCIInformantion*  inout_pciInfo)
{
    bool                                VendorIdFound = false;
    vector<PCI_NAME_MAPPING>::iterator  it;
    

    //UINT32 u32Index = 0; //check 
    for(it = m_pciNameList.begin(); it != m_pciNameList.end(); ++it)
    {
        if(!VendorIdFound)
        {
            if(it->u16VendorID == inout_pciInfo->u16VendorID)
            {
                //cout << "Got vendor id, u32Index: " << u32Index << endl; //check
                VendorIdFound = true;
                inout_pciInfo->szDeviceName.assign(it->strVendorName);
            }
        }
        if(VendorIdFound)
        {
            if(it->u16VendorID != inout_pciInfo->u16VendorID && it->u16VendorID != 0x0)
            {
                break;
            }
            if(it->u16DeviceID == inout_pciInfo->u16DeviceID)
            {
                //cout << "Got device id, u32Index: " << u32Index << endl; //check 
                inout_pciInfo->szDeviceName.append(" ");
                inout_pciInfo->szDeviceName.append(it->strDeviceName);
                return true;
            }
        }
        //u32Index++;
    }
    
    return false;
}

void 
PCIDDAPI::defineName(PCIInformantion*  inout_pciInfo)
{
    bool    deviceFound = false;
    UINT32  u32Index;
    char    temp[5];

    inout_pciInfo->szDeviceName.clear();
            
    //if a name is defined in pciNameMapping table, then use it instead of pci.ids
    u32Index = 0;
    while(pciNameMapping[u32Index].u16VendorID != 0xffff)
    {
        if(pciNameMapping[u32Index].u16VendorID == inout_pciInfo->u16VendorID &&
           pciNameMapping[u32Index].u16DeviceID == inout_pciInfo->u16DeviceID)
        {
            inout_pciInfo->szDeviceName.assign(pciNameMapping[u32Index].strVendorName);
            inout_pciInfo->szDeviceName.append(" ");
            inout_pciInfo->szDeviceName.append(pciNameMapping[u32Index].strDeviceName);
            deviceFound = true;
            break;
        }
        u32Index++;
    }
            
    if(!deviceFound)
    {
        inout_pciInfo->szDeviceName.assign("Vendor ");
        sprintf(temp, "%04x", inout_pciInfo->u16VendorID);
        inout_pciInfo->szDeviceName.append(temp);
                
        inout_pciInfo->szDeviceName.assign(" ");
                
        inout_pciInfo->szDeviceName.assign("Device ");
        sprintf(temp, "%04x", inout_pciInfo->u16DeviceID);
        inout_pciInfo->szDeviceName.append(temp);
    }
}

    
int 
PCIDDAPI::compare_them(const void *A, const void *B)
{
    const struct pci_dev *a = (*(const struct device **)A)->dev;
    const struct pci_dev *b = (*(const struct device **)B)->dev;

    if (a->domain < b->domain)
        return -1;
    if (a->domain > b->domain)
        return 1;
    if (a->bus < b->bus)
        return -1;
    if (a->bus > b->bus)
        return 1;
    if (a->dev < b->dev)
        return -1;
    if (a->dev > b->dev)
        return 1;
    if (a->func < b->func)
        return -1;
    if (a->func > b->func)
        return 1;

    return 0;
}

void 
PCIDDAPI::sort_them()
{
    struct device **index, **h, **last_dev;
    int cnt;
    struct device *d;

    cnt = 0;
    for (d=m_first_dev; d; d=d->next)
    {
        cnt++;
    }

    h = index = (struct device**)m_pciLibrary.xmalloc(sizeof(struct device*) * cnt);
    for (d=m_first_dev; d; d=d->next)
    {
        *h++ = d;
    }
    qsort(index, cnt, sizeof(struct device *), compare_them);
    last_dev = &m_first_dev;
    h = index;
    while(cnt--)
    {
        *last_dev = *h;
        last_dev = &(*h)->next;
        h++;
    }
    *last_dev = NULL;
}


void
PCIDDAPI::getDeviceList(vector<PCIInformantion>*   out_pPciList,
                        bool                       in_bSort)
{
    UINT32              u32Temp;
    struct device       *d;

    m_libPacc = m_pciLibrary.pci_alloc();
    m_pciLibrary.sysfs_init(m_libPacc);
    m_pciLibrary.sysfs_scan(m_libPacc);


    m_first_dev = NULL;
    m_device_amount = 0;
  
    //Iterate over all devices 
    for (m_libDevice = m_libPacc -> devices; m_libDevice; m_libDevice = m_libDevice -> next)
    {
        if ( (d = m_pciLibrary.scan_device(m_libDevice))  )
        {
            d->next = m_first_dev;
            m_first_dev = d;
        }
    }
    if(in_bSort)
    {
        //m_pciLibrary.sort_them(m_first_dev);
        sort_them();
    }
    /*
    //check 
    while(m_first_dev)
    {
        printf("%04x:%02x:%02x.%x\n",
                m_first_dev->dev->domain, m_first_dev->dev->bus, m_first_dev->dev->dev,
                m_first_dev->dev->func);
        m_first_dev = m_first_dev->next;
    }
    */

    
    //create device objects 
    while(m_first_dev)
    {
        m_pciInfo.u16Domain     = m_first_dev->dev->domain;
        m_pciInfo.u16Bus        = m_first_dev->dev->bus;
        m_pciInfo.u16Device     = m_first_dev->dev->dev;
        m_pciInfo.u8Function    = m_first_dev->dev->func;
        m_pciInfo.u16VendorID   = m_first_dev->dev->vendor_id;
        m_pciInfo.u16DeviceID   = m_first_dev->dev->device_id;
        m_pciInfo.szDeviceName.clear();


        m_pciDeviceList[m_device_amount] = new PCIDevice(m_pciInfo);
        
        u32Temp =  m_pciDeviceList[m_device_amount]->GetClassCode();
        m_first_dev->dev->device_class = (u32Temp & 0xffff00) >> 8;
        m_pciInfo.u32ClassCode  = m_first_dev->dev->device_class;

        out_pPciList->push_back(m_pciInfo);

        m_device_amount++;
        m_first_dev = m_first_dev->next;
    }

#if 0
    //check
    cout << "-------- check m_first_dev ----------" << endl; //check 
    struct device   *dd;
    for(dd = m_first_dev; dd; dd=dd->next)
    {
        struct pci_dev *dev = dd->dev;
        printf("%04x:%02x:%02x.%x\n", dev->domain, dev->bus, dev->dev, dev->func);
        
        //for(pciDev = dd->dev; pciDev; pciDev = pciDev->next)
        //{
        //    printf("---------------------------------\n");
        //}
    }
#endif

    releasePciObjects();
    m_pciLibrary.pci_cleanup(m_libPacc);
}

UINT32  
PCIDDAPI::getDeviceAmount()
{
    return m_device_amount;
}


void 
PCIDDAPI::GetDeviceByID(unsigned int    in_u32LogicalID, 
                        PCIDevice       *pciDeviceList)
{
    unsigned int u32LogicalID;

    for(u32LogicalID=0; u32LogicalID < m_device_amount; u32LogicalID++)
    {
        if(u32LogicalID == in_u32LogicalID)
        {
            pciDeviceList = m_pciDeviceList[u32LogicalID];
            break;
        }
    }
}


void 
PCIDDAPI::createPFA(PCIInformantion*    in_pPciInfo,
                    PFA*                out_pPFA)
{
    out_pPFA -> u16Domain   = in_pPciInfo -> u16Domain;
    out_pPFA -> u16Bus      = in_pPciInfo -> u16Bus;
    out_pPFA -> u16Device   = in_pPciInfo -> u16Device;
    out_pPFA -> u8Function  = in_pPciInfo -> u8Function;
}


/*
ERRORCODE 
PCIDDAPI::GetIDByBDF(unsigned short in_u16Domain,
                     unsigned short in_u16Bus,
                     unsigned short in_u16Device,
                     unsigned char  in_u8Function,
                     unsigned int   *out_u32LogicalID)
{
    unsigned int u32LogicalID;

    for(u32LogicalID = 0; u32LogicalID < m_device_amount; u32LogicalID++)
    {
        if(m_pciDeviceList[u32LogicalID]->GetDomain() == in_u16Domain && 
           m_pciDeviceList[u32LogicalID]->GetBus() == in_u16Bus && 
           m_pciDeviceList[u32LogicalID]->GetDevice() == in_u16Device &&
           m_pciDeviceList[u32LogicalID]->GetFunction() == in_u8Function)
        {
            *out_u32LogicalID = u32LogicalID;
            return NO_ERROR; 
        }
    }

    return ERROR_NO_DEVICE_FOUND;
}
*/



/*
void 
PCIDDAPI::GetDeviceByBDF(PFA,
                         PCIDevice      *pciDevice)
{
    unsigned int u32LogicalID;

    for(u32LogicalID=0; u32LogicalID < m_device_amount; u32LogicalID++)
    {
        if(m_pciDeviceList[u32LogicalID]->GetDomain() == in_u16Domain && 
           m_pciDeviceList[u32LogicalID]->GetBus() == in_u16Bus && 
           m_pciDeviceList[u32LogicalID]->GetDevice() == in_u16Device &&
           m_pciDeviceList[u32LogicalID]->GetFunction() == in_u8Function)
        {
            pciDeviceList = m_pciDeviceList[u32LogicalID];
            break;
        }
    }
}
*/


#if 0
void 
PCIDDAPI::DisplayPCIeSwitchEepromList()
{
    unsigned int u32LogicalID;

    for(u32LogicalID=0; u32LogicalID < m_device_amount; u32LogicalID++)
    {
        if(m_pciDeviceList[u32LogicalID]->GetPCIeType() == PCIE_SWITCH_UPSTREAM_PORT)
        {
            //check the presence of EEPROM
        }
    }

    /*
    for(i=0; i<m_device_amount; i++)
    {
        if(m_pciDeviceList[i]->IsPCIe())
        {
            //for PLX PCIe switch 
            if(m_pciDeviceList[i]->GetPCIeType() == PCIE_SWITCH_UPSTREAM_PORT)
            {
                //check the presence of EEPROM
                
            }

        }
    }
    */
    
}
#endif


ERRORCODE
PCIDDAPI::clearErrors(PFA  in_bdf)
{
    int     ret;
    UINT16  u16WriteData;
    UINT16  u16Temp;
    UINT32  u32Offset;

    createPciObject(in_bdf);

    //clear pci-compatible errors 
    u32Offset = 0x6;
    u16Temp = m_pciDevice->ConfigReadWord(u32Offset);
    u16WriteData = u16Temp | 0xf900;
    m_pciDevice->ConfigWriteWord(u32Offset, u16WriteData);
    if((m_pciDevice->GetHeaderType() & 0x1) == 0x1)
    {
        u32Offset = 0x1E;
        u16Temp = m_pciDevice->ConfigReadWord(u32Offset);
        u16WriteData = u16Temp | 0xf900;
        m_pciDevice->ConfigWriteWord(u32Offset, u16WriteData);
    }

    //clear PCIE errors if need 
    ret = m_pciDevice->GetCapabilityOffset(PCIE_CAPABILITY_ID, &u32Offset);
    if(ret == 0)
    {
        u16Temp = m_pciDevice->ConfigReadWord(u32Offset + 0xA);
        u16WriteData = u16Temp | 0xf;
        m_pciDevice->ConfigWriteWord(u32Offset + 0xA, u16WriteData);
    }

    //clear AER if need 
    if(m_pciDevice->GetVendorID() == 0x8086 && m_pciDevice->GetDeviceID() == 0x6f0d) //Intel NTB
    {
        //clear uncorrectable error status register
        m_pciDevice->ConfigWriteDword(0x14C, 0xffffffff);
        //clear correctable error status register
        m_pciDevice->ConfigWriteDword(0x158, 0xffffffff);
    }
    else 
    {
        ret = m_pciDevice->getExtendedCapabilityOffset(PCIE_AER_CAPABILITY_ID, &u32Offset);
        if(ret == 0)
        {
            //clear uncorrectable error status register
            m_pciDevice->ConfigWriteDword(u32Offset + 0x4, 0xffffffff); 
            //clear correctable error status register
            m_pciDevice->ConfigWriteDword(u32Offset + 0x10, 0xffffffff);
        }
    }

    releasePciObject();

    return NO_ERROR;
}


ERRORCODE
PCIDDAPI::getErrorStatistics(PFA                in_bdf,
                             PCI_ERROR_TYPE     in_pciError,
                             list<PCI_ERROR>*   out_pErrorList)
{
    int     ret;
    UINT32  u32CapOffset;

    createPciObject(in_bdf);
    
    switch(in_pciError)
    {
        case PCI_COMPATIBLE_ERROR:
            out_pErrorList->push_back(getPciCompatibleErrorStatistics(DETECTED_PARITY_ERROR));
            out_pErrorList->push_back(getPciCompatibleErrorStatistics(SIGNALED_SYSTEM_ERROR));
            out_pErrorList->push_back(getPciCompatibleErrorStatistics(RECEIVED_MASTER_ABORT));
            out_pErrorList->push_back(getPciCompatibleErrorStatistics(RECEIVED_TARGET_ABORT));
            out_pErrorList->push_back(getPciCompatibleErrorStatistics(SIGNALED_TARGET_ABORT));
            out_pErrorList->push_back(getPciCompatibleErrorStatistics(MASTER_DATA_PARITY_ERROR));
            if((m_pciDevice->GetHeaderType() & 0x1) == 0x1) //get secondary side errors for a bridge 
            {
                out_pErrorList->push_back(getPciCompatibleErrorStatistics(DETECTED_PARITY_ERROR_S));
                out_pErrorList->push_back(getPciCompatibleErrorStatistics(RECEIVED_SYSTEM_ERROR));
                out_pErrorList->push_back(getPciCompatibleErrorStatistics(RECEIVED_MASTER_ABORT_S));
                out_pErrorList->push_back(getPciCompatibleErrorStatistics(RECEIVED_TARGET_ABORT_S));
                out_pErrorList->push_back(getPciCompatibleErrorStatistics(SIGNALED_TARGET_ABORT_S));
                out_pErrorList->push_back(getPciCompatibleErrorStatistics(SIGNALED_TARGET_ABORT_S));
            }
            break;

        case PCIE_ERROR:
            ret = m_pciDevice->GetCapabilityOffset(PCIE_CAPABILITY_ID, &u32CapOffset);
            if(ret != 0)
            {
                return ERROR_PCI_DDAPI_CAPABILITY_UNSUPPORT;
            }
            out_pErrorList->push_back(getPCIeErrorStatistics(UNSUPPORTED_REQUEST_DETECTED, u32CapOffset));
            out_pErrorList->push_back(getPCIeErrorStatistics(FATAL_ERROR_DETECTED, u32CapOffset));
            out_pErrorList->push_back(getPCIeErrorStatistics(NON_FATAL_ERROR_DETECTED, u32CapOffset));
            out_pErrorList->push_back(getPCIeErrorStatistics(CORRECTABLE_ERROR_DETECTED, u32CapOffset));
            break;

        case AER_ERROR:
            if(m_pciDevice->GetVendorID() == 0x8086 && m_pciDevice->GetDeviceID() == 0x6f0d) //intel NTB 
            {
                out_pErrorList->push_back(getAerErrorStatistics(UNCORRECTABLE_ERROR_STATUS, 0x14C));
                out_pErrorList->push_back(getAerErrorStatistics(UNCORRECTABLE_ERROR_STATUS, 0x158));
            }
            else 
            {
                ret = m_pciDevice->getExtendedCapabilityOffset(PCIE_AER_CAPABILITY_ID, &u32CapOffset);
                if(ret != 0)
                {
                    return ERROR_PCI_DDAPI_CAPABILITY_UNSUPPORT;
                }
                out_pErrorList->push_back(getAerErrorStatistics(UNCORRECTABLE_ERROR_STATUS, u32CapOffset + 0x4));
                out_pErrorList->push_back(getAerErrorStatistics(CORRECTABLE_ERROR_STATUS, u32CapOffset + 0x10));
                break;
            }

        default:
            break;
    }

    releasePciObject();

    return NO_ERROR;
}


PCI_ERROR 
PCIDDAPI::getPciCompatibleErrorStatistics(PCI_ERROR_TYPE  in_pciError)
{
    PCI_ERROR   errorInfo;
    UINT16      u16Temp;
    UINT16      u16TempS;

    u16Temp = m_pciDevice->ConfigReadWord(0x6);
    u16TempS = m_pciDevice->ConfigReadWord(0x1E);

    switch(in_pciError)
    {
        case DETECTED_PARITY_ERROR:
            errorInfo.strErrorName.assign("DetectedParityError");
            errorInfo.u32Data = (UINT32)(u16Temp & 0x8000 >> 15);
            return errorInfo;

        case SIGNALED_SYSTEM_ERROR:
            errorInfo.strErrorName.assign("SignaledSystemError");
            errorInfo.u32Data = (UINT32)(u16Temp & 0x4000 >> 14);
            return errorInfo;

        case RECEIVED_MASTER_ABORT:
            errorInfo.strErrorName.assign("ReceivedMasterAbort");
            errorInfo.u32Data = (UINT32)(u16Temp & 0x2000 >> 13);
            return errorInfo;

        case RECEIVED_TARGET_ABORT:
            errorInfo.strErrorName.assign("ReceivedTargetAbort");
            errorInfo.u32Data = (UINT32)(u16Temp & 0x1000 >> 12);
            return errorInfo;

        case SIGNALED_TARGET_ABORT:
            errorInfo.strErrorName.assign("SignaledTargetAbort");
            errorInfo.u32Data = (UINT32)(u16Temp & 0x800 >> 11);
            return errorInfo;

        case MASTER_DATA_PARITY_ERROR:
            errorInfo.strErrorName.assign("MasterDataParityError");
            errorInfo.u32Data = (UINT32)(u16Temp & 0x100 >> 8);
            return errorInfo;


        //errors for secondary side of a bridge 
        case DETECTED_PARITY_ERROR_S:
            errorInfo.strErrorName.assign("SecondaryDetectedParityError");
            errorInfo.u32Data = (UINT32)(u16TempS & 0x8000 >> 15);
            return errorInfo;

        case RECEIVED_SYSTEM_ERROR:
            errorInfo.strErrorName.assign("ReceivedSystemError");
            errorInfo.u32Data = (UINT32)(u16TempS & 0x4000 >> 14);
            return errorInfo;

        case RECEIVED_MASTER_ABORT_S:
            errorInfo.strErrorName.assign("SecondaryReceivedMasterAbort");
            errorInfo.u32Data = (UINT32)(u16TempS & 0x2000 >> 13);
            return errorInfo;

        case RECEIVED_TARGET_ABORT_S:
            errorInfo.strErrorName.assign("SecondaryReceivedTargetAbort");
            errorInfo.u32Data = (UINT32)(u16TempS & 0x1000 >> 12);
            return errorInfo;

        case SIGNALED_TARGET_ABORT_S:
            errorInfo.strErrorName.assign("SecondarySignaledTargetAbort");
            errorInfo.u32Data = (UINT32)(u16TempS & 0x800 >> 11);
            return errorInfo;

        case MASTER_DATA_PARITY_ERROR_S:
            errorInfo.strErrorName.assign("SecondaryMasterDataParityError");
            errorInfo.u32Data = (UINT32)(u16TempS & 0x100 >> 8);
            return errorInfo;

        default:
            errorInfo.strErrorName.assign(" ");
            errorInfo.u32Data = 0x0;
            return errorInfo;
    }
}



PCI_ERROR 
PCIDDAPI::getPCIeErrorStatistics(PCI_ERROR_TYPE     in_pciError,
                                 UINT32             in_u32PCIeCapOffset)
{
    UINT16      u16Temp;
    PCI_ERROR   errorInfo;


    u16Temp = m_pciDevice->ConfigReadWord(in_u32PCIeCapOffset + 0xA); //device status register

    switch(in_pciError)
    {
        case UNSUPPORTED_REQUEST_DETECTED:
            errorInfo.strErrorName.assign("UnsupportedRequestDetected");
            errorInfo.u32Data = (UINT32)((u16Temp & 0x8) >> 3);
            return errorInfo;

        case FATAL_ERROR_DETECTED:
            errorInfo.strErrorName.assign("FatalErrorDetected");
            errorInfo.u32Data = (UINT32)((u16Temp & 0x4) >> 2);
            return errorInfo;

        case NON_FATAL_ERROR_DETECTED:
            errorInfo.strErrorName.assign("NonFatalErrorDetected");
            errorInfo.u32Data = (UINT32)((u16Temp & 0x2) >> 1);
            return errorInfo;

        case CORRECTABLE_ERROR_DETECTED:
            errorInfo.strErrorName.assign("CorrectableErrorDetected");
            errorInfo.u32Data = (UINT32)(u16Temp & 0x1);
            return errorInfo;

        default:
            errorInfo.strErrorName.assign(" ");
            errorInfo.u32Data = 0x0;
            return errorInfo;
    }
}


PCI_ERROR 
PCIDDAPI::getAerErrorStatistics(PCI_ERROR_TYPE  in_pciError,
                                UINT32          in_u32ErrorOffset)
{
    PCI_ERROR   errorInfo;

    switch(in_pciError)
    {
        case UNCORRECTABLE_ERROR_STATUS:
            errorInfo.strErrorName.assign("UncorrectableErrorStatus");
            errorInfo.u32Data = m_pciDevice->ConfigReadDword(in_u32ErrorOffset);
            return errorInfo;

        case CORRECTABLE_ERROR_STATUS:
            errorInfo.strErrorName.assign("CorrectableErrorStatus");
            errorInfo.u32Data = m_pciDevice->ConfigReadDword(in_u32ErrorOffset);
            return errorInfo;

        default:
            errorInfo.strErrorName.assign(" ");
            errorInfo.u32Data = 0x0;
            return errorInfo;
    }
    
}


void 
PCIDDAPI::CreatePCIInfoList()
{
    cout << "PCIDDAPI::CreatePCIInfoList" << endl; 
    //m_pciInfoList.push_back()
}

//void 
//PCIDDAPI::Save

unsigned char
PCIDDAPI::ConfigReadByte(unsigned int in_u32LogicalID,
                         unsigned int in_u32Offset)
{
    unsigned char   data;

    data = 0xff;

    data = m_pciDeviceList[in_u32LogicalID]->ConfigReadByte(in_u32Offset);

    return data;
}


void 
PCIDDAPI::ConfigReadByte(PFA       in_bdf,
                         UINT32    in_u32StartOffset,
                         UINT32    in_u32EndOffset,
                         UINT8*    out_u8Buffer)

{
    UINT32  u32Index;
    UINT32  u32Offset;
    //UINT32  u32Temp;

    vector<UINT32>  BarList;

    createPciObject(in_bdf);

    if(m_pciDevice->GetVendorID() == 0x10B5)
    {
        releasePciObject();
        ConfigReadBytePlx(in_bdf, in_u32StartOffset, in_u32EndOffset, out_u8Buffer);
    }
    else
    {
        u32Index = 0;
        for(u32Offset = in_u32StartOffset; u32Offset <= in_u32EndOffset; u32Offset++)
        {
            out_u8Buffer[u32Index] =  m_pciDevice->ConfigReadByte(u32Offset);
            //printf("PCIDDAPI::ConfigReadByte, out_u8Buffer[%d] --- 0x%x\n", u32Index, out_u8Buffer[u32Index]); //check 
            u32Index++;
        }
        releasePciObject();
    }
}

void
PCIDDAPI::ConfigReadBytePlx(PFA       in_bdf,
                            UINT32    in_u32StartOffset,
                            UINT32    in_u32EndOffset,
                            UINT8*    out_u8Buffer)
{
    UINT32              u32Index;
    UINT32              u32Offset;
    UINT32              u32PortNumber;
    vector<UINT32>      BarList;
    PHYSICAL_ADDRESS_T  portBAR;

    createPciObjects();

    //get PLX BAR
    for(u32Index = 0; u32Index < m_device_amount; u32Index++)
    {
        if(m_pciDeviceList[u32Index]->getPortTypeRawData() == PORT_TYPE_PCIE_SWITCH_UPSTREAM_PORT)
        {
            m_pciDeviceList[u32Index]->GetBARList(&BarList);
            m_u64PciBar = ((UINT64)BarList[1]) << 32 | BarList[0];
            break;
        }
    }
    releasePciObjects();

    //get port BAR 
    createPciObject(in_bdf);
    u32PortNumber = m_pciDevice->getPortNumber();
    portBAR = m_u64PciBar + u32PortNumber * 0x1000;
    
    //read data 
    u32Index = 0;
    for(u32Offset = in_u32StartOffset; u32Offset <= in_u32EndOffset; u32Offset++)
    {
        out_u8Buffer[u32Index] = m_memoryDriver->read8(portBAR + u32Offset);
        u32Index++;
    }
    releasePciObject();
}


#if 0
UINT8
PCIDDAPI::ConfigReadByte(PFA        in_bdf,
                         UINT32     in_u32Offset)
{
    unsigned char   data;
    //unsigned int    u32LogicalID;
    //int             ret; //error code, need to change !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    data = 0xff;

    createPciObject(in_bdf);

    data = m_pciDevice->ConfigReadByte(in_u32Offset);

    releasePciObject();
    
    //ret = GetIDByBDF(in_u16Domain, in_u16Bus, in_u16Device, in_u8Function, &u32LogicalID);
    //if(ret == 0x0)
    //{
    //    data = m_pciDeviceList[u32LogicalID]->ConfigReadByte(in_u32Offset);
    //}

    return data;
}
#endif 


UINT16 
PCIDDAPI::ConfigReadWord(PFA        in_bdf,
                         UINT32     in_u32Offset)
{
    unsigned short  data;

    data = 0xffff;

    createPciObject(in_bdf);
    
    data = m_pciDevice->ConfigReadWord(in_u32Offset);

    releasePciObject();

    return data;
}

UINT32
PCIDDAPI::ConfigReadDword(PFA       in_bdf,
                          UINT32    in_u32Offset)
{
    unsigned int    data;

    data = 0xffffffff;

    createPciObject(in_bdf);

    data = m_pciDevice->ConfigReadDword(in_u32Offset);

    releasePciObject();

    return data;
}


void 
PCIDDAPI::ConfigWriteBytePlx(PFA        in_bdf,
                             UINT32     in_u32StartAddress,
                             UINT8*     in_u8pBuffer,
                             UINT32     in_u32BufferSize)
{
    UINT32              u32Offset;
    UINT32              u32Index = 0;
    UINT32              u32EndAddress;
    UINT32              u32PortNumber;
    vector<UINT32>      BarList;
    PHYSICAL_ADDRESS_T  portBAR;

    //get PLX BAR 
    createPciObjects();
    for(u32Index = 0; u32Index < m_device_amount; u32Index++)
    {
        if(m_pciDeviceList[u32Index]->getPortTypeRawData() == PORT_TYPE_PCIE_SWITCH_UPSTREAM_PORT)
        {
            m_pciDeviceList[u32Index]->GetBARList(&BarList);
            m_u64PciBar = ((UINT64)BarList[1]) << 32 | BarList[0];
            break;
        }
    }
    releasePciObjects();

    //get port BAR 
    createPciObject(in_bdf);
    u32PortNumber = m_pciDevice->getPortNumber();
    portBAR = m_u64PciBar + u32PortNumber * 0x1000;

    //write data 
    u32EndAddress = in_u32StartAddress + in_u32BufferSize;
    u32Index = 0;
    for(u32Offset = in_u32StartAddress; u32Offset < u32EndAddress; u32Offset++)
    {
        m_memoryDriver->write(portBAR + u32Offset, in_u8pBuffer[u32Index]);
        u32Index++;
    }
    releasePciObject();
}

    
void 
PCIDDAPI::ConfigWriteByte(PFA        in_bdf,
                          UINT32     in_u32StartAddress,
                          UINT8*     in_u8pBuffer,
                          UINT32     in_u32BufferSize)
{
    UINT32  u32Offset;
    UINT32  u32Index = 0;
    UINT32  u32EndAddress;

    createPciObject(in_bdf);

    if(m_pciDevice->GetVendorID() == 0x10B5)
    {
        releasePciObject();
        ConfigWriteBytePlx(in_bdf, in_u32StartAddress, in_u8pBuffer, in_u32BufferSize);
    }
    else
    {
        u32EndAddress = in_u32StartAddress + in_u32BufferSize;
        //printf("offset range: 0x%x - 0x%x\n", in_u32StartAddress, u32EndAddress);//check 
        
        for(u32Offset = in_u32StartAddress; u32Offset < u32EndAddress; u32Offset++)
        {
            //printf("offset 0x%x data: 0x%x\n",u32Offset, in_u8pBuffer[u32Index]);//check 
            m_pciDevice->ConfigWriteByte(u32Offset, in_u8pBuffer[u32Index]);
            u32Index++;
        }
        releasePciObject();
    }
}


void 
PCIDDAPI::ConfigWriteByte(PFA       in_bdf,
                          UINT32    in_u32Offset,
                          UINT8     in_u8WriteData)
{
    createPciObject(in_bdf);

    m_pciDevice->ConfigWriteByte(in_u32Offset, in_u8WriteData);

    releasePciObject();
}


void 
PCIDDAPI::ConfigWriteWord(PFA       in_bdf,
                          UINT32    in_u32Offset,
                          UINT16    in_u16WriteData)
{
    cout << "PCIDDAPI::ConfigWriteWord" << endl; //check 

    createPciObject(in_bdf);

    m_pciDevice->ConfigWriteWord(in_u32Offset, in_u16WriteData);

    releasePciObject();
}


void 
PCIDDAPI::ConfigWriteDword(PFA      in_bdf,
                           UINT32   in_u32Offset,
                           UINT32   in_u32WriteData)
{
    createPciObject(in_bdf);

    m_pciDevice->ConfigWriteDword(in_u32Offset, in_u32WriteData);

    releasePciObject();
}

//This function assumes that the selected device is valid,
//so please check device Presence before using this function.
float
PCIDDAPI::GetFloatDataByID(PFA          in_bdf,
                           DATA_TYPE    in_eDataType)
{
    float   fTemp;
    UINT16  u16VendorID, u16DeviceID;
    bool    bIsPCIe = true;

    createPciObject(in_bdf);

    u16VendorID = m_pciDevice->GetVendorID();
    u16DeviceID = m_pciDevice->GetDeviceID();

    bIsPCIe = m_pciDevice->IsPCIe();
    if(!bIsPCIe)
    {
        releasePciObject();
        return 0xff;
    }

    switch(in_eDataType)
    {
        case PCI_DATA_TYPE_CURRENT_SPEED:
            if(u16VendorID == 0x8086 && u16DeviceID == 0x6f0d)
                fTemp = m_pciDevice->GetLinkSpeed(INTEL_NTB_LNK_STA_REG);
            else
                fTemp = m_pciDevice->GetCurrentLinkSpeed();
            break;
        case PCI_DATA_TYPE_MAX_SPEED:
            if(u16VendorID == 0x8086 && u16DeviceID == 0x6f0d)
                fTemp = m_pciDevice->GetLinkSpeed(INTEL_NTB_LNK_CTL_REG);
            else
                fTemp = m_pciDevice->GetMaxLinkSpeed();
           break;
        default:
           fTemp = 0xff;
           break;
    }

    releasePciObject();

    return fTemp;
}


string  
PCIDDAPI::GetDeviceNameByID(unsigned int  in_u32LogicalID)
{
    return m_pciDeviceList[in_u32LogicalID]->GetDeviceName();
}


//This function assumes that the selected device is valid,
//so please check device Presence before using this function.
string
PCIDDAPI::getPortTypeString(PFA  in_bdf)
{
    bool    isPCIe = false;
    string  strTemp;
    
    //create m_pciDevice
    createPciObject(in_bdf);
    
    isPCIe = m_pciDevice->IsPCIe();
    if(!isPCIe)
    {
        strTemp.assign("PCI");
    }
    else
    {
        strTemp = m_pciDevice->getPortTypeString();
    }

    releasePciObject();

    return strTemp;
}


//This function assumes that the selected device is valid,
//so please check device Presence before using this function.
string
PCIDDAPI::getClassTypeString(PFA  in_bdf)
{
    ERRORCODE   eErrorCode;
    bool        bBaseClassFound = false;
    UINT32      u32Temp;
    UINT8       u8BaseClass;
    UINT8       u8SubClass;

    list<PCI_CLASS_MAPPING>             pciClassList;
    list<PCI_CLASS_MAPPING>::iterator   it;

    eErrorCode = getPciClassMappingTable(&pciClassList);
    if(eErrorCode != NO_ERROR)
    {
        return "class unknown";
    }
    //create m_pciDevice
    createPciObject(in_bdf);
    
    //get BaseClass and SubClass
    u32Temp = m_pciDevice->GetClassCode();
    u8BaseClass = (u32Temp & 0xff0000) >> 16;
    u8SubClass = (u32Temp & 0xff00) >> 8;

    //printf("PCIDDAPI::getClassTypeString, u8BaseClass: 0x%02x\n", u8BaseClass);//check 
    //printf("PCIDDAPI::getClassTypeString, u8SubClass: 0x%02x\n", u8SubClass);//check 

    for(it = pciClassList.begin(); it != pciClassList.end(); ++it)
    {
        if(!bBaseClassFound)
        {
            if(it->u8BaseClass == u8BaseClass)
            {
                bBaseClassFound = true;
                continue;
            }
        }
        if(bBaseClassFound)
        {
            if(it->u8SubClass == u8SubClass)
            {
                //printf("Got SubClass: %s\n", it->strSubClass.c_str());//check 
                return it->strSubClass;
            }
        }
    }

    releasePciObject();

    return "class undefined";
}


bool 
PCIDDAPI::getLinkStatus(PCIDevice  in_classPciDevice)
{
    bool    bLink;
    UINT16  u16Temp;

    //Broadwell NTB
    if(in_classPciDevice.GetVendorID() == 0x8086 && in_classPciDevice.GetDeviceID() == 0x6f0d)
    {
        u16Temp = in_classPciDevice.ConfigReadWord(INTEL_NTB_LNK_STA_REG);
        (u16Temp & 0x2000) == 0x2000 ? bLink = true : bLink = false;
    }
    else 
    {
        bLink = in_classPciDevice.LinkStatus();
    }

    return bLink;
}


void 
PCIDDAPI::getRetrainRegs(PCIDevice  in_classPciDevice,
                         UINT32*    out_u32LnkCtlReg,
                         UINT32*    out_u32LnkStaReg)
{
    UINT32  u32PCIeCapOffset;

    if(in_classPciDevice.GetVendorID() == 0x8086 && in_classPciDevice.GetDeviceID() == 0x6f0d)
    {
        *out_u32LnkCtlReg = INTEL_NTB_LNK_CTL_REG;
        *out_u32LnkStaReg = INTEL_NTB_LNK_STA_REG;
    }
    else 
    {
        in_classPciDevice.GetCapabilityOffset(PCIE_CAPABILITY_ID, &u32PCIeCapOffset);
        
        *out_u32LnkCtlReg = u32PCIeCapOffset + PCIE_LNKCTL_REG_OFFSET;
        *out_u32LnkStaReg = u32PCIeCapOffset + LINK_STATUS_REG_OFFSET;
    }
}


//This function assumes that the selected device is valid,
//so please check device Presence before using this function.
string
PCIDDAPI::getLinkStatus(PFA  in_bdf)
{
    //cout << "PCIDDAPI::getLinkStatus" << endl; //check 

    string  strTemp;
    UINT16  u16VendorID;
    UINT16  u16DeviceID;
    UINT16  u16Temp;

    //create m_pciDevice
    createPciObject(in_bdf);

    u16VendorID = m_pciDevice->GetVendorID();
    u16DeviceID = m_pciDevice->GetDeviceID();

    if(u16VendorID == 0x8086 && u16DeviceID == 0x6f0d)
    {
        u16Temp = m_pciDevice->ConfigReadWord(INTEL_NTB_LNK_STA_REG);
        (u16Temp & 0x2000) == 0x2000 ? strTemp = "Up" : strTemp = "Down";
    }
    else 
    {
        m_pciDevice->LinkStatus() == true ? strTemp = "Up" : strTemp = "Down";
    }

    releasePciObject();

    return strTemp;
}


//This function assumes that the selected device is valid,
//so please check device Presence before using this function.
UINT32
PCIDDAPI::getLinkWidth(PCIDevice    in_classPciDevice,
                       DATA_TYPE    in_eDataType)
{
    string  strTemp;
    UINT16  u16VendorID;
    UINT16  u16DeviceID;
    UINT16  u16Temp;

    u16VendorID = in_classPciDevice.GetVendorID();
    u16DeviceID = in_classPciDevice.GetDeviceID();

    if(in_eDataType == PCI_DATA_TYPE_CURRENT_WIDTH)
    {
        if(u16VendorID == 0x8086 && u16DeviceID == 0x6f0d)
        {
            u16Temp = in_classPciDevice.ConfigReadWord(INTEL_NTB_LNK_STA_REG);
            return (u16Temp & 0x3f0) >> 4;
        }
        else 
            return in_classPciDevice.GetCurrentLinkWidth();
    }
    else //MAX_WIDTH
    {
        if(u16VendorID == 0x8086 && u16DeviceID == 0x6f0d)
        {
            u16Temp = in_classPciDevice.ConfigReadWord(INTEL_NTB_LNK_CTL_REG);
            return (u16Temp & 0x3f0) >> 4;
        }
        else 
            return in_classPciDevice.GetCurrentLinkWidth();
    }
}


void
PCIDDAPI::createPciObject(PFA   in_bdf)
{
    m_pciInfo.u16Domain     = in_bdf.u16Domain;
    m_pciInfo.u16Bus        = in_bdf.u16Bus;
    m_pciInfo.u16Device     = in_bdf.u16Device;
    m_pciInfo.u8Function    = in_bdf.u8Function;
    m_pciInfo.u16VendorID   = PCI_UNUSED_PARAMETER;
    m_pciInfo.u16DeviceID   = PCI_UNUSED_PARAMETER;

    m_pciInfo.szDeviceName.assign("NULL");

    m_pciDevice = new PCIDevice(m_pciInfo);
}


void
PCIDDAPI::createPciObjects()
{
    m_libPacc = m_pciLibrary.pci_alloc();
    m_pciLibrary.sysfs_init(m_libPacc);
    m_pciLibrary.sysfs_scan(m_libPacc);

    m_device_amount = 0;
  
    //Iterate over all devices 
    for (m_libDevice = m_libPacc -> devices; m_libDevice; m_libDevice = m_libDevice -> next)
    {
        m_pciInfo.u16Domain     = m_libDevice->domain;
        m_pciInfo.u16Bus        = m_libDevice->bus;
        m_pciInfo.u16Device     = m_libDevice->dev;
        m_pciInfo.u8Function    = m_libDevice->func;
        m_pciInfo.u16VendorID   = m_libDevice->vendor_id;
        m_pciInfo.u16DeviceID   = m_libDevice->device_id;

        m_pciInfo.szDeviceName.clear();

        m_pciDeviceList[m_device_amount] = new PCIDevice(m_pciInfo);
        
        m_device_amount++;
    }

    m_pciLibrary.pci_cleanup(m_libPacc);
}


ERRORCODE
PCIDDAPI::PciRetrainTest(PFA  in_bdf)
{
    createPciObject(in_bdf);
    releasePciObject();
    return NO_ERROR;
}


//This function assumes that the selected device is valid,
//so please check device Presence before using this function.
UINT32 
PCIDDAPI::GetDataByBDF(PFA          in_bdf,
                       DATA_TYPE    in_eDataType)
{
    UINT32  u32Data;

    createPciObject(in_bdf);
    
    switch(in_eDataType)
    {
        case PCI_DATA_TYPE_DOMAIN:
            u32Data = m_pciDevice->GetDomain();
            break;
        case PCI_DATA_TYPE_BUS:
            u32Data = m_pciDevice->GetBus();
            break;
        case PCI_DATA_TYPE_DEVICE:
            u32Data = m_pciDevice->GetDevice();
            break;
        case PCI_DATA_TYPE_FUNCTION:
            u32Data = m_pciDevice->GetFunction();
            break;
        case PCI_DATA_TYPE_VENDOR_ID:
            u32Data = m_pciDevice->GetVendorID();
            break;
        case PCI_DATA_TYPE_DEVICE_ID:
            u32Data = m_pciDevice->GetDeviceID();
            break;
        case PCI_DATA_TYPE_CLASS_CODE:
            u32Data = m_pciDevice->GetClassCode();
            break;
        case PCI_DATA_TYPE_REVISION_ID:
            u32Data = m_pciDevice->GetRevisionID();
            break;
        case PCI_DATA_TYPE_HEADER_TYPE:
            u32Data = m_pciDevice->GetRevisionID();
            break;
        
        case PCI_DATA_TYPE_CURRENT_WIDTH:
        case PCI_DATA_TYPE_MAX_WIDTH:
            u32Data = getLinkWidth(*m_pciDevice, in_eDataType);
            break;
        
        case PCI_DATA_TYPE_PORT_TYPE:
            u32Data = m_pciDevice->GetPCIeType();
            break;
        //case LINK_STATUS:
        //    u32Data = m_pciDevice->LinkStatus();
            break;
        default:
            u32Data = 0xffffffff;
            break;
    }

    releasePciObject();

    return u32Data;
}


UINT32 
PCIDDAPI::GetDataByID(UINT32    in_u32LogicalID,
                      DATA_TYPE in_eDataType)
{
    switch(in_eDataType)
    {
        case PCI_DATA_TYPE_DOMAIN:
            return m_pciDeviceList[in_u32LogicalID]->GetDomain();
        case PCI_DATA_TYPE_BUS:
            return m_pciDeviceList[in_u32LogicalID]->GetBus();
        case PCI_DATA_TYPE_DEVICE:
            return m_pciDeviceList[in_u32LogicalID]->GetDevice();
        case PCI_DATA_TYPE_FUNCTION:
            return m_pciDeviceList[in_u32LogicalID]->GetFunction();
        case PCI_DATA_TYPE_VENDOR_ID:
            return m_pciDeviceList[in_u32LogicalID]->GetVendorID();
        case PCI_DATA_TYPE_DEVICE_ID:
            return m_pciDeviceList[in_u32LogicalID]->GetDeviceID();
        case PCI_DATA_TYPE_CLASS_CODE:
            return m_pciDeviceList[in_u32LogicalID]->GetClassCode();
        case PCI_DATA_TYPE_REVISION_ID:
            return m_pciDeviceList[in_u32LogicalID]->GetRevisionID();
        case PCI_DATA_TYPE_HEADER_TYPE:
            return m_pciDeviceList[in_u32LogicalID]->GetRevisionID();

        case PCI_DATA_TYPE_CURRENT_WIDTH:
        case PCI_DATA_TYPE_MAX_WIDTH:
            return getLinkWidth(*m_pciDeviceList[in_u32LogicalID], in_eDataType);
        
        case PCI_DATA_TYPE_PORT_TYPE:
            return m_pciDeviceList[in_u32LogicalID]->GetPCIeType();
        //case LINK_STATUS:
        //    return m_pciDeviceList[in_u32LogicalID]->LinkStatus();
        default:
            return 0xffffffff;
    }
}


//void 
//PCIDDAPI::scanEepromsInVirtualSwitchMode(vector<EEPROM_INFO> *out_pEepromList)
//{

//}




void 
PCIDDAPI::scanEeproms(vector<EEPROM_INFO> *out_pEepromList)
{
    char            tempBuffer[16];
    UINT8           u8EepPrsnt = 0x0;
    UINT8           u8EepromCount = 0;
    UINT32          u32Temp;
    UINT32          u32Index;
    vector<UINT32>  BarList;
    //vector<UINT32>::iterator it; //check
    EEPROM_INFO     eepromInfo;
    
    createPciObjects(); 

    for(u32Index = 0; u32Index < m_device_amount; u32Index++)
    {
        //only pcie switches need to scan the presence of EEPROM, currently only base mode is supported. 
        //if(m_pciDeviceList[u32Index]->getPortTypeRawData() == PORT_TYPE_PCIE_SWITCH_UPSTREAM_PORT ||
        //   m_pciDeviceList[u32Index]->getPortTypeRawData() == PORT_TYPE_PCIE_SWITCH_DOWNSTREAM_PORT)
        if(m_pciDeviceList[u32Index]->getPortTypeRawData() == PORT_TYPE_PCIE_SWITCH_UPSTREAM_PORT)
        {

            //if(m_pciDeviceList[u32Index]->getPortNumber() == 0)
            //{
                //check 
                /*
                printf("%x:%x.%x\n", 
                        m_pciDeviceList[u32Index]->GetBus(), m_pciDeviceList[u32Index]->GetDevice(),
                        m_pciDeviceList[u32Index]->GetFunction());
                */

                //get BAR 
                m_pciDeviceList[u32Index]->GetBARList(&BarList);
                //check
                /*
                int barIndex = 0;
                for(it = BarList.begin(); it != BarList.end(); it++)
                {
                    printf("Bar[%d] - 0x%x\n",barIndex, *it);
                    barIndex++;
                }
                BarList.clear();
                printf("------------------------\n");
                */

                m_u64PciBar = ((UINT64)BarList[1]) << 32 | BarList[0];
                //printf("m_u64PciBar: 0x%x\n", m_u64PciBar);//check 

                //check EEPROM presence 
                u32Temp = m_memoryDriver->read32(m_u64PciBar + PCIE_SWITCH_EEPROM_STATUS_CTRL);
                u8EepPrsnt = (u32Temp & 0x30000) >> 16;
                //printf("u8EepPrsnt: 0x%x\n", u8EepPrsnt);//check 

                if(u8EepPrsnt == 0x1 || u8EepPrsnt == 0x3) //EEPROM is existent
                {
                    sprintf(tempBuffer, "eeprom%d", u8EepromCount);
                    eepromInfo.strEepromName.assign(tempBuffer);
                    eepromInfo.u16Domain = m_pciDeviceList[u32Index]->GetDomain();
                    eepromInfo.u16Bus = m_pciDeviceList[u32Index]->GetBus();
                    eepromInfo.u16Device = m_pciDeviceList[u32Index]->GetDevice();
                    eepromInfo.u8Function = m_pciDeviceList[u32Index]->GetFunction();
                    out_pEepromList->push_back(eepromInfo);
                    u8EepromCount++;
                }
           //}
        }
    }

    releasePciObjects();
}


ErrorMsg* 
PCIDDAPI::waitEepromToBeIdle()
{
    //cout << "PCIDDAPI::waitEepromToBeIdle" << endl;//check 

    UINT32      u32Command;
    UINT32      u32Check;
    UINT32      u32Timeout = 100000;
    ErrorMsg*   pErrorMsg;

    u32Command = m_memoryDriver->read32(m_u64PciBar + PCIE_SWITCH_EEPROM_STATUS_CTRL);
    //printf("u32Command: 0x%x\n", u32Command);//check 

    u32Command &= ~(7 << 13); // Clear EepCmd [15:13]
    u32Command &= ~(1 << 24); // Clear EepRdy [24]
    u32Command &= ~(7 << 28); // Clear EepWrStatus [30:28]
    u32Command |= (1 << 25) |  //set EepWen [25]
            (PLX_EEPROM_CMD_READ_STATUS << 13);

    //if((pErrorMsg = sendEepromCommand(u32Command)) != NO_ERROR_MSG)
    //{
    //    return returnErrorMsg(ERROR_PCI_DDAPI_EEPROM_WAIT_IDLE, NULL, pErrorMsg);
    //}

    //pErrorMsg = returnErrorMsg((ERRORCODE)(ERROR_PCI_DDAPI_EEPROM_WAIT_IDLE | ERROR_TIME_OUT), NULL, NULL);
    do
    {
        if((pErrorMsg = sendEepromCommand(u32Command)) != NO_ERROR_MSG)
        {
            return returnErrorMsg(ERROR_PCI_DDAPI_EEPROM_WAIT_IDLE, NULL, pErrorMsg);
        }
        
        u32Check = m_memoryDriver->read32(m_u64PciBar + PCIE_SWITCH_EEPROM_STATUS_CTRL);
        //printf("32Check: 0x%x\n", u32Check);//check 

        if(((u32Check & (1 << 24)) == 0) && ((u32Check & (7 << 28)) == 0))
        {
            pErrorMsg = NO_ERROR_MSG;
            break;
        }
        u32Timeout--;

    }while(u32Timeout);

    return pErrorMsg;    
}



ErrorMsg*
PCIDDAPI::sendEepromCommand(UINT32  in_u32Command)
{
    UINT32      u32Check;
    UINT32      u32Timeout = 100000;
    ErrorMsg*   pErrorMsg;

    pErrorMsg = returnErrorMsg((ERRORCODE)(ERROR_PCI_DDAPI_EEPROM_SEND_COMMAND | ERROR_TIME_OUT), NULL, NULL);

    m_memoryDriver->write(m_u64PciBar + PCIE_SWITCH_EEPROM_STATUS_CTRL, in_u32Command);

    do
    {
        u32Check = m_memoryDriver->read32(m_u64PciBar + PCIE_SWITCH_EEPROM_STATUS_CTRL);
        if((u32Check & ((1 << 18) | (1 << 24))) == 0)
        {
            pErrorMsg = NO_ERROR_MSG;
            break;
        }
        usleep(500);
        u32Timeout--;

    }while(u32Timeout);
    
    return pErrorMsg;
}


ErrorMsg*
PCIDDAPI::getEeprom(string       in_strEepromName,
                    EEPROM_INFO  *out_pEeprom)
{
    vector<EEPROM_INFO>             EepromList;
    vector<EEPROM_INFO>::iterator   it;

    scanEeproms(&EepromList);
    for(it = EepromList.begin(); it != EepromList.end(); ++it)
    {
        if(it->strEepromName.compare(in_strEepromName) == 0)
        {
            out_pEeprom->u16Domain = it->u16Domain;
            out_pEeprom->u16Bus = it->u16Bus;
            out_pEeprom->u16Device = it->u16Device;
            out_pEeprom->u8Function = it->u8Function;
            return NO_ERROR_MSG;
        }
    }
   
    return returnErrorMsg(ERROR_PCI_DDAPI_EEPROM_GET_DEVICE, "Device is not found", NULL);
}

ErrorMsg* 
PCIDDAPI::getEepromSize(PFA     in_bdf,
                        UINT32* out_u32pEepromSize)
{
    //cout << "PCIDDAPI::getEepromSize" << endl;//check 

    UINT32          u32Data;
    UINT32          u32ByteCount;
    vector<UINT32>  BarList;
    ErrorMsg*       pErrorMsg;

    *out_u32pEepromSize = 0xffffffff;

    //get BAR
    createPciObject(in_bdf);
    m_pciDevice->GetBARList(&BarList);
    m_u64PciBar = ((UINT64)BarList[1]) << 32 | BarList[0];
    //printf("PCIDDAPI::getEepromSize, m_u64PciBar: 0x%x\n", m_u64PciBar);//check 
    releasePciObject();
 
    ////check 
    //u32Data = m_memoryDriver->read32(m_u64PciBar);
    //printf("PCIDDAPI::getEepromSize, u32Data: 0x%x\n", u32Data);//check 

    pErrorMsg = readEepromDword(0x0, &u32Data);
    //printf("PCIDDAPI::getEepromSize, u32Data: 0x%x\n", u32Data);//check 
    if(pErrorMsg == NO_ERROR_MSG)
    {
        u32ByteCount = (u32Data >> 16) & 0xFFFF;
        //printf("32ByteCount: %d\n", u32ByteCount);//check 
        *out_u32pEepromSize = u32ByteCount + 8;
    }

    return pErrorMsg;
}

void 
PCIDDAPI::setEepromBase(PFA in_bdf)
{
    vector<UINT32>  BarList;

    createPciObject(in_bdf);
    m_pciDevice->GetBARList(&BarList);
    m_u64PciBar = ((UINT64)BarList[1]) << 32 | BarList[0];
    releasePciObject();
}


ErrorMsg* 
PCIDDAPI::readEeprom(PFA        in_bdf,
                     UINT32     in_u32Address,
                     UINT32     in_u32BufferSize, //DWord aligned
                     UINT32*    out_u32pBuffer)
{
    UINT32      u32StartAddress;

    vector<UINT32>  BarList;

    if(in_u32Address >= (1 << 14))
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PCI_DDAPI_READ_EEPROM | ERROR_OUT_OF_RANGE), NULL, NULL);
    }

    //get BAR 
    createPciObject(in_bdf);
    m_pciDevice->GetBARList(&BarList);
    m_u64PciBar = ((UINT64)BarList[1]) << 32 | BarList[0];
    releasePciObject();

    //printf("in_u32BufferSize: %d\n", in_u32BufferSize);//check 
    //u32StartAddress = 4 * (in_u32Address/sizeof(UINT32)); //PCIe switch eeprom is accessed by 4 bytes aligned.
    u32StartAddress = in_u32Address/sizeof(UINT32); //PCIe switch eeprom is accessed by 4 bytes aligned.
    //u32EndAddress = u32StartAddress + (in_u32BufferSize/4) * 4 - 4;
    //u32EndAddress = u32StartAddress + (in_u32BufferSize) * 4 - 4;
    //printf("read data from 0x%x to 0x%x, totally %d Dword\n", u32StartAddress, u32EndAddress, in_u32BufferSize/4);

    //m_pErrorMsg = readEepromBlock(u32StartAddress, u32EndAddress, out_u32pBuffer);
    m_pErrorMsg = readEepromBlock(u32StartAddress, in_u32BufferSize, out_u32pBuffer);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        m_pErrorMsg = returnErrorMsg(ERROR_PCI_DDAPI_READ_EEPROM, NULL, m_pErrorMsg);
    }
    return m_pErrorMsg;
}

ErrorMsg* 
PCIDDAPI::readEepromBlock(UINT32    in_u32StartAddress,
                          UINT32    in_u32DwordCount,
                          UINT32*   out_u32pBuffer)
{
    UINT32      u32Address;
    UINT32      u32Index;
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;

    for(u32Index = 0; u32Index < in_u32DwordCount; u32Index++)
    //for(u32Address = in_u32StartAddress; u32Address <= in_u32EndAddress; u32Address++)
    {
        u32Address = in_u32StartAddress + u32Index;
        pErrorMsg = readEepromDword(u32Address, &out_u32pBuffer[u32Index]);
        //printf("PCIDDAPI::readEepromBlock, data: 0x%x\n", out_u32pBuffer[u32Index]);//check 
        if(pErrorMsg != NO_ERROR_MSG)
        {
            break;
        }
    }

    if(pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PCI_DDAPI_EEPROM_READ_BLOCK, NULL, pErrorMsg);
    }
    return pErrorMsg;
}


#if 0
ErrorMsg*
PCIDDAPI::readEepromWord(UINT32     in_u32Address,
                         UINT32*    out_u32pData)
{
    //cout << "PCIDDAPI::readEepromWord" << endl;//check 

    UINT32      u32Command;
    ErrorMsg*   pErrorMsg;
    
    if((pErrorMsg = waitEepromToBeIdle()) != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PCI_DDAPI_EEPROM_READ, NULL, pErrorMsg);
    }

    u32Command = m_memoryDriver->read32(m_u64PciBar + PCIE_SWITCH_EEPROM_STATUS_CTRL);
    printf("PCIDDAPI::readEepromDword, u32Command ori: 0x%x\n", u32Command);//check 
    
    u32Command &= ~(7 << 13); // Clear EepCmd [15:13]
    u32Command &= ~((1 << 20) | (0x1FFF << 0)); // Clear [20, 12:0]
    u32Command |= (in_u32Address & 0x1fff) | 
                  (((in_u32Address >> 13) & 1) << 20) | // bit 13
                  (PLX_EEPROM_CMD_READ << 13); // bits [15:13]

    printf("PCIDDAPI::readEepromDword, u32Command: 0x%x\n", u32Command);//check 

    *out_u32pData = 0xFFFFFFFF;
    if((pErrorMsg = sendEepromCommand(u32Command)) == NO_ERROR_MSG)
    {
        *out_u32pData = m_memoryDriver->read32(m_u64PciBar + PCIE_SWITCH_SERIAL_EEPROM_BUFFER);
    }

    if(pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PCI_DDAPI_EEPROM_READ, NULL, pErrorMsg);
    }
    return NO_ERROR_MSG;
}
#endif 

ErrorMsg*
PCIDDAPI::readEepromDword(UINT32     in_u32Address,
                          UINT32*    out_u32pData)
{
    //cout << "PCIDDAPI::readEepromDword" << endl;//check 

    UINT32      u32Command;
    ErrorMsg*   pErrorMsg;
    
    if((pErrorMsg = waitEepromToBeIdle()) != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PCI_DDAPI_EEPROM_READ, NULL, pErrorMsg);
    }

    u32Command = m_memoryDriver->read32(m_u64PciBar + PCIE_SWITCH_EEPROM_STATUS_CTRL);
    //printf("PCIDDAPI::readEepromDword, u32Command ori: 0x%x\n", u32Command);//check 
    
    u32Command &= ~(7 << 13); // Clear EepCmd [15:13]
    u32Command &= ~((1 << 20) | (0x1FFF << 0)); // Clear [20, 12:0]
    u32Command |= (in_u32Address & 0x1fff) | 
                  (((in_u32Address >> 13) & 1) << 20) | // bit 13
                  (PLX_EEPROM_CMD_READ << 13); // bits [15:13]

    //printf("PCIDDAPI::readEepromDword, u32Command: 0x%x\n", u32Command);//check 

    *out_u32pData = 0xdeadbeaf;
    if((pErrorMsg = sendEepromCommand(u32Command)) == NO_ERROR_MSG)
    {
        *out_u32pData = m_memoryDriver->read32(m_u64PciBar + PCIE_SWITCH_SERIAL_EEPROM_BUFFER);
    }

    if(pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PCI_DDAPI_EEPROM_READ, NULL, pErrorMsg);
    }
    return NO_ERROR_MSG;
}


ErrorMsg* 
PCIDDAPI::writeEeprom(PFA       in_bdf,
                      UINT32    in_u32Address,
                      UINT32    in_u32BufferSize, //DWord aligned
                      UINT32*   in_u32pBuffer)
{
    UINT32          u32StartAddress;
    vector<UINT32>  BarList;

    if(in_u32Address >= (1 << 14))
    {
      return returnErrorMsg((ERRORCODE)(ERROR_PCI_DDAPI_EEPROM_WRITE | ERROR_OUT_OF_RANGE), NULL, NULL);
    }

    ////align up by 4 
    //if(in_u32BufferSize & 0x3)
    //{
    //    u32Size = (in_u32BufferSize + 3) & 0xfffffffc;
    //}
    //printf("PCIDDAPI::writeEeprom, u32Size: %d\n", u32Size);//check 

    //get BAR 
    createPciObject(in_bdf);
    m_pciDevice->GetBARList(&BarList);
    m_u64PciBar = ((UINT64)BarList[1]) << 32 | BarList[0];
    releasePciObject();

    u32StartAddress = in_u32Address/sizeof(UINT32);
    //printf("PCIDDAPI::writeEeprom, u32StartAddress: 0x%x\n", u32StartAddress);//check 

    m_pErrorMsg = writeEepromBlock(u32StartAddress, in_u32BufferSize, in_u32pBuffer);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PCI_DDAPI_EEPROM_WRITE, NULL, m_pErrorMsg);
    }
    return m_pErrorMsg;
}


ErrorMsg* 
PCIDDAPI::writeEepromBlock(UINT32     in_u32StartAddress,
                           UINT32     in_u32DWordCount,
                           UINT32*    in_u32pWriteData)
{
    UINT32      u32Address;
    UINT32      u32Index = 0;
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;

    //printf("PCIDDAPI::writeEepromBlock, in_u32DWordCount: %d\n", in_u32DWordCount);//check 

    for(u32Index = 0; u32Index < in_u32DWordCount; u32Index++)
    {
        u32Address = in_u32StartAddress + u32Index; 
        //printf("PCIDDAPI::writeEepromBlock, in_u32pWriteData[%d] - 0x%x\n", u32Index, in_u32pWriteData[u32Index]); //check 
        //printf("PCIDDAPI::writeEepromBlock, u32Address: 0x%x\n", u32Address);//check 

        m_pErrorMsg = writeEepromDword(u32Address, in_u32pWriteData[u32Index]);
        if(m_pErrorMsg != NO_ERROR_MSG)
        {
            break;
        }
    }
    if(pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PCI_DDAPI_EEPROM_WRITE_BLOCK, NULL, pErrorMsg);
    }
    return pErrorMsg;
}


ErrorMsg* 
PCIDDAPI::writeEepromDword(UINT32     in_u32Address,
                           UINT32     in_u32WriteData)
{
    UINT32      u32Command;
    ErrorMsg*   pErrorMsg;

    //printf("PCIDDAPI::writeEepromDword, waitEepromToBeIdle...\n");//check 
    if((pErrorMsg = waitEepromToBeIdle()) != NO_ERROR_MSG)
    {
        return pErrorMsg;
    }
    //printf("PCIDDAPI::writeEepromDword, waitEepromToBeIdle done...\n");//check 
    //printf("----------------------------------------\n");//check
    
    //enable eeprom write 
    u32Command = m_memoryDriver->read32(m_u64PciBar + PCIE_SWITCH_EEPROM_STATUS_CTRL);
    u32Command &= ~(7 << 13); // Clear EepCmd [15:13]
    u32Command &= ~((1 << 20) | (0x1FFF << 0)); // Clear [20, 12:0]
    u32Command |= PLX_EEPROM_CMD_WRITE_ENABLE << 13;// bits [15:13]
    u32Command |= (1 << 25);
    if((pErrorMsg = sendEepromCommand(u32Command)) != NO_ERROR_MSG)
    {
        return pErrorMsg;
    }
    //printf("PCIDDAPI::writeEepromDword, offset 0x260: 0x%08x\n", u32Command);//check 

    //write data to eeprom 
    //printf("PCIDDAPI::writeEepromDword, write data to eeprom...\n");//check 
    m_memoryDriver->write(m_u64PciBar + PCIE_SWITCH_SERIAL_EEPROM_BUFFER, in_u32WriteData);
    u32Command &= ~(7 << 13); // Clear EepCmd [15:13]
    u32Command |= (in_u32Address & 0x1FFF) |            // bits [12:0]
        (((in_u32Address >> 13) & 1) << 20) | // bit 20
        (PLX_EEPROM_CMD_WRITE << 13) |// bit [15:13]
        (1 << 25); // bits 25: EepWen
    pErrorMsg = sendEepromCommand(u32Command);
    //printf("#####################################################\n");//check 

    return pErrorMsg;
}


ErrorMsg*
PCIDDAPI::testEeprom(PFA in_bdf)
{
    vector<UINT32>  BarList;
    UINT32          u32Index;
    UINT32          u32Data;
    UINT32          u32ByteCount;
    UINT32         *u32pOriBuffer;
    UINT32          u32DWordCount;
    UINT32          u32TestPatterns[5] = {0xAAAAAAAA, 0x55555555, 0xFFFFFFFF, 0x0};
    bool            bRunPass = true;

    //get BAR 
    createPciObject(in_bdf);
    m_pciDevice->GetBARList(&BarList);
    m_u64PciBar = ((UINT64)BarList[1]) << 32 | BarList[0];
    releasePciObject();

    //verify eeprom signature
    readEepromDword(0x0, &u32Data);
    if((u32Data & 0xff) != EEPROM_VALIDATION_SIGNATURE)
    {
        return returnErrorMsg(ERROR_PCI_DDAPI_EEPROM_TEST, "Invalid eeprom signature", NULL);        
    }
    
    //get eeprom size 
    u32ByteCount = (u32Data >> 16) & 0xFFFF;
    u32ByteCount = u32ByteCount + 8;
    //printf("u32ByteCount: %d\n", u32ByteCount);//check 
    u32DWordCount = u32ByteCount/4;
    //printf("u32DWordCount: %d\n", u32DWordCount);//check 

    //allocate buffer to save original data
    u32pOriBuffer = (UINT32*)malloc(sizeof(UINT32) * u32DWordCount);
    readEepromBlock(0x0, u32DWordCount, u32pOriBuffer);

    //start to test 
    for(u32Index = 0; u32Index < 4; u32Index++)
    {
        m_pErrorMsg = testEepromRun(u32TestPatterns[u32Index], u32DWordCount);
        if(m_pErrorMsg != NO_ERROR_MSG)
        {
            bRunPass = false;
            break;
        }
    }

    //restore eeprom
    if(!bRunPass)
    {
        restoreEeprom(u32pOriBuffer, u32DWordCount);
    }
    else 
    {
        m_pErrorMsg = restoreEeprom(u32pOriBuffer, u32DWordCount);
    }

    //release memory buffer 
    free(u32pOriBuffer);

    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PCI_DDAPI_EEPROM_TEST, NULL, m_pErrorMsg);
    }
    return m_pErrorMsg;
}


ErrorMsg*
PCIDDAPI::restoreEeprom(UINT32*     in_u32DataToBeRestored,
                        UINT32      in_u32DWordCount)
{
    UINT32      u32Index;
    UINT32      u32Check;
    UINT32      u32StartAddress;
    bool        bComparePass = true;
    ErrorMsg*   pErrorMsg;

    //restore data
    u32StartAddress = 0x0;
    pErrorMsg = writeEepromBlock(u32StartAddress, in_u32DWordCount, in_u32DataToBeRestored);
    if(pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PCI_DDAPI_EEPROM_RESTORE, NULL, pErrorMsg);
    }

    //check data
    for(u32Index = 0; u32Index < in_u32DWordCount; u32Index++)
    {
        readEepromDword(u32Index, &u32Check);
        if(in_u32DataToBeRestored[u32Index] != u32Check)
        {
            bComparePass = false;
            break;
        }
    }

    if(!bComparePass)
    {
        return returnErrorMsg(ERROR_PCI_DDAPI_EEPROM_RESTORE, "compare data fail", NULL);
    }
    return NO_ERROR_MSG;
}


ErrorMsg*
PCIDDAPI::testEepromRun(UINT32  in_u32Pattern,
                        UINT32  in_u32DWordCount)
{
    UINT32      u32Index;
    UINT32      u32Address, u32StartAddress;
    UINT32      u32Check;
    ErrorMsg*   pErrorMsg = NO_ERROR_MSG;

    //printf("PCIDDAPI::testEepromRun, total eeprom size in DWord: %d\n", in_u32DWordCount);//check 

    //write pattern 
    u32StartAddress = 0x0;//EEPROM_START_ADDRESS;
    for(u32Index = 0; u32Index < in_u32DWordCount; u32Index++)
    {
        u32Address = u32StartAddress + u32Index; 
        //printf("PCIDDAPI::writeEepromBlock, in_u32pWriteData[%d] - 0x%x\n", u32Index, in_u32pWriteData[u32Index]); //check 
        //printf("PCIDDAPI::writeEepromBlock, u32Address: 0x%x\n", u32Address);//check 

        m_pErrorMsg = writeEepromDword(u32Address, in_u32Pattern);
        if(m_pErrorMsg != NO_ERROR_MSG)
        {
            return returnErrorMsg(ERROR_PCI_DDAPI_EEPROM_TEST, NULL, pErrorMsg);
        }
    }

    //read data and compare with pattern
    for(u32Index = 0; u32Index < in_u32DWordCount; u32Index++)
    {
        u32Address = u32StartAddress + u32Index;
        pErrorMsg = readEepromDword(u32Address, &u32Check);
        if(pErrorMsg != NO_ERROR_MSG)
        {
            return returnErrorMsg(ERROR_PCI_DDAPI_EEPROM_TEST, NULL, pErrorMsg);
        }

        if(u32Check != in_u32Pattern)
        {
            return returnErrorMsg(ERROR_PCI_DDAPI_EEPROM_TEST, "compare data fail", NULL);
        }
    }
    return pErrorMsg;
}


void 
PCIDDAPI::getHbaInfo(vector<SLOT_INFO>*     out_pSlotInfo,
                     vector<SUPPORTED_HBA>  in_supportedHbaList)
{
    bool                getHBA;
    PCIInformantion*    pciInfo;
    char                temp[16];

    vector<SLOT_INFO>::iterator     itSlot;
    vector<SUPPORTED_HBA>::iterator it;
    
    for(itSlot = out_pSlotInfo->begin(); itSlot != out_pSlotInfo->end(); ++itSlot)
    {
        pciInfo = &(itSlot->downStreamDevice);
        if(pciInfo->u16Domain == 0xffff)
        {
            itSlot->hbaName = "UNKNOWN";
            itSlot->description = "-";
        }
        else 
        {
            getHBA = false;
            for(it = in_supportedHbaList.begin(); it != in_supportedHbaList.end(); ++it)
            {
                if(pciInfo->u16VendorID == it->u16VendorID && 
                   pciInfo->u16DeviceID == it->u16DeviceID)
                {
                    itSlot->hbaName = it->hbaName;
                    itSlot->description = it->description;
                    getHBA = true;
                }
            }
            if(!getHBA)
            {
                itSlot->hbaName = "UNKNOWN";
                sprintf(temp, "%04x:%04x", pciInfo->u16VendorID, pciInfo->u16DeviceID);
                itSlot->description = temp;
            }
        }
    }
}


ERRORCODE 
PCIDDAPI::scanSlot(vector<SLOT_INFO>*    out_pSlotInfo)
{
    UINT32  u32Index;
    UINT32  u32SlotIndex;
    UINT32  u32SlotAmount;
    bool*   bGotFlag;

    vector<SLOT_INFO>::iterator     it;

    vector<SUPPORTED_HBA>               supportedHbaList;
    vector<SUPPORTED_HBA>::iterator     itHBA;

    createSlotInfo(out_pSlotInfo, &u32SlotAmount, &supportedHbaList);
    //check
    /*
    for(it = out_pSlotInfo->begin(); it != out_pSlotInfo->end(); it++)
    {
        printf("%x:%x.%x - slotType: %s\n", 
                it->u16Bus, it->u16Device, it->u8Function, it->slotType.c_str());
    }
    for(itHBA = supportedHbaList.begin(); itHBA != supportedHbaList.end(); itHBA++)
    {
        printf("%04x:%04x --- %s: %s\n", 
                itHBA->u16VendorID, itHBA->u16DeviceID, 
                itHBA->hbaName.c_str(), itHBA->description.c_str());
    }
    */

    bGotFlag = (bool*)malloc(sizeof(bool)*u32SlotAmount);
    for(u32Index = 0; u32Index < u32SlotAmount; u32Index++)
    {
        bGotFlag[u32Index] = false;
    }

    grow_tree();
    
    //create PCI objects  
    m_device_amount = 0;
    for (m_libDevice = m_libPacc -> devices; m_libDevice; m_libDevice = m_libDevice -> next)
    {
        m_pciInfo.u16Domain     = m_libDevice->domain;
        m_pciInfo.u16Bus        = m_libDevice->bus;
        m_pciInfo.u16Device     = m_libDevice->dev;
        m_pciInfo.u8Function    = m_libDevice->func;
        m_pciInfo.u16VendorID   = m_libDevice->vendor_id;
        m_pciInfo.u16DeviceID   = m_libDevice->device_id;

        m_pciInfo.szDeviceName.clear();

        m_pciDeviceList[m_device_amount] = new PCIDevice(m_pciInfo);
        
        m_device_amount++;
    }

    for(u32Index = 0; u32Index < m_device_amount; u32Index++)
    {
        u32SlotIndex = 0;
        for(it = out_pSlotInfo->begin(); it != out_pSlotInfo->end(); it++)
        {
            if(bGotFlag[u32SlotIndex])
            {
                u32SlotIndex++;
                continue;
            }
            if(it->u16Domain == m_pciDeviceList[u32Index]->GetDomain() && 
               it->u16Bus == m_pciDeviceList[u32Index]->GetBus() &&
               it->u16Device == m_pciDeviceList[u32Index]->GetDevice() &&
               it->u8Function == m_pciDeviceList[u32Index]->GetFunction())
            {
                initialPciInfoStructure(&(it->downStreamDevice));
                it->bLinkStatus = getLinkStatus(*m_pciDeviceList[u32Index]);
                if(it->bLinkStatus)
                {
                    getDownstreamDevice(*it, &(it->downStreamDevice));
                }
                bGotFlag[u32SlotIndex] = true;
            }
            u32SlotIndex++;
        }
    }
   
    getHbaInfo(out_pSlotInfo, supportedHbaList);

    free(bGotFlag);
    releasePciObjects();
    m_pciLibrary.pci_cleanup(m_libPacc);

    return NO_ERROR;
}


ErrorMsg*
PCIDDAPI::testRetrain(PFA   in_bdf)
{
    bool    bFlag;
    UINT32  u32Index;

    createPciObject(in_bdf);
   
    bFlag = isAvailableForRetrainTest(*m_pciDevice);
    if(!bFlag)
    {
        releasePciObject();
        return returnErrorMsg(ERROR_PCI_DDAPI_RETRAIN, "retrain is unsupported", NULL);
    }

    bFlag = getLinkStatus(*m_pciDevice);
    if(!bFlag)
    {
        releasePciObject();
        return returnErrorMsg(ERROR_PCI_DDAPI_RETRAIN, "link is down", NULL);
    }
    
    //start to do retrain test 1000 times 
    for(u32Index = 0; u32Index < 1000; u32Index++)
    {
        m_pErrorMsg = testRetrainRun(*m_pciDevice);
        if(m_pErrorMsg != NO_ERROR_MSG)
        {
            break;
        }
    }

    releasePciObject();

    return m_pErrorMsg;
}




ErrorMsg*
PCIDDAPI::testRetrainRun(PCIDevice  in_classPciDevice)
{
    UINT8   u8TimeCount = 0;
    UINT8   u8LinkTrainingBit;
    UINT16  u16Check;
    UINT16  u16Data;
    UINT32  u32Data;
    UINT32  u32LnkCtlReg;
    UINT32  u32LnkStaReg;
    UINT32  u32OriWidth;
    UINT32  u32CheckWidth;
    float   fOriSpeed;
    float   fCheckSpeed;
    bool    bLinkTrainingBitSet = false;

    getRetrainRegs(in_classPciDevice, &u32LnkCtlReg, &u32LnkStaReg);

    //clear Link Bandwidth Management Status Bit before doing retrain test 
    u16Data = m_pciDevice->ConfigReadWord(u32LnkStaReg);
    u16Data = u16Data | 0x4000;
    m_pciDevice->ConfigWriteWord(u32LnkStaReg, u16Data);
    //check Link Bandwidth Management Status Bit 
    u16Check = m_pciDevice->ConfigReadWord(u32LnkStaReg);
    //printf("Link Bandwidth Management Status Bit must be 0: 0x%x\n", (u16Check & 0x4000) >> 14); //check 
    if(u16Check & 0x4000)
    {
        return returnErrorMsg(ERROR_PCI_DDAPI_RETRAIN, "link bandwidth management status can't be cleared", NULL);
    }

    //save original link speed and width
    u32OriWidth = getLinkWidth(*m_pciDevice, PCI_DATA_TYPE_CURRENT_WIDTH);
    fOriSpeed = m_pciDevice->GetCurrentLinkSpeed();


    //begin to retrain by setting retrain bit to 1 
    u32Data = m_pciDevice->ConfigReadWord(u32LnkCtlReg);
    u32Data = u32Data | 0x0020; //bit5, retrain bit 
    m_pciDevice->ConfigWriteWord(u32LnkCtlReg, u32Data);
    do 
    {
        u32Data = m_pciDevice->ConfigReadWord(u32LnkStaReg);
        u8LinkTrainingBit = (u32Data & 0x8000) >> 11; //bit11, link training bit 
        if (u8LinkTrainingBit == 0)
        {
            bLinkTrainingBitSet = true;
            break;
        }
        usleep(10);
        u8TimeCount++;

    }while(u8LinkTrainingBit != 0 && u8TimeCount < 3000); //time out = 30ms
    if(!bLinkTrainingBitSet)
    {
        return returnErrorMsg(ERROR_PCI_DDAPI_RETRAIN, "check training bit timeout", NULL);
    }


    //check Link Bandwidth Management Status Bit after doing retrain test 
    u16Check = m_pciDevice->ConfigReadWord(u32LnkStaReg);
    //printf("Link Bandwidth Management Status Bit must be 1: 0x%x\n", (u16Check & 0x4000) >> 14); //check 
    if(!(u16Check & 0x4000))
    {
        return returnErrorMsg(ERROR_PCI_DDAPI_RETRAIN, "retrain didn't completed", NULL);
    }

    //check if the link width is restored 
    u32CheckWidth = getLinkWidth(*m_pciDevice, PCI_DATA_TYPE_CURRENT_WIDTH);
    fCheckSpeed = m_pciDevice->GetCurrentLinkSpeed();
    if(u32OriWidth != u32CheckWidth)
    {
        return returnErrorMsg(ERROR_PCI_DDAPI_RETRAIN, "link width check fail", NULL);
    }
    if(fOriSpeed != fCheckSpeed)
    {
        return returnErrorMsg(ERROR_PCI_DDAPI_RETRAIN, "link speed check fail", NULL);
    }

    return NO_ERROR_MSG;
}


bool
PCIDDAPI::isNtbDevice(RetrainInfo in_retrainInfo)
{
    //Broadwell NTB
    if(in_retrainInfo.PciInfo.u16VendorID == 0x8086 && in_retrainInfo.PciInfo.u16DeviceID== 0x6f0d)
    {
        return true;
    }
    else if(in_retrainInfo.PciInfo.u16VendorID == 0x10B5) //PLX
    {
        if(in_retrainInfo.u16DeviceIDDown == 0x87B0)
        {
            return true;
        }
    }

    return false;
}

bool
PCIDDAPI::isNtbDevice(UINT16  in_u16VendorID,
                      UINT16  in_u16DeviceID)
{
    //Broadwell NTB 
    if(in_u16VendorID == 0x8086 && in_u16DeviceID == 0x6f0d)
    {
        return true;
    }
    else if(in_u16VendorID == 0x10B5)
    {
        switch(in_u16DeviceID)
        {
            case 0x870B:
                return true;
            default:
                break;
        }
    }
    return false;
}


ERRORCODE 
PCIDDAPI::getRetrainSupportDevices(vector<RetrainInfo>  *out_pRetrainInfo)
{
    bool            bFlag;
    UINT32          u32Index;
    RetrainInfo     TempRetrainInfo;

    grow_tree();
    
    //create PCI objects  
    m_device_amount = 0;
    for (m_libDevice = m_libPacc -> devices; m_libDevice; m_libDevice = m_libDevice -> next)
    {
        m_pciInfo.u16Domain     = m_libDevice->domain;
        m_pciInfo.u16Bus        = m_libDevice->bus;
        m_pciInfo.u16Device     = m_libDevice->dev;
        m_pciInfo.u8Function    = m_libDevice->func;
        m_pciInfo.u16VendorID   = m_libDevice->vendor_id;
        m_pciInfo.u16DeviceID   = m_libDevice->device_id;

        m_pciInfo.szDeviceName.clear();

        m_pciDeviceList[m_device_amount] = new PCIDevice(m_pciInfo);
        
        m_device_amount++;
    }

    for(u32Index = 0; u32Index < m_device_amount; u32Index++)
    {
        bFlag = isAvailableForRetrainTest(*m_pciDeviceList[u32Index]);
        if(bFlag) //retrain supported 
        {
            //if((m_pciDeviceList[u32Index]->GetHeaderType() & 0x1) == 0x1 || bNtbDevice)
            bFlag = isAvailableForRetrainTest(*m_pciDeviceList[u32Index]);
            if(bFlag)
            {
                TempRetrainInfo.retrainResult = false; //default to false
                savePciInfoToPciStructure(*m_pciDeviceList[u32Index], &TempRetrainInfo.PciInfo);

                //TempRetrainInfo.LinkStatus = m_pciDeviceList[u32Index]->LinkStatus();
                TempRetrainInfo.LinkStatus = getLinkStatus(*m_pciDeviceList[u32Index]);
                getDownstreamDevice(&TempRetrainInfo);
                out_pRetrainInfo->push_back(TempRetrainInfo);
           }
        }
    }
    
    releasePciObjects();
    m_pciLibrary.pci_cleanup(m_libPacc);

    return NO_ERROR;
}


bool 
PCIDDAPI::isAvailableForRetrainTest(PCIDevice  in_classPciDevice)
{
    bool    bFlag;

    //NTB 
    if((in_classPciDevice.GetVendorID() == 0x8086 && in_classPciDevice.GetDeviceID() == 0x6f0d) ||
       (in_classPciDevice.GetVendorID() == 0x10B5 && in_classPciDevice.GetDeviceID() == 0x87B0)) 
    {
        return false;
    }

    bFlag = in_classPciDevice.checkRetrainCap();
    if(!bFlag)
    {
        return bFlag;
    }
    else 
    {
        (in_classPciDevice.GetHeaderType() & 0x1) == 0x1 ? bFlag = true : bFlag = false;
    }

    return bFlag;
}


ERRORCODE
PCIDDAPI::getDownstreamDevice(RetrainInfo *inout_pRetrainInfo)
{
    struct bridge   *b;
    struct device   *temp_device;
    struct pci_dev  *temp_pci_dev;
    bool            foundDevice = false;
    UINT32          u32Index = 0;
    UINT32          want_secondary = 0x0;
    UINT32          want_domain = 0x0;

    inout_pRetrainInfo->u16DomainDown   = 0xffff;
    inout_pRetrainInfo->u16BusDown      = 0xffff;
    inout_pRetrainInfo->u16DeviceDown   = 0xffff;
    inout_pRetrainInfo->u8FunctionDown  = 0xff;
    inout_pRetrainInfo->u8FunctionDown  = 0xff;
    inout_pRetrainInfo->u16VendorIDDown = 0xffff;

    inout_pRetrainInfo->szDeviceNameDown.clear();

    for(b=&m_host_bridge; b; b=b->chain)
    {
        for(temp_device = b->br_dev; temp_device; temp_device = temp_device->next)
        {
            for(temp_pci_dev = temp_device->dev; temp_pci_dev; temp_pci_dev = temp_pci_dev->next)
            {
                if(temp_pci_dev->domain == inout_pRetrainInfo->PciInfo.u16Domain &&
                    temp_pci_dev->bus == inout_pRetrainInfo->PciInfo.u16Bus &&
                    temp_pci_dev->dev == inout_pRetrainInfo->PciInfo.u16Device &&
                    temp_pci_dev->func == inout_pRetrainInfo->PciInfo.u8Function )
                {
                    want_secondary = b->secondary;
                    want_domain = b->domain;
                    foundDevice = true;
                    break;
                }
                if(foundDevice)
                {
                    break;
                }
            }
        }
    }

    if(foundDevice)
    {
        for(u32Index = 0; u32Index < m_device_amount; u32Index++)
        {
            if(m_pciDeviceList[u32Index]->GetDomain() == want_domain &&
               m_pciDeviceList[u32Index]->GetBus() == want_secondary)
            {
                inout_pRetrainInfo->u16DomainDown   = m_pciDeviceList[u32Index]->GetDomain();
                inout_pRetrainInfo->u16BusDown      = m_pciDeviceList[u32Index]->GetBus();
                inout_pRetrainInfo->u16DeviceDown   = m_pciDeviceList[u32Index]->GetDevice();
                inout_pRetrainInfo->u8FunctionDown  = m_pciDeviceList[u32Index]->GetFunction();
                inout_pRetrainInfo->u16VendorIDDown = m_pciDeviceList[u32Index]->GetVendorID();
                inout_pRetrainInfo->u16DeviceIDDown = m_pciDeviceList[u32Index]->GetDeviceID();
                break;
            }
        }
    }

    return NO_ERROR;
}

void
PCIDDAPI::initialPciInfoStructure(PCIInformantion*   out_pPciInfo)
{
    out_pPciInfo->szDeviceName.clear();
    out_pPciInfo->u16Domain     = 0xffff;
    out_pPciInfo->u16Bus        = 0xff;
    out_pPciInfo->u16Device     = 0xff;
    out_pPciInfo->u8Function    = 0xf;
    out_pPciInfo->u16VendorID   = 0xffff;
    out_pPciInfo->u16DeviceID   = 0xffff;
    out_pPciInfo->u32ClassCode  = 0xffffffff;
    out_pPciInfo->u8RevisionID  = 0xff;
    out_pPciInfo->u8HeaderType  = 0xff;
    out_pPciInfo->u8CapPtrValue = 0xff;
    out_pPciInfo->u8PortType    = 0xff;
    out_pPciInfo->u8PortNumber  = 0xff;
}


/*
void
PCIDDAPI::copyPciInfo(PCIInformantion*   out_pPciInfo)
{
    out_pPciInfo->szDeviceName.clear();
    out_pPciInfo->u16Domain     = 0xffff;
    out_pPciInfo->u16Bus        = 0xff;
    out_pPciInfo->u16Device     = 0xff;
    out_pPciInfo->u8Function    = 0xf;
    out_pPciInfo->u16VendorID   = 0xffff;
    out_pPciInfo->u16DeviceID   = 0xffff;
    out_pPciInfo->u32ClassCode  = 0xffffffff;
    out_pPciInfo->u8RevisionID  = 0xff;
    out_pPciInfo->u8HeaderType  = 0xff;
    out_pPciInfo->u8CapPtrValue = 0xff;
    out_pPciInfo->u8PortType    = 0xff;
    out_pPciInfo->u8PortNumber  = 0xff;
}
*/

void 
PCIDDAPI::getDownstreamDevice(SLOT_INFO         in_slotInfo,
                              PCIInformantion*  out_pPciInfo)
{
    struct bridge   *b;
    struct device   *temp_device;
    struct pci_dev  *temp_pci_dev;
    bool            foundDevice = false;
    UINT32          u32Index = 0;
    UINT32          want_secondary = 0x0;
    UINT32          want_domain = 0x0;

    for(b=&m_host_bridge; b; b=b->chain)
    {
        for(temp_device = b->br_dev; temp_device; temp_device = temp_device->next)
        {
            for(temp_pci_dev = temp_device->dev; temp_pci_dev; temp_pci_dev = temp_pci_dev->next)
            {
                if(temp_pci_dev->domain == in_slotInfo.u16Domain &&
                   temp_pci_dev->bus == in_slotInfo.u16Bus &&
                   temp_pci_dev->dev == in_slotInfo.u16Device &&
                   temp_pci_dev->func == in_slotInfo.u8Function)
                {
                    want_secondary = b->secondary;
                    want_domain = b->domain;
                    foundDevice = true;
                    break;
                }
                if(foundDevice)
                {
                    break;
                }
            }
        }
    }

    if(foundDevice)
    {
        for(u32Index = 0; u32Index < m_device_amount; u32Index++)
        {
            if(m_pciDeviceList[u32Index]->GetDomain() == want_domain &&
               m_pciDeviceList[u32Index]->GetBus() == want_secondary)
            {
                out_pPciInfo->szDeviceName.clear();
                out_pPciInfo->u16Domain     = m_pciDeviceList[u32Index]->GetDomain();
                out_pPciInfo->u16Bus        = m_pciDeviceList[u32Index]->GetBus();
                out_pPciInfo->u16Device     = m_pciDeviceList[u32Index]->GetDevice();
                out_pPciInfo->u8Function    = m_pciDeviceList[u32Index]->GetFunction();
                out_pPciInfo->u16VendorID   = m_pciDeviceList[u32Index]->GetVendorID();
                out_pPciInfo->u16DeviceID   = m_pciDeviceList[u32Index]->GetDeviceID();
                break;
            }
        }
    }
}




void 
PCIDDAPI::savePciInfoToPciStructure(PCIDevice           in_classPciDevice,
                                    PCIInformantion     *pciInfo)
{
    pciInfo->u16Domain      = in_classPciDevice.GetDomain();
    pciInfo->u16Bus         = in_classPciDevice.GetBus();
    pciInfo->u16Device      = in_classPciDevice.GetDevice();
    pciInfo->u8Function     = in_classPciDevice.GetFunction();
    pciInfo->u16VendorID    = in_classPciDevice.GetVendorID();
    pciInfo->u16DeviceID    = in_classPciDevice.GetDeviceID();
    pciInfo->u32ClassCode   = in_classPciDevice.GetClassCode();
    pciInfo->u8RevisionID   = in_classPciDevice.GetRevisionID();
    pciInfo->u8HeaderType   = in_classPciDevice.GetHeaderType();
    pciInfo->u8CapPtrValue  = in_classPciDevice.GetCapPtrValue();
}


ErrorMsg* 
PCIDDAPI::getPciInfoByClass(PCIClassCode             in_Class,
                            vector<PCIInformantion>  *out_pPciInfo)
{
    vector<PCIInformantion>             pciList;
    vector<PCIInformantion>::iterator   it;

    bool        deviceCount = 0;
    
    //printf("PCIDDAPI::getPciInfoByClass, in_Class: 0x%x\n", in_Class);//check 

    getDeviceList(&pciList, NOT_SORT_PCI_DEVICES); //not sort

    for(it = pciList.begin(); it != pciList.end(); ++it)
    {
        //printf("PCIDDAPI::getPciInfoByClass, it->u32ClassCode: 0x%x\n", it->u32ClassCode);//check 
        if(it->u32ClassCode == in_Class)
        {
            out_pPciInfo->push_back(*it);
            deviceCount++;
        }
    }

    if(deviceCount == 0)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PCI_DDAPI_GET_PCI_INFO | ERROR_PCI_DDAPI_NO_DEVICE_FOUND), NULL, NULL);
    }
    
    getDevicesName(out_pPciInfo);

    return NO_ERROR_MSG;
}


ErrorMsg*
PCIDDAPI::getBarByBDF(PFA       in_bdf,
                      UINT32    in_u32BarIndex,
                      UINT32    *out_u32BAR,
                      UINT32    *out_u32BarSize)
{
    UINT32                      u32Index;
    vector<UINT32>              BarList;
    vector<UINT32>::iterator    it;
    ERRORCODE                   eErrorCode;
    UINT32                      u32SaveData;
    UINT32                      u32BarSize;

    *out_u32BAR = 0x0;
    *out_u32BarSize = 0x0;
    
    eErrorCode = checkPresenceOfDevice(in_bdf); 
    if(eErrorCode != NO_ERROR)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PCI_DDAPI_GET_BAR | eErrorCode), NULL, NULL);
    }

    createPciObject(in_bdf); 

    m_pciDevice->GetBARList(&BarList);

    u32Index = 0;
    for(it = BarList.begin(); it != BarList.end(); ++it)
    {
        //printf("bar %d -- 0x%x\n", u32Index, *it);//check 

        if(u32Index == in_u32BarIndex)
        {
            u32SaveData = m_pciDevice->ConfigReadDword(PCI_BAR_START_OFFSET + u32Index*4);
            m_pciDevice->ConfigWriteDword(PCI_BAR_START_OFFSET + u32Index*4, 0xffffffff);
            u32BarSize = m_pciDevice->ConfigReadDword(PCI_BAR_START_OFFSET + u32Index*4);
            u32BarSize = ~(u32BarSize & 0xfffffff0) + 1;
            //printf("bar size: %x\n", u32BarSize);//check 
            *out_u32BarSize = u32BarSize;
            //restore data 
            m_pciDevice->ConfigWriteDword(PCI_BAR_START_OFFSET + u32Index*4, u32SaveData);
            //printf("data check: 0x%x\n", m_pciDevice->ConfigReadDword(PCI_BAR_START_OFFSET + u32Index*4)); //check 

            *out_u32BAR = *it & 0xfffffff0;
            releasePciObject();
            return NO_ERROR_MSG;
        }
        u32Index++;
    }

    releasePciObject();
    return returnErrorMsg((ERRORCODE)(ERROR_PCI_DDAPI_GET_BAR | ERROR_ILLEGAL_REQUEST), NULL, NULL);
}


ErrorMsg*
PCIDDAPI::getBarByBDF(UINT16    in_u16Domain,
                      UINT16    in_u16Bus,
                      UINT16    in_u16Device,
                      UINT8     in_u8Function,
                      UINT32    in_u32BarIndex,
                      UINT32    *out_u32BAR)
{
    UINT32                      u32Index;
    vector<UINT32>              BarList;
    vector<UINT32>::iterator    it;
    ERRORCODE                   eErrorCode;
    PFA                         bdf;

    *out_u32BAR = 0x0;
    
    bdf.u16Domain   = in_u16Domain;
    bdf.u16Bus      = in_u16Bus;
    bdf.u16Device   = in_u16Device;
    bdf.u8Function  = in_u8Function;

    eErrorCode = checkPresenceOfDevice(bdf);
    if(eErrorCode != NO_ERROR)
    {
        return returnErrorMsg((ERRORCODE)(ERROR_PCI_DDAPI_GET_BAR | eErrorCode), NULL, NULL);
    }

    createPciObject(bdf);

    m_pciDevice->GetBARList(&BarList);

    u32Index = 0;
    for(it = BarList.begin(); it != BarList.end(); ++it)
    {
        //printf("bar %d -- 0x%x\n", u32Index, *it);//check 

        if(u32Index == in_u32BarIndex)
        {
            *out_u32BAR = *it & 0xfffffff0;
            releasePciObject();
            return NO_ERROR_MSG;
        }
        u32Index++;
    }

    releasePciObject();
    return returnErrorMsg((ERRORCODE)(ERROR_PCI_DDAPI_GET_BAR | ERROR_ILLEGAL_REQUEST), NULL, NULL);
}


void 
PCIDDAPI::JustTest()
{
    list<unsigned int>  BarList;
    struct device *d;
    struct device *first_dev = NULL;

    list<unsigned int>::iterator it;

    PCICommonFunction   PCILibrary;

    m_libPacc = m_pciLibrary.pci_alloc();
    m_pciLibrary.sysfs_init(m_libPacc);
    m_pciLibrary.sysfs_scan(m_libPacc);

    m_device_amount = 0;

    for (m_libDevice = m_libPacc -> devices; m_libDevice; m_libDevice = m_libDevice -> next)   //Iterate over all devices
    {
        if ( (d = PCILibrary.scan_device(m_libDevice)) )
        {
            d->next = first_dev;
            first_dev = d;
        }
         
        m_device_amount++;
    }
   
    //check 
    struct device   *dd;
    struct pci_dev  *pciDev;
    for(dd = first_dev; dd; dd=dd->next)
    {
        struct pci_dev *dev = dd->dev;
        printf("%04x:%02x:%02x.%x\n", dev->domain, dev->bus, dev->dev, dev->func);
        
        for(pciDev = dd->dev; pciDev; pciDev = pciDev->next)
        {
            printf("---------------------------------\n");
        }
    }

    m_pciLibrary.pci_cleanup(m_libPacc);


    /*
    GetIDByBDF(0x0,0x0,0x3,0x0, &u32LogicalID);
    m_pciDeviceList[u32LogicalID]->GetBARList(&BarList);

    for(it = BarList.begin(); it != BarList.end(); ++it)
    {
        printf("BAR: 0x%x\n", *it);
    }
    */
}


#if 1
void
PCIDDAPI::grow_tree()
{
    UINT32  u32Class;
    UINT8   u8HeaderType;

    struct device *d, *d2;
    struct bridge **last_br, *b;
    
    m_first_dev = NULL;

    m_libPacc = m_pciLibrary.pci_alloc();
    m_pciLibrary.sysfs_init(m_libPacc);
    m_pciLibrary.sysfs_scan(m_libPacc);
  
    //Iterate over all devices 
    for (m_libDevice = m_libPacc -> devices; m_libDevice; m_libDevice = m_libDevice -> next)
    {
        if ( (d = m_pciLibrary.scan_device(m_libDevice))  )
        {
            d->next = m_first_dev;
            m_first_dev = d;
        }
    }
   
    //Build list of bridges
    last_br = &m_host_bridge.chain;

    for(d = m_first_dev; d; d=d->next) //m_first_dev is created in scanDevicesByPciLibrary() 
    {
        u32Class = m_pciLibrary.ConfigReadDword(d->dev->domain, d->dev->bus, d->dev->dev, d->dev->func, 0x8);
        u32Class = u32Class >> 16;

        u8HeaderType = m_pciLibrary.ConfigReadByte(d->dev->domain, d->dev->bus, d->dev->dev, d->dev->func, 0xe);
        u8HeaderType = u8HeaderType & 0x7f;
       
        ////check
        //printf("%04x:%02x:%02x.%x - class: 0x%x, header: 0x%x\n",
        //       d->dev->domain, d->dev->bus, d->dev->dev, d->dev->func, 
        //       u32Class, u8HeaderType);
        
        if(u32Class == PCI_CLASS_BRIDGE_PCI &&
                (u8HeaderType == PCI_HEADER_TYPE_BRIDGE || u8HeaderType == PCI_HEADER_TYPE_CARDBUS))
        {
            b = (struct bridge*)m_pciLibrary.xmalloc(sizeof(struct bridge));
            b->domain = d->dev->domain;
            b->primary = m_pciLibrary.ConfigReadByte(d->dev->domain, d->dev->bus, d->dev->dev, d->dev->func, 0x18);
            b->secondary = m_pciLibrary.ConfigReadByte(d->dev->domain, d->dev->bus, d->dev->dev, d->dev->func, 0x19);
            b->subordinate = m_pciLibrary.ConfigReadByte(d->dev->domain, d->dev->bus, d->dev->dev, d->dev->func, 0x1a);
            
            //check
            /*
            printf("domain: %x\n", b->domain);
            printf("primary: %x\n", b->primary);
            printf("secondary: %x\n", b->secondary);
            printf("subordinate: %x\n", b->subordinate);
            printf("------------------------------------------\n");
            */

            *last_br = b;
            last_br = &b->chain;
            b->next = b->child = NULL;
            b->first_bus = NULL;
            b->br_dev = d;
        }
    }
    *last_br = NULL;

    //Create a bridge tree
    for(b = &m_host_bridge; b; b=b->chain)
    {
        struct bridge *c, *best;
        best = NULL;
        for (c=&m_host_bridge; c; c=c->chain)
            if (c != b && (c == &m_host_bridge || b->domain == c->domain) &&
                    b->primary >= c->secondary && b->primary <= c->subordinate &&
                    (!best || best->subordinate - best->primary > c->subordinate - c->primary))
                best = c;
        if(best)
        {
            b->next = best->child;
            best->child = b;
        }
    }

    //Insert secondary bus for each bridge
    for (b=&m_host_bridge; b; b=b->chain)
    {
        //check 
        //printf("domain: %x\n", b->domain);
        //printf("primary: %x\n", b->primary);
        //printf("secondary: %x\n", b->secondary);
        //printf("subordinate: %x\n", b->subordinate);
        //printf("------------------------------------------\n");

        if (!m_pciLibrary.find_bus(b, b->domain, b->secondary))
            m_pciLibrary.new_bus(b, b->domain, b->secondary);
    }
    
    
    //Create bus structs and link devices
    for (d = m_first_dev; d;)
    {
        d2 = d->next;
        m_pciLibrary.insert_dev(d, &m_host_bridge);
        d = d2;
    }
    
    //check
    /*
    cout << "-------- check m_first_dev ----------" << endl; //check 
    struct device   *dd;
    for(dd = m_first_dev; dd; dd=dd->next)
    {
        struct pci_dev *dev = dd->dev;
        printf("%04x:%02x:%02x.%x\n", dev->domain, dev->bus, dev->dev, dev->func);
    }
    */
}
#endif


int 
PCIDDAPI::id_white_p(int c)
{
    return (c == ' ') || (c == '\t');
}

int 
PCIDDAPI::id_hex(char *p, int cnt)
{
    int x = 0;
    while (cnt--)
    {
        x <<= 4;
        if (*p >= '0' && *p <= '9')
        {
             x += (*p - '0');
        }
        else if (*p >= 'a' && *p <= 'f')
        {
            x += (*p - 'a' + 10);
        }
        else if (*p >= 'A' && *p <= 'F')
        {
            x += (*p - 'A' + 10);
        }
        else
            return -1;
        p++;
    }

    return x;
}


#if 0
//unsigned int //need to change to ERRORCODE !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
const char*
PCIDDAPI::getVendorName(UINT16 in_u16RequestID)
{
    char    requestID[5], temp[5];
    char*   p, p1;
    char    line[MAX_LINE];
    int     lino;
    int     nest;

    int     s32Index;
    int     s32Hex;

    unsigned char id1;
    FILE* pciIdFile;

    string strVendorID;
    
    pciIdFile = fopen("pci.ids", "r");
    if(!pciIdFile)
    {
        cout << "fail to open pci.ids" << endl;
        return "fail_to_open_pci.ids"; //need to change !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    }

    sprintf(requestID, "%x", in_u16RequestID);
    cout << "requestID: " << requestID << endl;

    lino = 0;
    //while(fgets(line, sizeof(line), pciIdFile) != NULL && lino < 40)
    while(fgets(line, sizeof(line), pciIdFile) != NULL)
    {
        lino++;

        p = line;

        //printf("p[0]: %c\n", p[0]);
        //printf("p[1]: %c\n", p[1]);
        //printf("p[2]: %c\n", p[2]);
        //printf("p[3]: %c\n", p[3]);
        //printf("p[4]: %c\n", p[4]);
        //printf("p[5]: %c\n", p[5]);
      
        if (!*p || *p == '#' || *p == '\n')
        //if (*p == '#')
        {
            continue;
        }

        if(p[0] == '\t' || (p[0] == '\t' && p[1] == '\t'))
        {
            continue;
        }

        //printf(line);
        
        sprintf(temp, "%c%c%c%c", p[0],p[1],p[2],p[3]);
        printf("temp: %s\n", temp);//check 
      
        if(strcmp(temp, requestID) == 0)
        {
            cout << "Intel!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" << endl;
            return p+6;
        }
        /*
        if(p[0] == '8' && p[1] == '0' && p[2] == '8' && p[3] == '6')
        {
            printf(line);

            nest = p - line;
            cout << "nest 1:" << nest << endl;
            if(!nest)
            {
                cout << "--------------------------------" << endl;
            }
            while(*p != NULL)
            {
                cout << *p << endl;
                *p++;
            }
            nest = p - line;
            cout << "nest 2:" << nest << endl;
            if(!nest)
            {
                cout << "-----------------@@@@---------------" << endl;
            }
            //p1 = strstr(line, "8086");
            //printf("Vendor ID: %s\n", p1);
        }
        */

        //if(*p == '\t')
        //{
         //   continue;
        //}
        //if (!*p || *p == '#')
        //{
        //    continue;
        //}

        //printf(line);

        //while (*p && *p != '\n' && *p != '\r')
        //{
        //    p++;
        //}

        //if (!*p && !feof(pciIdFile))
        //{
        //    cout << "Line too long" << endl;;
        //}

        //*p = 0;

        //if (p > line && (p[-1] == ' ' || p[-1] == '\t'))
        //{
        //    *--p = 0;
        //}

        //p = line;
        //while (id_white_p(*p))
        //{
        //    p++;
        //}

        //if (!*p || *p == '#')
        //{
        //    continue;
        //}

        //p = line;
        //while (*p == '\t')
        //{
        //    p++;
        //}


        //nest = p - line;
        //if (!nest)
        //{
        //    if (p[0] == 'C' && p[1] == ' ') //Class block
        //    {
        //        printf("line: %d, This is Class block!!!!!\n", lino);//check 
        //        //if ((id1 = id_hex(p+2, 2)) < 0 || !id_white_p(p[4]))
        //        //{
        //        //    return 1;
        //        //}
        //        //cat = ID_CLASS;
        //        p += 5;
        //    }
        //}
        //printf(line);
    }
    printf("\n");

    fclose(pciIdFile);

    cout << "read pci.ids ok" << endl;
    return "unknown"; //need to change !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
}
#endif 


#if 0
const char*
PCIDDAPI::getDeviceName(UINT16 in_u16RequestID)
{
    char    requestID[5], temp[5];
    char*   p, p1;
    char    line[MAX_LINE];
    int     lino;
    int     nest;

    int     s32Index;
    int     s32Hex;

    unsigned char id1;
    FILE* pciIdFile;

    string strVendorID;
    
    pciIdFile = fopen("pci.ids", "r");
    if(!pciIdFile)
    {
        return "fail_to_open_pci.ids"; 
    }

    sprintf(requestID, "%x", in_u16RequestID);
    cout << "requestID: " << requestID << endl; //check 

    lino = 0;
    //while(fgets(line, sizeof(line), pciIdFile) != NULL && lino < 40)
    while(fgets(line, sizeof(line), pciIdFile) != NULL)
    {
        lino++;

        p = line;

        if (!*p || *p == '#' || *p == '\n')
        {
            continue;
        }

        if(p[0] == '\t' &&  p[1] != '\t')
        {
            printf(line);
        }

        //sprintf(temp, "%c%c%c%c", p[0],p[1],p[2],p[3]);
        //printf("temp: %s\n", temp);//check 
      
        //if(strcmp(temp, requestID) == 0)
        //{
        //    cout << "Intel!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" << endl;
        //    return p+6;
        //}
        /*
        if(p[0] == '8' && p[1] == '0' && p[2] == '8' && p[3] == '6')
        {
            printf(line);

            nest = p - line;
            cout << "nest 1:" << nest << endl;
            if(!nest)
            {
                cout << "--------------------------------" << endl;
            }
            while(*p != NULL)
            {
                cout << *p << endl;
                *p++;
            }
            nest = p - line;
            cout << "nest 2:" << nest << endl;
            if(!nest)
            {
                cout << "-----------------@@@@---------------" << endl;
            }
            //p1 = strstr(line, "8086");
            //printf("Vendor ID: %s\n", p1);
        }
        */

        //if(*p == '\t')
        //{
         //   continue;
        //}
        //if (!*p || *p == '#')
        //{
        //    continue;
        //}

        //printf(line);

        //while (*p && *p != '\n' && *p != '\r')
        //{
        //    p++;
        //}

        //if (!*p && !feof(pciIdFile))
        //{
        //    cout << "Line too long" << endl;;
        //}

        //*p = 0;

        //if (p > line && (p[-1] == ' ' || p[-1] == '\t'))
        //{
        //    *--p = 0;
        //}

        //p = line;
        //while (id_white_p(*p))
        //{
        //    p++;
        //}

        //if (!*p || *p == '#')
        //{
        //    continue;
        //}

        //p = line;
        //while (*p == '\t')
        //{
        //    p++;
        //}


        //nest = p - line;
        //if (!nest)
        //{
        //    if (p[0] == 'C' && p[1] == ' ') //Class block
        //    {
        //        printf("line: %d, This is Class block!!!!!\n", lino);//check 
        //        //if ((id1 = id_hex(p+2, 2)) < 0 || !id_white_p(p[4]))
        //        //{
        //        //    return 1;
        //        //}
        //        //cat = ID_CLASS;
        //        p += 5;
        //    }
        //}
        //printf(line);
    }
    printf("\n");

    fclose(pciIdFile);

    cout << "read pci.ids ok" << endl;
    return "unknown"; //need to change !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
}
#endif 

ERRORCODE 
PCIDDAPI::getPciClassMappingTable(list<PCI_CLASS_MAPPING> *out_pClassList)
{
    char    temp[5];
    char*   ptr;
    char    line[MAX_LINE];
    int     lino = 0;
    bool    bClassStart = false;

    FILE*   pciIdFile;

    PCI_CLASS_MAPPING   tempPciClass;

    pciIdFile = fopen("/usr/share/pci.ids", "r");
    if(!pciIdFile)
    {
        cout << "fail_to_open_pci.ids" << endl; //check
        return ERROR_OPEN_FILE;
    }

    while(fgets(line, sizeof(line), pciIdFile) != NULL)
    {
        lino++;

        ptr = line;

        if (!*ptr || *ptr == '#' || *ptr == '\n')
        {
            continue;
        }

        tempPciClass.u8BaseClass = 0x0;
        tempPciClass.u8SubClass = 0x0;
        tempPciClass.u8ProgrammingInterface = 0x0;
        tempPciClass.strBassClass.assign(" ");
        tempPciClass.strSubClass.assign(" ");
        tempPciClass.strProgrammingInterface.assign(" ");

        //replace the last character '\n' of each line to 0x0
        while(*ptr != '\n')
        {
             ptr++;
        }
        *ptr = 0x0;
            
        ptr = line;

        if(*ptr == 'C')
        {
            bClassStart = true;
            sprintf(temp, "%c%c", ptr[2],ptr[3]);
            //cout << "temp: " << temp << endl;//check 
            tempPciClass.u8BaseClass = strtol(temp, NULL, 16);
            //cout << "tempPciNameInfo.u16VendorID: " << tempPciNameInfo.u16VendorID << endl; //check 
            tempPciClass.strBassClass.assign(ptr+6);
        }

        if(bClassStart)
        {
            if(ptr[0] == '\t')
            {
                sprintf(temp, "%c%c", ptr[1],ptr[2]);
                tempPciClass.u8SubClass = strtol(temp, NULL, 16);
                tempPciClass.strSubClass.assign(ptr+5);
            }
            else if(ptr[0] == '\t' && ptr[1] == '\t')
            {
                sprintf(temp, "%c%c", ptr[1],ptr[2]);
                tempPciClass.u8ProgrammingInterface = strtol(temp, NULL, 16);
                tempPciClass.strProgrammingInterface.assign(ptr+6);
            }
            else
            {
                //do nothing
            }
        }
       
        if(bClassStart)
        {
            out_pClassList->push_back(tempPciClass);
        }

    }//end of while loop
    fclose(pciIdFile);

    /*
    //check
    list<PCI_CLASS_MAPPING>::iterator it;
    for(it = out_pClassList->begin(); it != out_pClassList->end(); ++it)
    {
        printf("BassClass: %02x\n", it->u8BaseClass);
        printf("SubClass: %02x\n", it->u8SubClass);
        printf("ProgrammingInterface: %02x\n", it->u8ProgrammingInterface);
        printf("BassClassName: %s\n", it->strBassClass.c_str());
        printf("SubClassName: %s\n", it->strSubClass.c_str());
        printf("ProgrammingInterfaceName: %s\n", it->strProgrammingInterface.c_str());
    }
    */

    return NO_ERROR; 
}


ERRORCODE
PCIDDAPI::convertStringToBDF(string in_strParameter,
                             PFA*   out_pBDF)
{
    string  strHexHeader = "0x";
    string  strTemp;
    string  strBDF;
    UINT32  s32ReturnPosition;
    UINT32  u32Temp;

    s32ReturnPosition = in_strParameter.find(':', 0);
    if(s32ReturnPosition == 2)
    {
        strBDF = "0000:";
        strBDF.append(in_strParameter);
    }
    else
    {
        strBDF = in_strParameter;
    }
    //printf("PCIOperation::convertStringToBDF, strBDF: %s\n", strBDF.c_str());//check 
    
    //get domain 
    strTemp = strBDF.substr(0,4);
    strHexHeader.append(strTemp);
    sscanf(strHexHeader.c_str(), "%x", &u32Temp);
    out_pBDF->u16Domain = (UINT16)u32Temp;
    strHexHeader = "0x"; //reset hex header 


    //get bus 
    strTemp = strBDF.substr(5,2);
    strHexHeader.append(strTemp);
    sscanf(strHexHeader.c_str(), "%x", &u32Temp);
    out_pBDF->u16Bus = (UINT16)u32Temp;
    strHexHeader = "0x"; //reset hex header

    //get device
    strTemp = strBDF.substr(8,2);
    strHexHeader.append(strTemp);
    sscanf(strHexHeader.c_str(), "%x", &u32Temp);
    out_pBDF->u16Device = (UINT16)u32Temp;
    strHexHeader = "0x"; //reset hex header

    //get function 
    strTemp = strBDF.substr(11,1);
    strHexHeader.append(strTemp);
    sscanf(strHexHeader.c_str(), "%x", &u32Temp);
    out_pBDF->u8Function = (UINT8)u32Temp;

    return NO_ERROR; 
}


ERRORCODE
PCIDDAPI::createSlotInfo(vector<SLOT_INFO>*     out_pSlotInfoList,
                         UINT32*                out_u32SlotAmount,
                         vector<SUPPORTED_HBA>* out_pHbaList)
{
    FILE            *pFile;
    char            line[500], tag[20];
    //char            temp[100];
    char            *tempPtr;// *tempPtr2;
    char            filePath[64];
    int             tag_number, string_length;
    PFA             bdf;
    SLOT_INFO       tempSlotInfo;
    UINT32          u32SlotAmount = 0;
    SUPPORTED_HBA   hbaInfo;
    UINT32          u32Index;
    UINT32          u32SpaceIndex;
    string          tempString;

    sprintf(filePath, "%sPCI_profile.ini", LEMUR_CONFIG_DIR);
    //sprintf(filePath, "./PCI_profile.ini", LEMUR_CONFIG_DIR); //test !!!!!!!!!!!!!!!!!!!
    pFile = fopen (filePath, "r");
    if (pFile == NULL)
    {
        return ERROR_OPEN_FILE; 
    }

    //convert ini to m_expHbaList
    while(fgets(line, 500, pFile) != NULL)
    {
        if (line[0] == '#')
            continue;

        tag_number = sscanf(line, " [%[^]]", tag); //paser the teg
        if ( (strlen(line) == 1 && line[0] == '\n') || (tag_number != 0) )
        {
            continue;
        }
        //printf("Tag(%d):%s line:%s",tag_number, tag, line); //check 

        //Remove the '\n' at the end of line 
        string_length = strlen(line);
        if (line[string_length-1] == '\n')
        {
            line[string_length-1] = '\0';
        }

        if(strcmp(tag, "slot_Info") == 0)
        {
            tempPtr = strtok(line, " ");
            u32SlotAmount++;
            convertStringToBDF(tempPtr, &bdf);
            //convertStringToBDF(line, &bdf);
            tempSlotInfo.u16Domain  = bdf.u16Domain;
            tempSlotInfo.u16Bus     = bdf.u16Bus;
            tempSlotInfo.u16Device  = bdf.u16Device;
            tempSlotInfo.u8Function = bdf.u8Function;
            tempPtr = strtok(NULL, " ");
            tempSlotInfo.slotType = tempPtr;
            out_pSlotInfoList->push_back(tempSlotInfo);
        }
        if(strcmp(tag, "supported_HBA") == 0)
        {
            tempPtr = strtok(line, " ");
            hbaInfo.u16VendorID = strtol(tempPtr, NULL, 16);
            tempPtr = strtok(NULL, " ");
            hbaInfo.u16DeviceID = strtol(tempPtr, NULL, 16);
            tempPtr = strtok(NULL, ":");
            tempString = tempPtr;
            
            u32SpaceIndex = 0;
            for(u32Index = 0; u32Index < tempString.length(); u32Index++)
            {
                if(tempString[u32Index] != 0x20)
                {
                    hbaInfo.hbaName.assign(tempString, u32Index, tempString.length());
                    break;
                }
            }
            tempPtr = strtok(NULL, ":");
            hbaInfo.description = tempPtr;
            out_pHbaList->push_back(hbaInfo);
        }
    }
    fclose(pFile);

    *out_u32SlotAmount = u32SlotAmount;
    return NO_ERROR;
}


ERRORCODE
PCIDDAPI::getPciDeviceNameMappingTable()
{
    char    temp[5];
    char*   ptr;
    char    line[MAX_LINE];
    int     lino = 0;
    FILE*   pciIdFile;

    PCI_NAME_MAPPING    tempPciNameInfo;

    pciIdFile = fopen("/usr/share/pci.ids", "r");
    if(!pciIdFile)
    {
        cout << "fail_to_open_pci.ids" << endl; //check
        return ERROR_OPEN_FILE;
    }

    while(fgets(line, sizeof(line), pciIdFile) != NULL)
    {
        lino++;

        ptr = line;

        if (!*ptr || *ptr == '#' || *ptr == '\n' || *ptr == 'C' || (ptr[0] == '\t' && ptr[1] == '\t'))
        {
            continue;
        }

        //replace the last character '\n' of each line to 0x0
        while(*ptr != '\n')
        {
            ptr++;
        }
        *ptr = 0x0;
        ptr = line;

        tempPciNameInfo.u16VendorID = 0x0;
        tempPciNameInfo.u16DeviceID = 0x0;
        tempPciNameInfo.strVendorName.clear();
        tempPciNameInfo.strDeviceName.clear();

        if(ptr[0] != '\t')
        {
            sprintf(temp, "%c%c%c%c", ptr[0],ptr[1],ptr[2],ptr[3]);
            //cout << "temp: " << temp << endl;//check 
            tempPciNameInfo.u16VendorID = strtol(temp, NULL, 16);
            //cout << "tempPciNameInfo.u16VendorID: " << tempPciNameInfo.u16VendorID << endl; //check 
            tempPciNameInfo.strVendorName.assign(ptr+6);
        }
        else //ptr[0] == '\t'
        {
            sprintf(temp, "%c%c%c%c", ptr[1],ptr[2],ptr[3],ptr[4]);
            tempPciNameInfo.u16DeviceID = strtol(temp, NULL, 16);
            tempPciNameInfo.strDeviceName.assign(ptr+7);
        }
      
        m_pciNameList.push_back(tempPciNameInfo);
    }

    fclose(pciIdFile);

#if 0
    //check
    list<PCI_NAME_MAPPING>::iterator it;
    for(it = m_pciNameList.begin(); it != m_pciNameList.end(); ++it)
    {
        printf("VendorID: %04x\n", it->u16VendorID);
        printf("DeviceID: %04x\n", it->u16DeviceID);
        printf("VendorName: %s\n", it->strVendorName.c_str());
        printf("DeviceName: %s\n", it->strDeviceName.c_str());
    }
#endif 
    return NO_ERROR; 
}



//for domain 0 only 
ErrorMsg*
PCIDDAPI::scanDevicesFromIO(vector<PCIInformantion>*    out_pPciList)
{
    int     ret;
    UINT16  u16Bus;
    UINT16  u16Dev;
    UINT8   u8Func;
    PFA     bdf;
    UINT32  u32Data = 0x0;
    UINT32  u32Offset;

    PCIInformantion   pciInfo;

    ret = iopl(3);
    if(ret < 0)
    {
        return returnErrorMsg(ERROR_PCI_DDAPI_GET_BAR, "iopl fail", NULL); 
    }

    for(u16Bus = 0; u16Bus < 256; u16Bus++)
    {
        for(u16Dev = 0; u16Dev < 32; u16Dev++)
        {
            for(u8Func = 0; u8Func < 8; u8Func++)
            {
                u32Offset = 0x80000000 + (u16Bus * 0x10000)
                                       + (u16Dev * 0x800)
                                       + (u8Func * 0x100)
                                       + 0;
                
                outl(u32Offset, 0xcf8);
                u32Data = inl(0xcfc);
                if(u32Data != 0xffffffff)
                {
                    //printf("%02x:%02x.%x --- 0x%x\n", u16Bus, u16Dev, u8Func, u32Data); //check 
                    bdf.u16Domain   = 0x0;
                    bdf.u16Bus      = u16Bus;
                    bdf.u16Device   = u16Dev;
                    bdf.u8Function  = u8Func;
                    createPciObject(bdf);
                    savePciInfoToPciStructure(*m_pciDevice, &pciInfo);
                    pciInfo.u16VendorID = u32Data & 0xffff;
                    pciInfo.u16DeviceID = (u32Data & 0xffff0000) >> 16;
                    out_pPciList->push_back(pciInfo);
                    releasePciObject();
                }
            }
        }
    }

    iopl(0);

    return NO_ERROR_MSG;
}

ErrorMsg* 
PCIDDAPI::showEepromImageVersion(string   in_strEepromName)
{
    PFA                 bdf;
    UINT16              u16Temp;
    UINT32              u32Index;
    UINT32              u32PortNumber;
    PHYSICAL_ADDRESS_T  portBAR;
    vector<UINT32>      BarList;
    EEPROM_INFO         eepromInfo;
        
    m_pErrorMsg = getEeprom(in_strEepromName, &eepromInfo);
    if(m_pErrorMsg != NO_ERROR_MSG)
    {
        return returnErrorMsg(ERROR_PCI_DDAPI_SHOW_EEPROM_FW_VER, NULL, m_pErrorMsg);
    }
    
    //get PLX BAR 
    createPciObjects();
    for(u32Index = 0; u32Index < m_device_amount; u32Index++)
    {
        if(m_pciDeviceList[u32Index]->getPortTypeRawData() == PORT_TYPE_PCIE_SWITCH_UPSTREAM_PORT)
        {
            m_pciDeviceList[u32Index]->GetBARList(&BarList);
            m_u64PciBar = ((UINT64)BarList[1]) << 32 | BarList[0];
            break;
        }
    }
    releasePciObjects();

    //get version 
    bdf.u16Domain   = eepromInfo.u16Domain;
    bdf.u16Bus      = eepromInfo.u16Bus;
    bdf.u16Device   = eepromInfo.u16Device;
    bdf.u8Function  = eepromInfo.u8Function;

    createPciObject(bdf);

    u32PortNumber = m_pciDevice->getPortNumber();
    portBAR = m_u64PciBar + u32PortNumber * 0x1000;
    u16Temp = m_memoryDriver->read16(portBAR + 0x20C);
    printf("EEPROM Image version: %02d.%02d\n", 
            ((u16Temp & 0xff00) >> 8), 
            u16Temp & 0xff) ;

    releasePciObject();
    
    return NO_ERROR_MSG;
}


void
PCIDDAPI::showPciDefinedTypes()
{
    UINT32  u32Index;

    cout << "PCI defined types:" << endl;
    
    u32Index = 0;
    while(PCIClassInformation[u32Index].code != 0xffff)
    {
        cout << PCIClassInformation[u32Index].szShortName << endl;
        u32Index++;
    }
}


/*
ERRORCODE
PCIDDAPI::getPciDeviceName(UINT16   in_u16VendorID,
                           UINT16   in_u16DeviceID,
                           string   *out_strVendorName,
                           string   *out_strDeviceName)
{
    printf("PCIDDAPI::getPciDeviceName\n");//check 

    char    requestID[5], temp[5];
    char*   pVendor;
    char*   pDevice;
    char    line[MAX_LINE];
    int     lino = 0;
    int     device_lino = 0;;
    int     nest;

    int     s32Index;
    int     s32Hex;

    FILE*   pciIdFile;
    bool    VendorFound = false;

    pciIdFile = fopen("pci.ids", "r");
    if(!pciIdFile)
    {
        cout << "fail_to_open_pci.ids" << endl; //check
        return ERROR_NO_DEVICE_FOUND;
    }

    //get vendor name by vendor ID 
    sprintf(requestID, "%x", in_u16VendorID);
    cout << "Vendor ID: " << requestID << endl; //check 

    while(fgets(line, sizeof(line), pciIdFile) != NULL)
    {
        lino++;

        pVendor = line;

        if (!*pVendor || *pVendor == '#' || *pVendor == '\n')
        {
            continue;
        }
        if(pVendor[0] == '\t' || (pVendor[0] == '\t' && pVendor[1] == '\t'))
        {
            continue;
        }
        
        printf(line);
        
        sprintf(temp, "%c%c%c%c", pVendor[0],pVendor[1],pVendor[2],pVendor[3]);
        printf("temp: %s\n", temp);//check 
      
        if(strcmp(temp, requestID) == 0)
        {
            cout << "Intel!!!!!!!!!!!!!!!!!!!!!!!!!!!!!" << endl;
            out_strVendorName->assign(temp);
            VendorFound = true;
            break;
        }
    }

    if(!VendorFound)
    {
        out_strVendorName->assign("unknown");
        out_strDeviceName->assign("unknown");
        fclose(pciIdFile);
        return ERROR_NO_DEVICE_FOUND;
    }

    fclose(pciIdFile);

    //get device name by device ID
    pciIdFile = fopen("pci.ids", "r");

    cout << "line number: " << lino << endl; //check 

    sprintf(requestID, "%x", in_u16DeviceID);
    cout << "Device ID: " << requestID << endl; //check

    while(fgets(line, sizeof(line), pciIdFile) != NULL)
    {
        printf("lino: %d\n", lino);//check 
        printf("device_lino: %d\n", device_lino);//check 
        pDevice = line;
        device_lino++;
        if(device_lino > lino)
        {
            printf(line);
            
            sprintf(temp, "%c%c%c%c", pDevice[0],pDevice[1],pDevice[2],pDevice[3]);
            printf("temp: %s\n", temp);//check 
      
            if(strcmp(temp, requestID) == 0)
            {
                out_strDeviceName->assign(temp);
            }

            if(pDevice[0] == '#')
            {
                continue;
            }
            if(pDevice[0] != '\t')
            {
                break;
            }
        }
    }

    printf("\n");

    fclose(pciIdFile);

    //try to find the name in PCINameMapping.hpp
    s32Index = 0;
    while(pciNameMapping[s32Index].u16VendorID != 0xffff)
    {
        if(pciNameMapping[s32Index].u16VendorID == in_u16VendorID &&
            pciNameMapping[s32Index].u16DeviceID == in_u16DeviceID)
        {
            out_strVendorName->assign(pciNameMapping[s32Index].VendorBriefName);
            out_strDeviceName->assign(pciNameMapping[s32Index].DeviceBriefName);
            return NO_ERROR;
        }
        s32Index++;
    }

    return ERROR_NO_DEVICE_FOUND; 
}
*/



















