/// @file       NetAppPSU.hpp
/// @brief      NetAppPSU class
/// @details    Provide low layer functional operations for NetApp PSU
/// @author     KenHu
/// @copyright  Copyright 2015-2016 Hon Hai Technology Group. All Rights Reserved.

#ifndef NETAPP_PSU_HPP
#define NETAPP_PSU_HPP

#include "PMBusPSU.hpp"

#define BYTES_TO_LINEAR_DATA(bytes)     ((UINT16)((bytes)[0]) | (UINT16)((bytes)[1]) << 8)

#define PMBUS_CMD_MFR_SPECIFIC_PSU_PART_NUMBER          0xee
#define PMBUS_CMD_MFR_SPECIFIC_FW_REV                   0xd5
#define PMBUS_CMD_MFR_SPECIFIC_LED_CONTROL              0xf1
#define PMBUS_CMD_MFR_SPECIFIC_FAN_SPEED                0xf8
#define PMBUS_CMD_MFR_PROGRAMMING_STATUS                0xf2
#define PMBUS_CMD_MFR_FW_DOWNLOAD_DATA_ADDRESS_MODE     0xf5
#define PMBUS_CMD_MFR_DOWNLOAD_INFO                     0xf6

#ifndef PS_FAULT_LED_NAME
#define PS_FAULT_LED_NAME       NULL
#endif
#ifndef PS_SAA_LED_NAME
#define PS_SAA_LED_NAME         NULL
#endif
#ifndef PS_STDBY_LED_NAME
#define PS_STDBY_LED_NAME       NULL
#endif

#ifndef FAN_CRU1_SAR_LED_NAME
#define FAN_CRU1_SAR_LED_NAME   NULL
#endif
#ifndef FAN_CRU1_SAA_LED_NAME
#define FAN_CRU1_SAA_LED_NAME   NULL
#endif
#ifndef FAN_CRU2_SAR_LED_NAME
#define FAN_CRU2_SAR_LED_NAME   NULL
#endif
#ifndef FAN_CRU2_SAA_LED_NAME
#define FAN_CRU2_SAA_LED_NAME   NULL
#endif

#define BIT_MASK_PS_SAA_LED         0x2
#define BIT_MASK_FAULT_OVERRIDE     0x4
#define BIT_MASK_PS_FAULT_LED       0x8
#define BIT_MASK_STDBY_OVERRIDE     0x10
#define BIT_MASK_PS_STDBY_LED       0x20
#define BIT_MASK_FAN_CRU1_SAR_LED   0x100
#define BIT_MASK_FAN_CRU1_SAA_LED   0x200
#define BIT_MASK_FAN_CRU2_SAR_LED   0x400
#define BIT_MASK_FAN_CRU2_SAA_LED   0x800

/// @class      NetAppPSU NetAppPSU.hpp "NetAppPSU.hpp"
/// @brief      Provide low layer functional operations for NetApp PSU
class NetAppPSU : public PMBusPSU
{
    public:
                            NetAppPSU(string in_strConfig);
        virtual             ~NetAppPSU(void);

        ErrorMsg*           getFWVersion(string *out_strpValue);
        ErrorMsg*           getLEDList(list<string> *out_pList);
        ErrorMsg*           getLEDStatus(string in_strDeviceName,
                                         string *out_strpStatus);
        ErrorMsg*           getPartNumber(string *out_strpValue);
        ErrorMsg*           getSerialNumber(string *out_strpValue);
        ErrorMsg*           getSupportFanSpeed(string  in_strDeviceName,
                                               list<string> *out_pList);
        ErrorMsg*           getVendor(string *out_strpValue);

        ErrorMsg*           program(UINT8   in_u8Sector,
                                    CHAR    *in_chpFwBuffer,
                                    INT32   in_s32FwSize);

        ErrorMsg*           setLEDStatus(string   in_strDeviceName,
                                         string   in_strStatus);
    protected:
        
    private:
        struct ProgrammingStatus
        {
            UINT8 PROG_MODE    : 1;     // 1:Programming mode, 0:Normal mode
            UINT8 READ_MODE    : 1;     // 1:allow the reading of a flash region
            UINT8 FLASH_ERASE  : 1;     // 1:to erase memory. read value of 1 indicate erase in progress
            UINT8 SECTOR       : 3;     // 000 writeing FW, 001 reading secondary code, 010 reading primary code
            UINT8 RESERVED     : 1;     // WBY-P091.2+
            UINT8 PROG_ERROR   : 1;     // READ ONLY. 1:Package received is invalid, 0:Package received is valid
        };
};
#endif

