#ifndef RTC_DDAPI_H
#define RTC_DDAPI_H

#include "BaseClassDDAPI.hpp"

#define RTC_DEV_NAME                        "/dev/rtc0"
/*
#define ALTACC_DEV_NAME                     "/dev/altacc_drv"
#define ALTACC_MAGIC_NUMBER                 '&'
// IOCTL
#define SWITCH_ALT_IOCTL                    0x77
#define IOCTL_LOCK_ALT_ACC_COMMAND_CODE     _IOR(ALTACC_MAGIC_NUMBER, SWITCH_ALT_IOCTL, UINT8)
*/

#define IO_PERMON                           1
#define IO_PERMOFF                          0

#define RTC_1_INDEX_IOPORT					0x70
#define RTC_1_TARGET_IOPORT					0x71
#define RTC_2_INDEX_IOPORT					0x72
#define RTC_2_TARGET_IOPORT					0x73
#define RTC_3_INDEX_IOPORT					0x74
#define RTC_3_TARGET_IOPORT					0x75

#define RTC_SECONDS_INDEX					0x00
#define RTC_SECONDS_ALARM_INDEX				0x01
#define RTC_MINUTES_INDEX					0x02
#define RTC_MINUTES_ALARM_INDEX				0x03
#define RTC_HOURS_INDEX					    0x04
#define RTC_HOURS_ALARM_INDEX				0x05
#define RTC_DAY_OF_WEEK_INDEX				0x06
#define RTC_DAY_OF_MONTH_INDEX				0x07
#define RTC_MONTH_INDEX					    0x08
#define RTC_YEAR_INDEX					    0x09
#define RTC_REG_A_INDEX					    0x0A
#define RTC_REG_B_INDEX					    0x0B
#define RTC_REG_C_INDEX					    0x0C
#define RTC_REG_D_INDEX					    0x0D

#define RTC_REG_A_UIP_BIT					0x80
#define RTC_REG_A_FRQ_BIT					0x70
#define RTC_REG_A_RS_BIT					0x0F
#define RTC_REG_B_SET_BIT					0x80
#define RTC_REG_B_PIE_BIT                   0x40
#define RTC_REG_B_AIE_BIT					0x20
#define RTC_REG_B_DM_BIT					0x04
#define RTC_REG_B_HF_BIT					0x02
#define RTC_REG_B_INTR_BIT					0x70
#define RTC_REG_C_AF_BIT					0x20
#define RTC_REG_C_PF_BIT                    0x40
#define RTC_REG_C_IRQF_BIT					0x80


#define GPT_8254_SYSTEM_COUNTER_IOPORT		0x40
#define GPT_8254_CONTROL_COUNTER_IOPORT		0x43

#define TIEMER_ACCURACY_TEST_TIME           31250   // Set 31.25 ms for RTC periodic select 
#define TIEMER_ACCURACY_TEST_MIDDLE_COUNT   20
#define TIEMER_ACCURACY_TEST_LIMIT          560

#define GPT_RATE                            838     //(Unit: ns)

typedef enum {
    
    GPT_MODE0 = 0,
    GPT_MODE1,
    GPT_MODE2,
    GPT_MODE3,
    GPT_MODE4,
    GPT_MODE5,
    
} GPT_TIMER_MODE;

class RTCDDAPI : public BaseClassDDAPI
{
public:
                        RTCDDAPI(STRING className);
                        ~RTCDDAPI(void);

         ErrorMsg*      showDateTime(void);
         ErrorMsg*      setDate(int year, int month, int date);
         ErrorMsg*      setTime(int hour, int minute, int second);
         //RTC
         ErrorMsg*      setRTCTimerRate(FLOAT timerRate);
         ErrorMsg*      startRTCTimer(void);
         void           stopRTCTimer(void);
         ErrorMsg*      readRTCTimerStatus(BOOL *timerStatus);
         //8254
         void           startGPTTimer(UINT32 timerCount, GPT_TIMER_MODE selectMode);
         void           stopGPTTimer(void);
         void           readGPTTimerStatus(BOOL *timerStatus);
         void           convertGPTTimeToCount(FLOAT usTime, UINT32* timerCount);
         void           readGPTTimer(UINT32* currentValue);

		 ErrorMsg*      rtcWDTTest(void);
protected:

private:
         void           setGPTTimerControlWord(GPT_TIMER_MODE selectMode);
         void           setGPTReadbackCommand(void);
         
         INT32          m_rtcDevHandle;
         STRING         m_strDay[7];
         CHAR           m_tmpString[100];
         GPT_TIMER_MODE m_selectMode;
         UINT32         m_rtcTimerRate;
};

#endif

