#ifndef SATADDAPI_HPP
#define SATADDAPI_HPP

//#include <unistd.h>
//#include <sys/ioctl.h>
//#include <sys/types.h>
#include <iostream>
#include <sstream>
#include <string>

#include <linux/fs.h>
#include <dirent.h>
#include <sys/stat.h>
#include <scsi/sg.h>

#include "BaseClassDDAPI.hpp"
#include "PCIDDAPI.hpp"
#include "SATADDAPIStruct.hpp"
#include "AHCIArchitecture.hpp"
#include "SATADDAPIATASMART.hpp"
#include "SATADDAPISgio.hpp"

using namespace std;

#define SATA_SECTOR_SIZE 512 //in bytes
#define HEXDUMP_COLS     32  //in bytes

/// @emun       IOCTL operations
///             List all supported IOCTL operations
typedef enum
{
     eGenerateSATAList = 0,
     eListController,
     eListPort,
     eListDrive,
     eFlushBufferCache,
     eSATIdentifyCommandTest,
     eReadPerformanceTest,
     eWritePerformanceTest,
     eReadWriteTest,
} SATAIOCTLOperation;

/// @emun       data elements
///             List all supported SATA data elements
typedef enum
{
     eControllerDataAll = 0,
     eControllerID,
     eControllerBDF,
     eControllerPortAmount,

     eDriveDataAll,
     eDriveDeviceNodeName,
     eDriveSGDeviceNodeName,
     eDriveSerialNumber,
     eDriveModelName,
     eDriveFirmwareVersion,
     eDriveCapacity,
     eDrivePortNumber,
     eDriveLinkStatus,
     eDriveLinkSpeed,
     eDriveTemperature,
} SATADataElement;

/// @emun       statistic elements
///             List all supported SATA statistics
typedef enum
{
     eClearErr = 0,
     eAllErr,

     eTransportStateTransitionErr,
     eLinkSequenceErr,
     eHandshakeErr,
     eCRCErr,
     eDisparityErr,
     e10bTo8bDecodeErr,
     ePHYInternalErr,
     eInternalErr,
     eProtocolErr,
     ePersistentCommunicationOrDataIntegrityErr,
     eTransientDataIntegrityErr,
     eRecoverCommunicationErr,
     eRecoverDataIntegrityErr,
} SATAErrorElement;

/// @class      SATADDAPI SATADDAPI.hpp "SATADDAPI.hpp"
/// @brief      Provide low layer diagnostic and utility APIs for SATA module
/// @param      *m_pPCIDDAPI is a pointer to access PCI DDAPI
class SATADDAPI: public BaseClassDDAPI
{
public:
     SATADDAPI(STRING className);
     virtual ~SATADDAPI(void);

     list <SATAControllerStruct>  m_SATAControllerList;

     ErrorMsg *IOCTL(UINT32 in_u32LogicalID, SATAIOCTLOperation in_eOperation);
     ErrorMsg *generateSATAList();
     ErrorMsg *listController();
     ErrorMsg *listPort();
     ErrorMsg *listDrive();

     ErrorMsg *flushBufferCache(int in_s32FileDescriptor);

     ErrorMsg *identifyCommandTest(UINT32 in_u32LogicalID);
     ErrorMsg *readCapacity16(UINT32 in_u32LogicalID, UINT32* out_u32pCapacityMB);

     ErrorMsg *getLogicalIdByName(string in_strWellKnownName, UINT32 *out_u32LogicalID);
     ErrorMsg *getDataByID(UINT32 in_u32LogicalID, SATADataElement in_eSATADataElement, string *out_strpReturnData);
     ErrorMsg *getErrorRegisterByID(UINT32 in_u32LogicalID, SATAErrorElement in_eSATAErrorElement, string *out_strpReturnData);
     ErrorMsg *getSMARTInfoByID(UINT32 in_u32LogicalID, UINT32 in_u32StartIDNumber, UINT32 in_u32EndIDNumber, string *out_strpReturnData);
     ErrorMsg *getDataByName(string in_strWellKnownName, SATADataElement in_eSATADataElement, string *out_strpReturnData);

     ErrorMsg *PREAD(INT32 fileDescriptor, UINT32* buffer, UINT32  blockSize, UINT32 logicalSector);
     ErrorMsg *PWRITE(INT32 fileDescriptor, UINT32* buffer, UINT32  blockSize, UINT32 logicalSector);
     
protected:

private:
     PCIDDAPI                    *m_pPCIDDAPI;
     UCHAR                        m_uchaCDBBlock[64];
     UINT32                       m_u32CDBLength;

     ErrorMsg *initControllerInfo(list <SATAControllerStruct> *inout_pSATAControllerList);
     ERRORCODE checkDeviceNodeName(string in_strDeviceNodeName);
     
     ERRORCODE initPortInfo(list <SATAControllerStruct> *inout_pSATAControllerList);
     ERRORCODE initStatisticInfo(list <SATAControllerStruct> *inout_pSATAControllerList);
     ERRORCODE getPortDeviceNode(list <SATAControllerStruct> *inout_pSATAControllerList);

     ERRORCODE SCSICDBGenerate(UINT32 in_u32BlockSize, UINT32 in_u32LogicalSector, UINT32 in_u32OperationSwitch);
     ERRORCODE SCSICDBSend(INT32 in_s32FileDescriptor, UINT32 blockSize, UINT32* inOutBuffer, BOOL zeroReadOneWrite);

     ErrorMsg *SATIdentify16(UINT32 in_u32LogicalID, string* out_strpFirmwareVersion, string* out_strpModelName, string* out_strpSerialNumber);
     ERRORCODE dumpHexDataToString(string *out_strpDataBuffer, UINT32 *in_u32pDataBuffer, UINT32 in_u32DataLengthInBytes, string in_strDumpStyle, BOOL in_blsDumpAddressHead);
     string convertStringToUpperCase(string in_strInputString);
     UINT32 SWAB4(UINT32 in_s32Value);
};
#endif

